/*
	$Id: dict.c,v 1.1 2004/07/22 08:24:27 alex Exp alex $
	dict---
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: dict.c,v 1.1 2004/07/22 08:24:27 alex Exp alex $";
#endif

#include	<stdio.h>
#include	<math.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<ctype.h>
#include	<string.h>
#include	<signal.h>
#include	"edb/edb.h"
#include	"dict.h"

static	BiTree	bt_dict_ja2pr = NULL;

struct JA2PR {
	ucs4_t	*ja;
	ucs4_t	*pr;
	int		level;
};
typedef struct JA2PR *Ja2Pr;

static
int
ja2pr_compare_for_search(BiTree_obj_t o1, BiTree_obj_t o2)
{
	register ucs4_t	*ja = o1;
	register Ja2Pr	j2p = o2;

	return ucs4_strcasecmp(ja, j2p->ja);
}

static
int
ja2pr_compare(BiTree_obj_t o1, BiTree_obj_t o2)
{
	register Ja2Pr	j2p1 = o1;
	register Ja2Pr	j2p2 = o2;

	return ucs4_strcasecmp(j2p1->ja, j2p2->ja);
}

int
dict_register(ucs4_t *ja, ucs4_t *pr, int level)
{
	register Ja2Pr	j2p;

	ja = UCS4_STRDUP(ja);
	ucs4_str_onephrase(ja);
	pr = UCS4_STRDUP(pr);
	ucs4_str_onephrase(pr);

	j2p = bitree_search(bt_dict_ja2pr, ja, bitree_hint_ucs4strcase(ja), ja2pr_compare_for_search);
	if(j2p==NULL) {
		j2p = (Ja2Pr)malloc(sizeof(struct JA2PR));
		bzero(j2p, sizeof(struct JA2PR));
		j2p->ja = ucs4_strdup(ja);
		j2p->pr = ucs4_strdup(pr);
		j2p->level = level;
		bt_dict_ja2pr = bitree_add(bt_dict_ja2pr, j2p, bitree_hint_ucs4strcase(j2p->ja), ja2pr_compare, NULL, NULL);
	} else if(j2p->level>=level) {
		ucs4_t	*p1 = UCS4_STRDUP(j2p->pr);
		ucs4_t	*p2 = UCS4_STRDUP(pr);

		pronounce_convert_order(p1);
		pronounce_convert_order(p2);
		if(ucs4_strcmp(p1, p2)) {
			utf8_t	*w, *p1, *p2;

			w = UCS4toUTF8(j2p->ja);
			p1 = UCS4toUTF8(j2p->pr);
			p2 = UCS4toUTF8(pr);
			fprintf(stderr, "dict_register : %s : (%s)!=(%s)\n", w, p1, p2);
		}
	}

	return TRUE;
}

static
ucs4_t *
ja_to_pr(ucs4_t *word, int start, int len)
{
	register Ja2Pr	j2p;
	register ucs4_t	*ja, *pr = NULL;

	ja = UCS4_STRDUP(&word[start]);
	ja[len] = UCS4_NULL;
	j2p = bitree_search(bt_dict_ja2pr, ja, bitree_hint_ucs4strcase(ja), ja2pr_compare_for_search);
	if(j2p) {
		pr = ucs4_strdup(j2p->pr);
	} else if(ucs4_str_iskana(ja)) {
		pr = ucs4_strdup(ja);
	}
	return pr;
}


static
ucs4_t *
ja_to_pr_compound(ucs4_t *word, int start, int len)
{
	register Ja2Pr	j2p;
	register ucs4_t	*ja, *pr = NULL;

	ja = UCS4_STRDUP(&word[start]);
	ja[len] = UCS4_NULL;
	j2p = bitree_search(bt_dict_ja2pr, ja, bitree_hint_ucs4strcase(ja), ja2pr_compare_for_search);
	if(j2p) {
		pr = ucs4_strdup(j2p->pr);
	} else if(ucs4_str_iskana(ja)) {
		pr = ucs4_strdup(ja);
	} else if(len>1) {
		ucs4_t	*p1, *p2;
		int	i;

		for(i=len-1; i>=1; --i) {
			ucs4_t	*p;

			if((ucs4_isalpha(word[i-1])||ucs4_isdigit(word[i-1])) && (ucs4_isalpha(word[i])||ucs4_isdigit(word[i]))) continue;
			p1 = ja_to_pr(word, 0, i);
			if(p1==NULL) continue;
			p2 = ja_to_pr_compound(&word[i], 0, len-i);
			if(p2==NULL) {
				FREE(p1);
				continue;
			}
			p = ucs4_strmerge(p1, p2);
			if(pr) {
				ucs4_t *ss1 = UCS4_STRDUP(pr);
				ucs4_t *ss2 = UCS4_STRDUP(p);

				pronounce_convert_order(ss1);
				pronounce_convert_order(ss2);
				if(ucs4_strcmp(ss1, ss2)) {
					utf8_t	*s1 = UCS4toUTF8(word);
					utf8_t	*s2 = UCS4toUTF8(pr);
					utf8_t	*s3 = UCS4toUTF8(p);

					fprintf(stderr, "ja_to_pr : %s : (%s)!=(%s)\n", s1, s2, s3);
				}
				FREE(p);
			} else {
				pr = p;
			}
			FREE(p1);
			FREE(p2);
		}
	}
	return pr;
}

static
ucs4_t *
dict_japanese_to_pronounce_sub(ucs4_t *ja)
{
	ucs4_t	*w, *word;
	ucs4_t	*pr = NULL;
	Link		l, lwords, l_pr;

	word = UCS4_STRDUP(ja);
	ucs4_str_oneline(word);
	lwords = ucs4_str_split(word, UCS4_SPACE);
	l_pr = NULL;
	ForLink(l, lwords) {
		ucs4_t	*p;

		w = l->d;
		p = ja_to_pr_compound(w, 0, ucs4_strlen(w));
		if(p) {
			LinkAppend(l_pr, link_atom_new(p, free));
		} else {
			link_free(l_pr);
			l_pr = NULL;
			break;
		}
	}
	link_free(lwords);
	if(l_pr) {
		pr = ucs4_str_concatenate(l_pr, UCS4_NULL);
		link_free(l_pr);
	}
	return pr;
}

ucs4_t *
dict_japanese_to_pronounce(ucs4_t *ja)
{
	ucs4_t	*w, *word;
	ucs4_t	*pr = NULL;

	word = UCS4_STRDUP(ja);
	pr = dict_japanese_to_pronounce_sub(word);
	if(pr) return pr;

	word = UCS4_STRDUP(ja);
	for(w=word; *w; ++w) {
		if(ucs4_isascii(*w) && !ucs4_isalnum(*w)) {
			*w = UCS4_SPACE;
		} else if(*w==0xff0c) {	/* ， */
			*w = UCS4_SPACE;
		} else if(*w==0x223c) {	/* 〜 */
			*w = UCS4_SPACE;
		} else if(*w==0x00b7) {	/* ・ */
			*w = UCS4_SPACE;
		} else if(*w=='(') {	/* ( */
			*w = UCS4_SPACE;
		} else if(*w==')') {	/* ) */
			*w = UCS4_SPACE;
		} else if(*w=='/') {	/* / */
			*w = UCS4_SPACE;
		}
	}
	pr = dict_japanese_to_pronounce_sub(word);
	return pr;
}

