/*
	$Id: edb_erd.c,v 1.3 2004/07/22 08:24:27 alex Exp alex $
	erd--- Educator & Researcher Directory
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: edb_erd.c,v 1.3 2004/07/22 08:24:27 alex Exp alex $";
#endif

#include	<stdio.h>
#include	<math.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<ctype.h>
#include	<string.h>
#include	<signal.h>
#include	"edb/edb.h"
#include	"edb/html.h"
#include	"edb/publish.h"
#include	"edb/javascript.h"
#include	"erd.h"

#define	DEBUG
#undef	DEBUG

int	erd_preview = FALSE;

EdbCatalogue	ca_erd_omit_eid = NULL;
EdbCatalogue	ca_erd_prefer_listing_top_eid = NULL;
EdbCatalogue	ca_erd_person_title_listing_order = NULL;
EdbCatalogue	ca_erd_lecture_course_to_name = NULL;
EdbCatalogue	ca_erd_lecture_style_to_name = NULL;
EdbCatalogue	ca_erd_person_work_list = NULL;
eid_t			erd_history_academic_kind = 0;
eid_t			erd_history_work_kind = 0;

/* 名前排列 */
Link	l_person_eid_order_ja = NULL;
Link	l_person_eid_order_en = NULL;

EdbTableInfo	erd_table_organization = NULL;
EdbTableInfo	erd_table_person = NULL;
EdbTableInfo	erd_table_article = NULL;
EdbTableInfo	erd_table_patent = NULL;
EdbTableInfo	erd_table_history = NULL;
EdbTableInfo	erd_table_lecture = NULL;
EdbTableInfo	erd_table_teaching = NULL;
EdbTableInfo	erd_table_study = NULL;
EdbTableInfo	erd_table_asa = NULL;
EdbTableInfo	erd_table_sa = NULL;
EdbTableInfo	erd_table_posthistory = NULL;
EdbTableInfo	erd_table_prize = NULL;
EdbTableInfo	erd_table_keyword = NULL;

static
struct {
	EdbTableInfo	*table;
	char	*xmlname;
	int	do_prefetch;
} preload_tables[] = {
{	&erd_table_organization,	EDB_TABLE_Organization_XN,				TRUE,	},
{	&erd_table_person,		EDB_TABLE_Person_XN,					TRUE,	},
{	&erd_table_article,		EDB_TABLE_Article_XN,					FALSE,	},
{	&erd_table_patent,		EDB_TABLE_Patent_XN,					FALSE,	},
{	&erd_table_history,		"history",								FALSE,	},
{	&erd_table_lecture,		EDB_TABLE_Lecture_XN,					FALSE,	},
{	&erd_table_teaching,		"teaching",							FALSE,	},
{	&erd_table_study,		EDB_TABLE_Study_XN,					FALSE,	},
{	&erd_table_asa,		EDB_TABLE_AcademicSocietyActivity_XN,	FALSE,	},
{	&erd_table_sa,			EDB_TABLE_SocialActivity_XN,				FALSE,	},
{	&erd_table_posthistory,	"posthistory",							FALSE,	},
{	&erd_table_prize,		EDB_TABLE_Prize_XN,					FALSE,	},
{	&erd_table_keyword,		"keyword",							FALSE,	},
};

int
erd_print_tuple_postfix(EdbPrint ep, EdbTupleInfo tuple)
{
	register Doc	doc = ep->doc;

	CSV(ep) return FALSE;
	LaTeX(ep) {
		doc_printf(doc, "\\end{%s:%s}%%\n", EP_LaTeX_PREFIX, TABLE_XN(TupleTable(tuple)));
	}
	HTML(doc) {
		return TRUE;
	}
	return FALSE;
}

static
Link
get_eid_list(EdbContext ec, EdbTableInfo ti, char *constraint)
{
	char	*query;
	char	*order;
	int	n;
	Link	l_eid;

	order = edb_table_make_order(ec, ti, "t1");
	asprintf(&query,
		"SELECT %s FROM %s AS t1 WHERE %s %s %s %s %s",
		EDB_BASE_EID,
		TABLE_SN(ti),
		EDB_BASE_AVAILABLE,
		STRisVALID(constraint) ? "AND" : "",
		STRisVALID(constraint) ? constraint : "",
		STRisVALID(order) ? "ORDER BY" : "", order);
	FREE(order);
	l_eid = edb_pgsql_query_one_field(ec, query, &n);
	FREE(query);
	return l_eid;
}

static
struct ERD_PAGE	top_page_template = {
	"index.html",
	"index.html",
	"index.tex",
	"index.tex",
	"",
	"",
	TRUE,
	0,
	NULL,
};

static
struct ERD_PAGE	person_index_pages = {
	PERSON_INDEX_PAGE_HTML_ja,
	PERSON_INDEX_PAGE_HTML_en,
	PERSON_INDEX_PAGE_TeX_ja,
	PERSON_INDEX_PAGE_TeX_en,
	"氏名索引 (顔写真付)",
	"Faculty index (with portrait)",
	TRUE,
	0,
	NULL,
};

static
struct ERD_PAGE	person_index2_pages = {
	PERSON_INDEX2_PAGE_HTML_ja,
	PERSON_INDEX2_PAGE_HTML_en,
	PERSON_INDEX2_PAGE_TeX_ja,
	PERSON_INDEX2_PAGE_TeX_en,
	"氏名索引",
	"Faculty index",
	TRUE,
	0,
	NULL,
};

static
struct ERD_PAGE	person_all_pages = {
	PERSON_ALL_PAGE_HTML_ja,
	PERSON_ALL_PAGE_HTML_en,
	PERSON_ALL_PAGE_TeX_ja,
	PERSON_ALL_PAGE_TeX_en,
	"全員",
	"All Persons",
	TRUE,
	0,
	NULL,
};

static
struct ERD_PAGE	keyword_index_pages = {
	KEYWORD_INDEX_PAGE_HTML_ja,
	KEYWORD_INDEX_PAGE_HTML_en,
	KEYWORD_INDEX_PAGE_TeX_ja,
	KEYWORD_INDEX_PAGE_TeX_en,
	"キーワード索引",
	"Keyword index",
	TRUE,
	0,
	NULL,
};

static
struct ERD_PAGE	organization_index_pages = {
	ORGANIZATION_INDEX_PAGE_HTML_ja,
	ORGANIZATION_INDEX_PAGE_HTML_en,
	ORGANIZATION_INDEX_PAGE_TeX_ja,
	ORGANIZATION_INDEX_PAGE_TeX_en,
	"組織名索引",
	"Affiliation index",
	TRUE,
	0,
	NULL,
};

EdbCatalogue	ca_erd_person_page = NULL;
EdbCatalogue	ca_erd_organization_page = NULL;

EdbCatalogue
erd_catalogue_get(EdbContext ec, EdbTableInfo table, char *constraint)
{
	register EdbCatalogue	ca;
	register Link	l, l_eid;

	l_eid = get_eid_list(ec, table, constraint);
	ca = edb_catalogue_new();
	ForLink(l, l_eid) {
		register eid_t	eid = ATOI(l->d);
		register EdbTupleInfo	tuple = NULL;

		if(!EIDisVALID(eid)) continue;
		tuple = edb_tuple_get(ec, eid, 0);
		if(tuple) edb_catalogue_append(ca, TupleEID(tuple), tuple);
	}
	link_free(l_eid);
	return ca;
}

static
int
erd_make_js()
{
	char	*dir, *name, *nametmp;
	Doc	doc;
	static char	*scripts[] = {
"var lang_is_japanese = true;",
"",
"function",
"edb_ref_by_eid(eid, cls)",
"{",
"	var	doc = self.window.document;",
"",
"	doc.write('<input type=\"button\" value=\"" NAME_OF_DB "\" onclick=\"window.open(');",
"	doc.write(\"'" WEB_SERVER_URL EDB_BROWSER_URL_DEFAULT "?EID=\"+eid);",
"	doc.write(\"', '_blank');\");",
"	doc.writeln('\" class='+cls+'>');",
"	return true;",
"}",
"",
"function",
"write_if_top(s)",
"{",
"	var	doc = self.window.document;",
"",
"	if(top==self) doc.write(s);",
"	return true;",
"}",
"",
"function",
"moveto_anchor(v)",
"{",
"	var work_page = '" PERSON_WORK_HTML_en "';",
"",
"	if(lang_is_japanese) work_page = '" PERSON_WORK_HTML_ja "';",
"	if(v) {",
"		top.frames['" PERSON_WORK_FRAME "'].location.replace(work_page+'#'+v);",
"	} else {",
"		window.open(work_page, '_blank');",
"	}",
"	return true;",
"}",
"",
"function",
"work_frame_by_selection(sel)",
"{",
"",
"	if(sel.selectedIndex!=0) {",
"		moveto_anchor(sel.options[sel.selectedIndex].value);",
"		sel.selectedIndex = 0;",
"	}",
"	return true;",
"}",
"",
"function",
"move_page(sel) {",
"",
"	v = sel.options[sel.selectedIndex].value;",
"	if(v) {",
"		top.document.location.href=v;",
"	}",
"	sel.selectedIndex = 0;",
"}",
"",
NULL
};
	char	**s;

	asprintf(&dir, "%s", ERD_JS_DIR);
	makedir_hierarchy(dir);
	FREE(dir);

	asprintf(&name, "%s", ERD_JS_NAME);
	asprintf(&nametmp, "%s.tmp", name);

	doc = doc_open(DocML_PLAIN, nametmp, "w+");
	if(doc==NULL) return FALSE;

	for(s=scripts; *s; ++s) {
		doc_puts(doc, *s);
		doc_puts(doc, "\n");
	}
	doc_close(doc);

	file_update(nametmp, name);
	return TRUE;
}

static
int
erd_make_css()
{
	char	*dir, *name, *nametmp;
	Doc	doc;
	static char	*style_ja[] = {
"body {",
"	background-image: url(../" ERD_BACKGROUND_IMAGE ");",
"}",
"* {",
"	font-family: Helvetica, sans-serif;",
"}",
"body {",
"	font-size: 10pt;",
"}",
".affiliation {",
"	font-size: 12pt;",
"	font-weight: bolder;",
"}",
".affiliation2 {",
"	font-size: 10pt;",
"	font-weight: bolder;",
"}",
".titlebar {",
"	font-size: 14pt;",
"	font-weight: bolder;",
"}",
".subtitlebar {",
"	font-size: 12pt;",
"	font-weight: bolder;",
"}",
".subsubtitlebar {",
"	font-size: 11pt;",
"}",
".listing {",
"	font-size: 8pt;",
"}",
".listing2 {",
"	font-size: 6pt;",
"}",
".cap {",
"	font-size: 14pt;",
"	font-weight: bolder;",
"}",
".button1 {",
"	font-size: 10pt;",
"}",
".button2 {",
"	font-size: 8pt;",
"}",
".button3 {",
"	font-size: 6pt;",
"}",
".colophon {",
"	font-size: 8pt;",
"	font-style: italic;",
"}",
"a {",
"	behavior: url(#default#AnchorClick);",
"	text-decoration: none;",
"}",
"a:hover {",
"	behavior: url(#default#AnchorClick);",
"	text-decoration: none;",
"}",
NULL};
	static char	*style_en[] = {
"body {",
"	background-image: url(../" ERD_BACKGROUND_IMAGE ");",
"}",
"* {",
"	font-family: Helvetica, sans-serif;",
"}",
"body {",
"	font-size: 9pt;",
"}",
".affiliation {",
"	font-size: 10pt;",
"	font-weight: bolder;",
"}",
".affiliation2 {",
"	font-size: 8pt;",
"	font-weight: bolder;",
"}",
".titlebar {",
"	font-size: 12pt;",
"	font-weight: bolder;",
"}",
".subtitlebar {",
"	font-size: 10pt;",
"	font-weight: bolder;",
"}",
".subsubtitlebar {",
"	font-size: 9pt;",
"}",
".listing {",
"	font-size: 8pt;",
"}",
".listing2 {",
"	font-size: 6pt;",
"}",
".cap {",
"	font-size: 12pt;",
"	font-weight: bolder;",
"}",
".button1 {",
"	font-size: 10pt;",
"}",
".button2 {",
"	font-size: 8pt;",
"}",
".button3 {",
"	font-size: 6pt;",
"}",
".colophon {",
"	font-size: 8pt;",
"	font-style: italic;",
"}",
"a {",
"	behavior: url(#default#AnchorClick);",
"	text-decoration: none;",
"}",
"a:hover {",
"	behavior: url(#default#AnchorClick);",
"	text-decoration: none;",
"}",
NULL};
	char	**s;

	asprintf(&dir, "%s", ERD_CSS_DIR);
	makedir_hierarchy(dir);
	FREE(dir);

	asprintf(&name, "%s", ERD_CSS_NAME_ja);
	asprintf(&nametmp, "%s.tmp", name);
	doc = doc_open(DocML_PLAIN, nametmp, "w+");
	if(doc==NULL) return FALSE;
	for(s=style_ja; *s; ++s) { doc_puts(doc, *s); doc_puts(doc, "\n"); }
	doc_close(doc);
	file_update(nametmp, name);

	asprintf(&name, "%s", ERD_CSS_NAME_en);
	asprintf(&nametmp, "%s.tmp", name);
	doc = doc_open(DocML_PLAIN, nametmp, "w+");
	if(doc==NULL) return FALSE;
	for(s=style_en; *s; ++s) { doc_puts(doc, *s); doc_puts(doc, "\n"); }
	doc_close(doc);
	file_update(nametmp, name);
	return TRUE;
}


static
void
erd_person_index_page(EdbContext ec, EdbContext ec_univ, ErdPage page, int japanese)
{
	EdbPrint	ep_html = PAGE_EP_HTML(page, japanese);
	Doc		doc_html = ep_html->doc;
	EdbPrint	ep_tex = PAGE_EP_TeX(page, japanese);
	Doc		doc_tex = ep_tex->doc;
	Link		l, l_order;
	int		num;
	ucs4_t	*cap_index0;
	ucs4_t	*cap_index1;
	int		cap_idx, cap_max;
	int		i;

	edb_print_tuple_postfix_register_func(ep_html, PrintStyle(ep_html), erd_print_tuple_postfix);
	edb_print_eoi_add(ep_html, MY_ORGAN_EID);

	erd_page_begin(page, japanese);

	doc_puts(doc_html, "<table border=\"0\" width=\"100%\">\n");

	doc_puts(doc_html, "<tr valign=\"top\"><td class=affiliation>\n");
	erd_page_menu(page, japanese, FALSE);
//	doc_text_puts3(doc_html, "<a href=\"index.html\">", BILINGUAL(japanese, NAME_OF_CONTENTS_ja, NAME_OF_CONTENTS_en), "</a>");
	doc_text_puts(doc_html, " > ");
	doc_puts(doc_html, "</td></tr>\n");

	doc_puts(doc_html, "<tr valign=\"top\">\n");
	doc_puts(doc_html, "<td bgcolor=\"" INDEX_TITLE_BGC "\">");
	doc_puts(doc_html, "<table border=\"0\" width=\"100%\">\n");
	doc_puts(doc_html, "<tr class=titlebar><td>");
	doc_text_puts(doc_html, PAGE_TITLE(page, japanese));
	doc_puts(doc_html, "</td><td width=\"1\">");
	doc_printf(doc_html, "<input type=\"button\" value=\"%s\" onclick=\"top.location='%s';\" class=button1>",
		BILINGUAL(japanese, "English", "Japanese"), PAGE_NAME_HTML(page, !japanese));
	doc_puts(doc_html, "</td></tr></table>\n");
	doc_puts(doc_html, "</td></tr>\n");
	doc_puts(doc_html, "<tr><td><hr /></td></tr>\n");

	doc_puts(doc_html, "<tr><td>\n");
	if(japanese) {
		cap_index0 = UTF8toUCS4(LEXICOGRAPHICAL_ORDER_ja);
		cap_index1 = UCS4_STRDUP(cap_index0);
		ucs4_str_tokatakana(cap_index1);
	} else {
		cap_index0 = UTF8toUCS4(LEXICOGRAPHICAL_ORDER_en);
		cap_index1 = UCS4_STRDUP(cap_index0);
	}
	cap_max = ucs4_strlen(cap_index0);

	for(i=0; i<cap_max; ++i) {
		ucs4_t	u[2];

		doc_puts(doc_html, "<input type=\"button\" value=\"");
		u[0] = cap_index0[i];
		u[1] = UCS4_NULL;
		doc_text_puts(doc_html, UCS4toUTF8(u));
		doc_printf(doc_html, "\" onclick=\"top.location.replace('#%04x');\" class=button2>\n", cap_index0[i]);
	}

	doc_puts(doc_html, "</td></tr>");
	doc_puts(doc_html, "<tr><td><hr /></td></tr>\n");
	doc_puts(doc_html, "<tr><td align=\"center\">\n");

	l_order = BILINGUAL(japanese, l_person_eid_order_ja, l_person_eid_order_en);
	num = 0;
	cap_idx = -1;
	ForLink(l, l_order) {
		eid_t			person_eid = ATOI(l->d);
		ErdPage		person_page;
		PersonInfo	pi;
		ucs4_t		new_cap;

		if(!EIDisVALID(person_eid)) continue;
		if(!edb_catalogue_lookup(ca_erd_person_page, person_eid)) continue;
		if(num%NUM_OF_PORTRAITs_PER_COLUMN==0) {
			doc_puts(doc_html, (num==0) ? "<table border=\"0\">" : "</tr>");
			doc_puts(doc_html, "<tr align=\"center\" valign=\"top\">\n");
		}
		person_page = erd_person_page(ec, ec_univ, person_eid);
		pi = person_page->info;
		new_cap = japanese ? pi->order_ja[0] : pi->order_en[0] ;
		if(cap_idx<0 || cap_index1[cap_idx]<new_cap) {
			while(cap_idx+1<cap_max && cap_index1[cap_idx+1]<=new_cap) {
				ucs4_t	u[2];

				++cap_idx;
				doc_printf(doc_html, "<td width=\"%d\" bgcolor=\"%s\" align=\"left\">", PORTRAIT_ICON_WIDTH, cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
				doc_printf(doc_html, "<span class=cap><a name=\"%04x\">", cap_index0[cap_idx]);
				u[0] = cap_index0[cap_idx];
				u[1] = UCS4_NULL;
				doc_text_puts(doc_html, UCS4toUTF8(u));
				doc_text_puts3(doc_tex, "\\CAPTION{", UCS4toUTF8(u), "}%\n");
				doc_puts(doc_html, "</a></span>");
				doc_puts(doc_html, "<br />");
				doc_puts(doc_html, BILINGUAL(japanese, RtnTOP_ja, RtnTOP_en));
				doc_puts(doc_html, "</td>\n");
				++num;
				if(num%NUM_OF_PORTRAITs_PER_COLUMN==0) doc_puts(doc_html, "</tr><tr align=\"center\" valign=\"top\">\n");
			}
		}
		doc_printf(doc_html, "<td width=\"%d\" bgcolor=\"%s\">", PORTRAIT_ICON_WIDTH, cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
		erd_linkto_person_page(page, person_page, P_SMALL|P_PORTRAIT|P_TITLE|P_BOLD, japanese);
		doc_puts(doc_html, "</td>\n");
		doc_printf(doc_tex, "%%%%PERSON %d\n", person_page->eid);
		doc_printf(doc_tex, "\\PERSON{%d}%\n", person_page->eid);
		++num;
	}
	while(cap_idx+1<cap_max) {
		ucs4_t	u[2];

		++cap_idx;
		if(num%NUM_OF_PORTRAITs_PER_COLUMN==0) {
			doc_puts(doc_html, (num==0) ? "<table border=\"0\">" : "</tr>");
			doc_puts(doc_html, "<tr align=\"center\" valign=\"top\">\n");
		}
		doc_printf(doc_html, "<td width=\"%d\" bgcolor=\"%s\" align=\"left\">", PORTRAIT_ICON_WIDTH, cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
		doc_printf(doc_html, "<span class=cap><a name=\"%04x\">", cap_index0[cap_idx]);
		u[0] = cap_index0[cap_idx];
		u[1] = UCS4_NULL;
		doc_text_puts(doc_html, UCS4toUTF8(u));
		doc_text_puts3(doc_tex, "\\CAPTION{", UCS4toUTF8(u), "}%\n");
		doc_puts(doc_html, "</a></span>");
		doc_puts(doc_html, "<br />");
		doc_puts(doc_html, BILINGUAL(japanese, RtnTOP_ja, RtnTOP_en));
		doc_puts(doc_html, "</td>\n");
		++num;
		if(num%NUM_OF_PORTRAITs_PER_COLUMN==0) doc_puts(doc_html, "</tr><tr align=\"center\" valign=\"top\">\n");
	}
	if(num>0) {
		for( ; num<NUM_OF_PORTRAITs_PER_COLUMN; ++num) {
			doc_printf(doc_html, "<td width=\"%d\"><img src=\"%s\" alt=\"\" width=\"%d\" height=\"%d\" border=\"0\" /></td>\n",
				PORTRAIT_ICON_WIDTH, ICON_DUMMY_PORTRAIT, PORTRAIT_ICON_WIDTH, PORTRAIT_ICON_HEIGHT);
		}
		doc_puts(doc_html, "</tr></table>\n");
	}
	doc_puts(doc_html, "</td></tr></table>\n");
	erd_page_end(page, japanese);
}

static
void
erd_person_index2_page(EdbContext ec, EdbContext ec_univ, ErdPage page, int japanese)
{
	EdbPrint	ep = PAGE_EP_HTML(page, japanese);
	Doc		doc = ep->doc;
	Link		l, l_order;
	int		num;
	ucs4_t	*cap_index0;
	ucs4_t	*cap_index1;
	int		cap_idx, cap_max;
	int		i;
	struct	{
		int	person;
		int	title;
		int	birthdate;
		int	phone;
		int	email;
		int	academicbackground;
		int	degree;
		int	workhistory;
		int	portrait;
	} sum;

	edb_print_tuple_postfix_register_func(ep, PrintStyle(ep), erd_print_tuple_postfix);
	edb_print_eoi_add(ep, MY_ORGAN_EID);

	erd_page_begin(page, japanese);

	doc_puts(doc, "<table border=\"0\" width=\"100%\">\n");

	doc_puts(doc, "<tr valign=\"top\"><td class=affiliation>\n");
	erd_page_menu(page, japanese, FALSE);
//	doc_text_puts3(doc, "<a href=\"index.html\">", BILINGUAL(japanese, NAME_OF_CONTENTS_ja, NAME_OF_CONTENTS_en), "</a>");
	doc_text_puts(doc, " > ");
	doc_puts(doc, "</td></tr>\n");

	doc_puts(doc, "<tr valign=\"top\">\n");
	doc_puts(doc, "<td bgcolor=\"" INDEX_TITLE_BGC "\">");
	doc_puts(doc, "<table border=\"0\" width=\"100%\">\n");
	doc_puts(doc, "<tr class=titlebar><td>");
	doc_text_puts(doc, PAGE_TITLE(page, japanese));
	doc_puts(doc, "</td><td width=\"1\">");
	doc_printf(doc, "<input type=\"button\" value=\"%s\" onclick=\"top.location='%s';\" class=button1>",
		BILINGUAL(japanese, "English", "Japanese"), PAGE_NAME_HTML(page, !japanese));
	doc_puts(doc, "</td></tr></table>\n");
	doc_puts(doc, "</td></tr>\n");
	doc_puts(doc, "<tr><td><hr /></td></tr>\n");

	doc_puts(doc, "<tr><td>\n");
	if(japanese) {
		cap_index0 = UTF8toUCS4(LEXICOGRAPHICAL_ORDER_ja);
		cap_index1 = UCS4_STRDUP(cap_index0);
		ucs4_str_tokatakana(cap_index1);
	} else {
		cap_index0 = UTF8toUCS4(LEXICOGRAPHICAL_ORDER_en);
		cap_index1 = UCS4_STRDUP(cap_index0);
	}
	cap_max = ucs4_strlen(cap_index0);

	for(i=0; i<cap_max; ++i) {
		ucs4_t	u[2];

		doc_puts(doc, "<input type=\"button\" value=\"");
		u[0] = cap_index0[i];
		u[1] = UCS4_NULL;
		doc_text_puts(doc, UCS4toUTF8(u));
		doc_printf(doc, "\" onclick=\"top.location.replace('#%04x');\" class=button2>\n", cap_index0[i]);
	}

	doc_puts(doc, "</td></tr>");
	doc_puts(doc, "<tr><td><hr /></td></tr>\n");
	doc_puts(doc, "<tr><td align=\"center\">\n");

	l_order = BILINGUAL(japanese, l_person_eid_order_ja, l_person_eid_order_en);
	num = 0;
	cap_idx = -1;
	doc_puts(doc, "<table border=\"0\" class=listing>\n");
	if(erd_preview) bzero(&sum, sizeof(sum));
	ForLink(l, l_order) {
		eid_t			person_eid = ATOI(l->d);
		ErdPage		person_page;
		PersonInfo	pi;
		ucs4_t		new_cap;

		if(!EIDisVALID(person_eid)) continue;
		if(!edb_catalogue_lookup(ca_erd_person_page, person_eid)) continue;
		person_page = erd_person_page(ec, ec_univ, person_eid);
		pi = person_page->info;
		new_cap = japanese ? pi->order_ja[0] : pi->order_en[0] ;
		if(cap_idx<0 || cap_index1[cap_idx]<new_cap) {
			while(cap_idx+1<cap_max && cap_index1[cap_idx+1]<=new_cap) {
				ucs4_t	u[2];

				++cap_idx;
				doc_printf(doc, "<tr align=\"left\" valign=\"top\"><td bgcolor=\"%s\" align=\"left\">", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
				doc_printf(doc, "<span class=cap><a name=\"%04x\">", cap_index0[cap_idx]);
				u[0] = cap_index0[cap_idx];
				u[1] = UCS4_NULL;
				doc_text_puts(doc, UCS4toUTF8(u));
				doc_puts(doc, "</a></span>");
				doc_puts(doc, BILINGUAL(japanese, RtnTOP_ja, RtnTOP_en));
				doc_puts(doc, "</td>");
				if(erd_preview) {
					doc_puts(doc, "<td align=\"center\"></td>");	/* for affiliation */
					doc_puts(doc, "<td align=\"center\">職<br />名</td>");
					doc_puts(doc, "<td align=\"center\">生<br />年</td>");
					doc_puts(doc, "<td align=\"center\">電<br />話</td>");
					doc_puts(doc, "<td align=\"center\">電<br />メ</td>");
					doc_puts(doc, "<td align=\"center\">学<br />歴</td>");
					doc_puts(doc, "<td align=\"center\">学<br />位</td>");
					doc_puts(doc, "<td align=\"center\">職<br />歴</td>");
					doc_puts(doc, "<td align=\"center\">写<br />真</td>");
				}
				doc_puts(doc, "</tr>\n");
				++num;
			}
		}
		doc_printf(doc, "<tr align=\"left\" valign=\"top\" bgcolor=\"%s\"><td>", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
		erd_linkto_person_page(page, person_page, P_AFFILIATION|P_InTABLE|P_BOLD, japanese);
		doc_puts(doc, "</td>");
		if(erd_preview) {
			++sum.person;
#define	COUNT(f, v) if((f)) { ++(v); doc_puts(doc, "<td align=\"center\">○</td>"); } else {doc_puts(doc, "<td align=\"center\" bgcolor=\"red\">×</td>"); }
			COUNT(pi->title_is_set, sum.title);
			COUNT(pi->birthdate_is_set, sum.birthdate);
			COUNT(pi->phone_is_set, sum.phone);
			COUNT(pi->email_is_set, sum.email);
			COUNT(edb_catalogue_size(pi->ca_history_a)>0, sum.academicbackground);
			COUNT(pi->nod_is_set, sum.degree);
			COUNT(edb_catalogue_size(pi->ca_history_w)>0, sum.workhistory);
			COUNT(pi->portrait_is_set, sum.portrait);
		}
		doc_puts(doc, "</tr>\n");
		++num;
	}
	while(cap_idx+1<cap_max) {
		ucs4_t	u[2];

		++cap_idx;
		doc_printf(doc, "<tr align=\"left\" valign=\"top\"><td bgcolor=\"%s\" align=\"left\">", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
		doc_printf(doc, "<span class=cap><a name=\"%04x\">", cap_index0[cap_idx]);
		u[0] = cap_index0[cap_idx];
		u[1] = UCS4_NULL;
		doc_text_puts(doc, UCS4toUTF8(u));
		doc_puts(doc, "</a></span>");
		doc_puts(doc, BILINGUAL(japanese, RtnTOP_ja, RtnTOP_en));
		doc_puts(doc, "</td>");
		if(erd_preview) {
			doc_puts(doc, "<td align=\"center\"></td>");	/* for affiliation */
			doc_puts(doc, "<td align=\"center\">職<br />名</td>");
			doc_puts(doc, "<td align=\"center\">生<br />年</td>");
			doc_puts(doc, "<td align=\"center\">電<br />話</td>");
			doc_puts(doc, "<td align=\"center\">電<br />メ</td>");
			doc_puts(doc, "<td align=\"center\">学<br />歴</td>");
			doc_puts(doc, "<td align=\"center\">学<br />位</td>");
			doc_puts(doc, "<td align=\"center\">職<br />歴</td>");
			doc_puts(doc, "<td align=\"center\">写<br />真</td>");
		}
		doc_puts(doc, "</tr>\n");
		++num;
	}
	doc_puts(doc, "</table>\n");
	doc_puts(doc, "</td></tr></table>\n");

	if(erd_preview && sum.person>0) {
		int	s;

		doc_puts(doc, "<table border=\"1\" width=\"100%\">\n");
		doc_puts(doc, "<tr>");
		doc_puts(doc, "<th>Person</th>");
		doc_puts(doc, "<th>職名</th>");
		doc_puts(doc, "<th>生年</th>");
		doc_puts(doc, "<th>電話</th>");
		doc_puts(doc, "<th>電メ</th>");
		doc_puts(doc, "<th>学歴</th>");
		doc_puts(doc, "<th>学位</th>");
		doc_puts(doc, "<th>職歴</th>");
		doc_puts(doc, "<th>写真</th>");
		doc_puts(doc, "<th>ALL</th>");
		doc_puts(doc, "</tr>\n");
		doc_puts(doc, "<tr>");
		s = 0;
		doc_printf(doc, "<td align=\"right\">%d</td>", sum.person);
		doc_printf(doc, "<td align=\"right\">%d (%.0f%%)</td>", sum.title, (double)sum.title/sum.person*100.0);
		s += sum.title;
		doc_printf(doc, "<td align=\"right\">%d (%.0f%%)</td>", sum.birthdate, (double)sum.birthdate/sum.person*100.0);
		s += sum.birthdate;
		doc_printf(doc, "<td align=\"right\">%d (%.0f%%)</td>", sum.phone, (double)sum.phone/sum.person*100.0);
		s += sum.phone;
		doc_printf(doc, "<td align=\"right\">%d (%.0f%%)</td>", sum.email, (double)sum.email/sum.person*100.0);
		s += sum.email;
		doc_printf(doc, "<td align=\"right\">%d (%.0f%%)</td>", sum.academicbackground, (double)sum.academicbackground/sum.person*100.0);
		s += sum.academicbackground;
		doc_printf(doc, "<td align=\"right\">%d (%.0f%%)</td>", sum.degree, (double)sum.degree/sum.person*100.0);
		s += sum.degree;
		doc_printf(doc, "<td align=\"right\">%d (%.0f%%)</td>", sum.workhistory, (double)sum.workhistory/sum.person*100.0);
		s += sum.workhistory;
		doc_printf(doc, "<td align=\"right\">%d (%.0f%%)</td>", sum.portrait, (double)sum.portrait/sum.person*100.0);
		s += sum.portrait;
		doc_printf(doc, "<td align=\"right\">%.1f (%.0f%%)</td>", (double)s/8, (double)s/(sum.person*8)*100.0);
		doc_puts(doc, "</tr>\n");
		doc_puts(doc, "</table>\n");
	}
	erd_page_end(page, japanese);
}

static
void
erd_person_all_page(EdbContext ec, EdbContext ec_univ, ErdPage page, int japanese)
{
	EdbPrint	ep = PAGE_EP_HTML(page, japanese);
	Doc		doc = ep->doc;
	Link		l, l_order;
	int		num;
	ucs4_t	*cap_index0;
	ucs4_t	*cap_index1;
	int		cap_idx, cap_max;
	int		i;

	edb_print_tuple_postfix_register_func(ep, PrintStyle(ep), erd_print_tuple_postfix);
	edb_print_eoi_add(ep, MY_ORGAN_EID);

	erd_page_begin(page, japanese);

	l_order = BILINGUAL(japanese, l_person_eid_order_ja, l_person_eid_order_en);
	ForLink(l, l_order) {
		eid_t			person_eid = ATOI(l->d);
		ErdPage		person_page;
		PersonInfo	pi;
		ucs4_t		new_cap;

		if(!EIDisVALID(person_eid)) continue;
		if(!edb_catalogue_lookup(ca_erd_person_page, person_eid)) continue;
		person_page = erd_person_page(ec, ec_univ, person_eid);
		pi = person_page->info;
		erd_person_page_for_all(page, person_page, japanese);
	}

	erd_page_end(page, japanese);
}

static
int
erd_initialize(EdbContext ec)
{
	int	i;

	for(i=0; i<Ns(preload_tables); ++i) {
		*preload_tables[i].table = edb_table_get_by_name(ec, preload_tables[i].xmlname);
		if(*preload_tables[i].table==NULL) return FALSE;
		if(preload_tables[i].do_prefetch) edb_table_prefetch_tuples(ec, *preload_tables[i].table);
	}

	ca_erd_person_page = edb_catalogue_new();
	ca_erd_organization_page = edb_catalogue_new();

	ca_erd_omit_eid = edb_catalogue_new();
	ca_erd_prefer_listing_top_eid = edb_catalogue_new();
	ca_erd_person_title_listing_order = edb_catalogue_new();
	ca_erd_lecture_course_to_name = edb_catalogue_new();
	ca_erd_lecture_style_to_name = edb_catalogue_new();
	ca_erd_person_work_list = edb_catalogue_new();

	edb_setdatabasespeaking(ec, "Japanese");
	l_person_eid_order_ja = get_eid_list(ec, erd_table_person, NULL);

	edb_setdatabasespeaking(ec, "English");
	l_person_eid_order_en = get_eid_list(ec, erd_table_person, NULL);

	edb_setdatabasespeaking(ec, "Japanese");
	return TRUE;
}

static
int
edb_make_erd(EdbContext ec, EdbContext ec_univ, char *www_path, FILE *fp)
{
	Doc	doc;
	ErdPage	top_page;
	ErdPage	pidx_page, pidx2_page, kidx_page, oidx_page, pall_page;
	EdbPrint	ep;
	Link	l, l_eid;

	makedir_hierarchy(www_path);
	if(chdir(www_path)<0) {
		fprintf(stderr, "edb_erd: cannot change directory (%s)\n", www_path);
		return FALSE;
	}

	erd_make_js();
	erd_make_css();

	edb_caption_omit_append(ec, MY_ORGAN_EID);

	top_page = erd_page_new(ec, ec_univ, &top_page_template, NULL);
	erd_page_open(top_page);
	ep = PAGE_EP_HTML(top_page, TRUE);
	doc = ep->doc;
	asprintf(&top_page->title_ja, MY_ORGAN_NAME_ja " " NAME_OF_CONTENTS_ja " (%d)", ERD_THIS_YEAR(ec));
	asprintf(&top_page->title_en, MY_ORGAN_NAME_en ", " NAME_OF_CONTENTS_en " (%d)", ERD_THIS_YEAR(ec));

	erd_page_begin(top_page, TRUE);
	doc_text_puts3(doc, "<h1>", PAGE_TITLE(top_page, TRUE), "</h1>\n");

	doc_puts(doc, "<hr />\n");
	doc_puts(doc, "<h2>目次</h2>\n");
	doc_puts(doc, "<ul>\n");

	erd_organization_page(ec, ec_univ, MY_ORGAN_EID, 0, TRUE);
	erd_organization_page(ec, ec_univ, MY_ORGAN_EID, 0, FALSE);

	/* shrink person_eid_list */
	l_eid = NULL;
	ForLink(l, l_person_eid_order_ja) {
		if(edb_catalogue_lookup(ca_erd_person_page, ATOI(l->d))) {
			l_eid = link_append(link_atom_new(strdup(l->d), free), l_eid);
		}
	}
	link_free(l_person_eid_order_ja);
	l_person_eid_order_ja = link_reverse(l_eid);

	l_eid = NULL;
	ForLink(l, l_person_eid_order_en) {
		if(edb_catalogue_lookup(ca_erd_person_page, ATOI(l->d))) {
			l_eid = link_append(link_atom_new(strdup(l->d), free), l_eid);
		}
	}
	link_free(l_person_eid_order_en);
	l_person_eid_order_en = link_reverse(l_eid);

	pidx_page = erd_page_new(ec, ec_univ, &person_index_pages, NULL);
	erd_page_open(pidx_page);
	erd_person_index_page(ec, ec_univ, pidx_page, TRUE);
	erd_person_index_page(ec, ec_univ, pidx_page, FALSE);
	erd_page_close(pidx_page);

	pidx2_page = erd_page_new(ec, ec_univ, &person_index2_pages, NULL);
	erd_page_open(pidx2_page);
	erd_person_index2_page(ec, ec_univ, pidx2_page, TRUE);
	erd_person_index2_page(ec, ec_univ, pidx2_page, FALSE);
	erd_page_close(pidx2_page);

	pall_page = erd_page_new(ec, ec_univ, &person_all_pages, NULL);
	erd_page_open(pall_page);
	if(MAKE_PERSON_ALL_PAGE && erd_preview) {
		erd_person_all_page(ec, ec_univ, pall_page, TRUE);
		erd_person_all_page(ec, ec_univ, pall_page, FALSE);
	}
	erd_page_close(pall_page);

	kidx_page = erd_page_new(ec, ec_univ, &keyword_index_pages, NULL);
	erd_page_open(kidx_page);
	erd_keyword_index_page(ec, kidx_page, TRUE);
	erd_keyword_index_page(ec, kidx_page, FALSE);
	erd_page_close(kidx_page);

	oidx_page = erd_page_new(ec, ec_univ, &organization_index_pages, NULL);
	erd_page_open(oidx_page);
	erd_organization_index_page(ec, oidx_page, TRUE);
	erd_organization_index_page(ec, oidx_page, FALSE);
	erd_page_close(oidx_page);

	doc_printf(doc, "<li><a href=\"%s/%d/%s\"><b>所属別教職員一覧</b></a>\n",
		TABLE_XN(erd_table_organization),
		MY_ORGAN_EID,
		ORGANIZATION_PAGE_HTML_ja);

	doc_printf(doc, "<li><a href=\"%s\"><b>%s</b></a>\n", PAGE_NAME_HTML(pidx2_page, TRUE), PAGE_TITLE(pidx2_page, TRUE));
	doc_printf(doc, "<li><a href=\"%s\"><b>%s</b></a>\n", PAGE_NAME_HTML(pidx_page, TRUE), PAGE_TITLE(pidx_page, TRUE));
	doc_printf(doc, "<li><a href=\"%s\"><b>%s</b></a>\n", PAGE_NAME_HTML(kidx_page, TRUE), PAGE_TITLE(kidx_page, TRUE));
	doc_printf(doc, "<li><a href=\"%s\"><b>%s</b></a>\n", PAGE_NAME_HTML(oidx_page, TRUE), PAGE_TITLE(oidx_page, TRUE));

	doc_puts(doc, "</ul>\n");

	erd_page_footer(top_page, TRUE);
	doc_puts(doc, "<hr />\n");

	doc_text_puts3(doc, "<h3>", PAGE_TITLE(top_page, FALSE), "</h3>\n");
	doc_puts(doc, "<dl><dt><b>English oriented version.</b></dt><dd>\n");
	doc_puts(doc, "This means that the pages are described in English as possible.\n");
	doc_puts(doc, "Since these pages still contain many Asian characters,\n");
	doc_puts(doc, "we recommend you to look these pages with browser which is able to handle Asian character set.\n");
	doc_puts(doc, "</dd></dl>\n");
	doc_puts(doc, "<h3>Table of Contents</h3>\n");
	doc_puts(doc, "<ul>\n");

	doc_printf(doc, "<li><a href=\"%s/%d/%s\"><b>Faculty list by affiliation.</b></a>\n",
		TABLE_XN(erd_table_organization),
		MY_ORGAN_EID,
		ORGANIZATION_PAGE_HTML_en);

	doc_printf(doc, "<li><a href=\"%s\"><b>%s</b></a>\n", PAGE_NAME_HTML(pidx2_page, FALSE), PAGE_TITLE(pidx2_page, FALSE));
	doc_printf(doc, "<li><a href=\"%s\"><b>%s</b></a>\n", PAGE_NAME_HTML(pidx_page, FALSE), PAGE_TITLE(pidx_page, FALSE));
	doc_printf(doc, "<li><a href=\"%s\"><b>%s</b></a>\n", PAGE_NAME_HTML(kidx_page, FALSE), PAGE_TITLE(kidx_page, FALSE));
	doc_printf(doc, "<li><a href=\"%s\"><b>%s</b></a>\n", PAGE_NAME_HTML(oidx_page, FALSE), PAGE_TITLE(oidx_page, FALSE));

	doc_puts(doc, "</ul>\n");

	erd_page_end(top_page, FALSE);
	erd_page_close(top_page);

	return TRUE;
}

static
void
erd_configure(EdbContext ec, char *name)
{
	FILE	*fp;
	char	*line;
	int	lineno = 0;

	fp = fopen(name, "r");
	if(fp==NULL) {
		fprintf(stderr, "edb_erd : can't open configuration file '%s'.\n", name);
		exit(1);
	}
	while((line=file_getline(fp))!=NULL) {
		char	*p, *s, *cmd;
		Link	lstr;
		eid_t	eid;
		int	err = FALSE;

		++lineno;
		if(line==NULL) continue;
		s = index(line, '#');
		if(s) *s = '\0';
		str_oneline(line);
		if(!STRisVALID(line)) {
			FREE(line);
			continue;
		}
		lstr = str_split(line, ' ');
		if(lstr==NULL) {
			FREE(line);
			continue;
		}
		if(link_atoms(lstr)<=0) {
			link_free(lstr);
			FREE(line);
			continue;
		}
		cmd = lstr->d;
		if(!strcmp(cmd, "omit_eid")) {
			if(lstr->n && (eid=ATOI(lstr->n->d)) && EIDisVALID(eid)) {
				edb_catalogue_append(ca_erd_omit_eid, eid, "");
			} else err = TRUE;
		} else if(!strcmp(cmd, "prefer_listing_top_eid")) {
			if(lstr->n && (eid=ATOI(lstr->n->d)) && EIDisVALID(eid)) {
				edb_catalogue_append(ca_erd_prefer_listing_top_eid, eid, "");
			} else err = TRUE;
		} else if(!strcmp(cmd, "person_title_listing_order")) {
			if(lstr->n && (eid=ATOI(lstr->n->d)) && EIDisVALID(eid)) {
				edb_catalogue_append(ca_erd_person_title_listing_order, eid, strdup(lstr->n->d));
			} else err = TRUE;
		} else if(!strcmp(cmd, "history_academic_kind")) {
			if(lstr->n && (eid=ATOI(lstr->n->d)) && EIDisVALID(eid)) {
				erd_history_academic_kind = eid;
			} else err = TRUE;
		} else if(!strcmp(cmd, "history_work_kind")) {
			if(lstr->n && (eid=ATOI(lstr->n->d)) && EIDisVALID(eid)) {
				erd_history_work_kind = eid;
			} else err = TRUE;
		} else if(!strcmp(cmd, "person_work_list")) {
			if(link_atoms(lstr)>5 && (eid=ATOI(lstr->n->n->d)) && EIDisVALID(eid)) {
				Link	lstr2;

				s = str_concatenate(lstr->n->n->n->n, ' ');
				lstr2 = str_split(s, '"');
				FREE(s);
				if(link_atoms(lstr2)>=4) {
					struct PERSON_WORK_LIST *wl;

					wl = (struct PERSON_WORK_LIST *)malloc(sizeof(struct PERSON_WORK_LIST));
					bzero(wl, sizeof(struct PERSON_WORK_LIST));
					s = link_get_obj(lstr, 1);
					wl->tablename = strdup(s);
					wl->kind = eid;
					s = link_get_obj(lstr, 3);
					wl->anchor = strdup(s);
					s = link_get_obj(lstr2, 1);
					str_oneline(s);
					wl->caption_ja = strdup(s);
					s = link_get_obj(lstr2, 3);
					str_oneline(s);
					wl->caption_en = strdup(s);
					edb_catalogue_append(ca_erd_person_work_list, eid, wl);
				} else err = TRUE;
				link_free(lstr2);
			} else err = TRUE;
		} else if(!strcmp(cmd, "lecture_course_to_name")) {
			if(lstr->n && (eid=ATOI(lstr->n->d)) && EIDisVALID(eid)) {
				Link	lstr2;

				s = str_concatenate(lstr->n->n, ' ');
				lstr2 = str_split(s, '"');
				FREE(s);
				if(link_atoms(lstr2)>=4) {
					struct LECTURE_COURSE_to_NAME *lc2n;

					lc2n = (struct LECTURE_COURSE_to_NAME *)malloc(sizeof(struct LECTURE_COURSE_to_NAME));
					bzero(lc2n, sizeof(struct LECTURE_COURSE_to_NAME));
					lc2n->eid = eid;
					s = link_get_obj(lstr2, 1);
					str_oneline(s);
					lc2n->name_ja = strdup(s);
					s = link_get_obj(lstr2, 3);
					str_oneline(s);
					lc2n->name_en = strdup(s);
					if((s = link_get_obj(lstr2, 4))!=NULL) {
						lc2n->order = ATOI(s);
					} else {
						lc2n->order = 100000;
					}
					edb_catalogue_append(ca_erd_lecture_course_to_name, eid, lc2n);
				} else err = TRUE;
				link_free(lstr2);
			} else err = TRUE;
		} else if(!strcmp(cmd, "lecture_style_to_name")) {
			if(lstr->n && (eid=ATOI(lstr->n->d)) && EIDisVALID(eid)) {
				Link	lstr2;

				s = str_concatenate(lstr->n->n, ' ');
				lstr2 = str_split(s, '"');
				FREE(s);
				if(link_atoms(lstr2)>=4) {
					struct LECTURE_STYLE_to_NAME *lc2n;

					lc2n = (struct LECTURE_STYLE_to_NAME *)malloc(sizeof(struct LECTURE_STYLE_to_NAME));
					bzero(lc2n, sizeof(struct LECTURE_STYLE_to_NAME));
					lc2n->eid = eid;
					s = link_get_obj(lstr2, 1);
					str_oneline(s);
					lc2n->name_ja = strdup(s);
					s = link_get_obj(lstr2, 3);
					str_oneline(s);
					lc2n->name_en = strdup(s);
					if((s = link_get_obj(lstr2, 4))!=NULL) {
						lc2n->order = ATOI(s);
					} else {
						lc2n->order = 100000;
					}
					edb_catalogue_append(ca_erd_lecture_style_to_name, eid, lc2n);
				} else err = TRUE;
				link_free(lstr2);
			} else err = TRUE;
		} else if(!strcmp(cmd, "pronounce")) {
			Link	lstr2;

			s = str_concatenate(lstr->n, ' ');
			lstr2 = str_split(s, '"');
			FREE(s);
			if(link_atoms(lstr2)>=4) {
				ucs4_t	*ja, *pr;

				s = link_get_obj(lstr2, 1);
				str_oneline(s);
				ja = UTF8toUCS4(s);
				s = link_get_obj(lstr2, 3);
				str_oneline(s);
				pr = UTF8toUCS4(s);
				if(UCS4_STRisVALID(ja) && UCS4_STRisVALID(pr)) dict_register(ja, pr, DICT_CONF);
			} else err = TRUE;
			link_free(lstr2);
		} else if(!strcmp(cmd, "end_input")) {
			link_free(lstr);
			FREE(line);
			break;
		} else err = TRUE;
		if(err) fprintf(stderr, "edb_erd : configure : %d : unknown command '%s'.\n", lineno, line);
		link_free(lstr);
		FREE(line);
	}
	fclose(fp);
}

/*#main--- main function */
int
main(int ac, char *av[])
{
	char	*dbname = NULL;
	char	*conffile = NULL;
	int	i;
	int	status;
	int	today = 0;
	EdbContext	ec, ec_univ;
	char	*www_path = WWW_PATH;
	struct tm	tm;

	for(i=1; i<ac; ++i) {
		if(av[i][0]=='-') {
			if(!strcmp(av[i], "-D") && i+1<ac) {
				dbname = strdup(av[i+1]);
				++i;
			} else if(!strcmp(av[i], "-C") && i+1<ac) {
				conffile = strdup(av[i+1]);
				++i;
			} else if(!strcmp(av[i], "-preview")) {
				erd_preview = TRUE;
				printf("Making Preview Version.\n");
			} else if(!strcmp(av[i], "-today") && i+1<ac) {
				today = atoi(av[i+1]);
				++i;
				printf("Making %08d Version.\n", today);
			} else if(!strcmp(av[i], "-help")) {
				fprintf(stderr, "usage: edb_erd [-D <db>] [-C <conf>] [-preview] [-today YYYYMMDD] [dir]\n");
				return TRUE;
			}
		} else {
			asprintf(&www_path, "%s", av[i]);
		}
	}

	ec = edb_configure(PUBLIC_DBCONF);
	ec->university = FALSE;
	if(dbname) edb_dbpath_set(ec, dbname, TRUE);

	if(!edb_open(ec)) {
		fprintf(stderr, "%s: can't open database\n", av[0]);
		exit(1);
	}
	edb_trace_set(ec, stdout);

	edb_setdatabasespeaking(ec, "Japanese");
	ec->operator = 0;
	ec->staff = FALSE;
	edb_begin(ec, FALSE);

	ec_univ = edb_configure(UNIVERSITY_DBCONF);
	ec_univ->university = TRUE;
	if(dbname) edb_dbpath_set(ec_univ, dbname, TRUE);

	if(!edb_open(ec_univ)) {
		fprintf(stderr, "%s: can't open database\n", av[0]);
		exit(1);
	}
	edb_trace_set(ec_univ, stdout);

	edb_setdatabasespeaking(ec_univ, "Japanese");
	ec_univ->operator = 0;
	ec_univ->staff = FALSE;
	edb_begin(ec_univ, FALSE);

#if defined(ERD_PRESENT_DATE)
	ec->present_date = ERD_PRESENT_DATE;
#else
	if(today>0) {
		ec->present_date = today;
	} else {
		localtime_r(&ec->now, &tm);
		ec->present_date = YMD_DATE(tm.tm_year + 1900, tm.tm_mon+1, tm.tm_mday);
		ec->present_wday = tm.tm_wday;
		ec->present_time = HMS_TIME(tm.tm_hour, tm.tm_min, tm.tm_sec);
	}
#endif

#if defined(ERD_PRESENT_DATE)
	ec_univ->present_date = ERD_PRESENT_DATE;
#else
	if(today>0) {
		ec_univ->present_date = today;
	} else {
		localtime_r(&ec_univ->now, &tm);
		ec_univ->present_date = YMD_DATE(tm.tm_year + 1900, tm.tm_mon+1, tm.tm_mday);
		ec_univ->present_wday = tm.tm_wday;
		ec_univ->present_time = HMS_TIME(tm.tm_hour, tm.tm_min, tm.tm_sec);
	}
#endif

	if(!erd_initialize(ec)) {
		edb_end(ec);
		exit(1);
	}
	if(STRisVALID(conffile)) erd_configure(ec, conffile);
	keyword_initialize(ec);
	status = edb_make_erd(ec, ec_univ, www_path, stdout);
	edb_end(ec_univ);
	edb_end(ec);

	edb_close(ec_univ);
	edb_close(ec);
	exit(0);
}


