/*
	$Id: keyword.c,v 1.3 2004/07/22 08:24:27 alex Exp alex $
	keyword---
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: keyword.c,v 1.3 2004/07/22 08:24:27 alex Exp alex $";
#endif

#include	<stdio.h>
#include	<math.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<ctype.h>
#include	<string.h>
#include	<signal.h>
#include	"edb/edb.h"
#include	"edb/html.h"
#include	"edb/publish.h"
#include	"edb/javascript.h"
#include	"erd.h"

struct KEYWORD_INFO {
	ucs4_t		*word;
	ucs4_t		*pr;
	ucs4_t		*order;
	int			pr_synth;
	EdbCatalogue	ca_page;
};
typedef	struct KEYWORD_INFO	*KeywordInfo;

static	BiTree	bt_keyword_ja = NULL;
static	BiTree	bt_keyword_en = NULL;
static	Doc	test_doc = NULL;
static	Link	l_omit_article = NULL;

void
keyword_initialize(EdbContext ec)
{
	ucs4_t	*u;
	EdbCatalogue	ca_keyword;
	Link	l;

	test_doc = doc_sopen(DocML_PLAIN);
	u = UTF8toUCS4("a ");
	LinkAppend(l_omit_article, link_atom_new(ucs4_strdup(u), free));
	u = UTF8toUCS4("an ");
	LinkAppend(l_omit_article, link_atom_new(ucs4_strdup(u), free));
	u = UTF8toUCS4("the ");
	LinkAppend(l_omit_article, link_atom_new(ucs4_strdup(u), free));

	ca_keyword = erd_catalogue_get(ec, erd_table_keyword, NULL);
	ForCatalogueLink(l, ca_keyword) {
		EdbTupleInfo	tuple = CatalogueObject(l);
		EdbTupleColumn	tc;
		EdbDatum	datum;

		tc = edb_tuple_seek_column(ec, tuple, "@.word");
		if(tc==NULL) continue;
		ForTCDatum(datum, tc) {
			ucs4_t	*w, *p;

			if(datum==NULL) continue;
			if(!DatumJapaneseIsValid(datum)) continue;
			if(!DatumPronounceIsValid(datum)) continue;
			w = UTF8toUCS4(DatumJapanese(datum));
			p = UTF8toUCS4(DatumPronounce(datum));
			dict_register(w, p, DICT_KEYWORD);
		}
	}
	edb_catalogue_free(ca_keyword);
}

static
KeywordInfo
keyword_info_new(EdbContext ec)
{
	KeywordInfo	ki;

	ki = (KeywordInfo)malloc(sizeof(struct KEYWORD_INFO));
	bzero(ki, sizeof(struct KEYWORD_INFO));
	ki->ca_page = edb_catalogue_new();
	return ki;
}

static
void
keyword_info_free(EdbContext ec, KeywordInfo ki)
{

	if(ki==NULL) return;
	FREE(ki->word);
	FREE(ki->pr);
	FREE(ki->order);
	edb_catalogue_free(ki->ca_page);
	free(ki);
}

static
int
keyword_info_compare(BiTree_obj_t o1, BiTree_obj_t o2)
{
	KeywordInfo	ki1 = o1;
	KeywordInfo	ki2 = o2;

	return ucs4_strcasecmp(ki1->word, ki2->word);
}

void
erd_keyword_regist(EdbContext ec, EdbDatum datum, ErdPage page)
{
	utf8_t		*ja, *en, *pr;
	ucs4_t		*u, *ustr;
	Link	l;

	en = (utf8_t *)(DatumEnglishIsValid(datum) ? STRDUP((char *)DatumEnglish(datum)) : STRDUP("") );
	ja = (utf8_t *)(DatumJapaneseIsValid(datum) ? STRDUP((char *)DatumJapanese(datum)) : STRDUP("") );
	pr = (utf8_t *)(DatumPronounceIsValid(datum) ? STRDUP((char *)DatumPronounce(datum)) : STRDUP("") );

	if(ja && en && !strcasecmp((char *)ja, (char *)en)) ja = NULL;
	if(!STRisVALID(en) && STRisVALID(ja) && str_isascii((char *)ja)) {
		en = ja;
		ja = NULL;
	}
	/* japanese */
	if(STRisVALID(ja)) {
		KeywordInfo	ki, rki;

		ki = keyword_info_new(ec);
		ki->word = UTF8toUCS4(ja);
		ki->word = ucs4_strdup(ki->word);
		ki->pr = UTF8toUCS4(pr);
		ki->pr = ucs4_strdup(ki->pr);
		if(STRisVALID(pr)) {
			ustr = UTF8toUCS4(pr);
		} else {
			doc_sflush(test_doc);
			doc_text_puts(test_doc, ja);
			u = doc_ucs4_sgets(test_doc);
			if(UCS4_STRisVALID(u) && ucs4_str_iskana(u)) {
				FREE(ki->pr);
				ki->pr = ucs4_strdup(u);
				ustr = UCS4_STRDUP(u);
			} else {
				ustr = UTF8toUCS4("");
			}
			FREE(u);
		}
		if(UCS4_STRisVALID(ustr)) dict_register(ki->word, ustr, DICT_USER);
		ucs4_str_onephrase(ustr);
		pronounce_convert_order(ustr);
		ki->order = ucs4_strdup(ustr);

		rki = bitree_search(bt_keyword_ja, ki, bitree_hint_ucs4strcase(ki->word), keyword_info_compare);
		if(rki==NULL) {
			bt_keyword_ja = bitree_add(bt_keyword_ja, ki, bitree_hint_ucs4strcase(ki->word), keyword_info_compare, NULL, NULL);
		} else {
			if(ucs4_strcmp(rki->word, ki->word)>0) {
				FREE(rki->word);
				rki->word = ki->word;
				ki->word = NULL;
			}
			if(!UCS4_STRisVALID(rki->pr) && UCS4_STRisVALID(ki->pr)) {
				FREE(rki->pr);
				rki->pr = ki->pr;
				ki->pr = NULL;
				FREE(rki->order);
				rki->order = ki->order;
				ki->order = NULL;
			}
			if(UCS4_STRisVALID(rki->order) && UCS4_STRisVALID(ki->order)
				&& ucs4_str_iskana(rki->order) && ucs4_str_iskana(ki->order)
				&& ucs4_strcmp(rki->order, ki->order)) {
				char	*w, *p1, *p2;

				w = UCS4toUTF8(rki->word);
				p1 = UCS4toUTF8(rki->order);
				p2 = UCS4toUTF8(ki->order);
				edb_trace(ec, "%s: (%s)!=(%s)\n", w, p1, p2);
			}
			keyword_info_free(ec, ki);
			ki = rki;
		}
		if(page) edb_catalogue_append(ki->ca_page, page->eid, page);
	}

	/* english */
	if(STRisVALID(en)) {
		KeywordInfo	ki, rki;

		ki = keyword_info_new(ec);
		ki->word = UTF8toUCS4(en);
		ki->word = ucs4_strdup(ki->word);
		ki->pr = NULL;
		doc_sflush(test_doc);
		doc_text_puts(test_doc, en);
		ustr = doc_ucs4_sgets(test_doc);
		for(u=ustr; *u; ++u) if(!ucs4_isalnum(*u)) *u = UCS4_SPACE;
		ucs4_str_oneline(ustr);
		ucs4_str_capitalize1(ustr);
		ForLink(l, l_omit_article) {
			int	len;

			u = l->d;
			if(!ucs4_strncasecmp(ustr, u, len=ucs4_strlen(u))) {
				ki->order = ucs4_strdup(&ustr[len]);
				break;
			}
		}
		if(l==NULL) ki->order = ucs4_strdup(ustr);
		FREE(ustr);

		rki = bitree_search(bt_keyword_en, ki, bitree_hint_ucs4strcase(ki->word), keyword_info_compare);
		if(rki==NULL) {
			bt_keyword_en = bitree_add(bt_keyword_en, ki, bitree_hint_ucs4strcase(ki->word), keyword_info_compare, NULL, NULL);
		} else {
			if(ucs4_strcmp(rki->word, ki->word)>0) {
				FREE(rki->word);
				rki->word = ki->word;
				ki->word = NULL;
				FREE(rki->order);
				rki->order = ki->order;
				ki->order = NULL;
			}
			keyword_info_free(ec, ki);
			ki = rki;
		}
		if(page) edb_catalogue_append(ki->ca_page, page->eid, page);
	}
}

static
int
bitree_to_link_func(BiTree_obj_t obj, void *data)
{
	Link	*link = data;

	*link = link_append(link_atom_new(obj, NULL), *link);
	return TRUE;
}

static
Link
bitree_to_link(BiTree bt)
{
	Link	link = NULL;

	bitree_callback(bt, bitree_to_link_func, &link);
	link = link_reverse(link);
	return link;
}

static
int
keyword_sort_compare_ja(LinkObj o1, LinkObj o2)
{
	KeywordInfo	ki1 = (KeywordInfo)o1->d;
	KeywordInfo	ki2 = (KeywordInfo)o2->d;
	int	st;

	st = ucs4_strcmp(ki1->order, ki2->order);
	if(st) return st;
	return ucs4_strcmp(ki1->word, ki2->word);
}

static
int
keyword_sort_compare_en(LinkObj o1, LinkObj o2)
{
	KeywordInfo	ki1 = (KeywordInfo)o1->d;
	KeywordInfo	ki2 = (KeywordInfo)o2->d;

	return ucs4_strcmp(ki1->order, ki2->order);
}

static
void
keyword_index_js(Doc doc)
{

	JavaScript_begin(doc);
	doc_puts(doc, "var	kindex_bgc = '#d0ffd0';\n");
	doc_puts(doc, "var	kindex = new Array;\n");
	doc_puts(doc, "var	ki;\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "function\n");
	doc_puts(doc, "kindex_show(doc)\n");
	doc_puts(doc, "{\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "	if(doc) {\n");
	doc_puts(doc, "		show_kindex(doc, this);\n");
	doc_puts(doc, "	} else {\n");
	doc_puts(doc, "		show_kindex(self.document, this);\n");
	doc_puts(doc, "	}\n");
	doc_puts(doc, "	return true;\n");
	doc_puts(doc, "}\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "function\n");
	doc_puts(doc, "PI(e, n)\n");
	doc_puts(doc, "{\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "	this.e = e;\n");
	doc_puts(doc, "	this.n = n;\n");
	doc_puts(doc, "}\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "function\n");
	doc_puts(doc, "kindex_add(e, n)\n");
	doc_puts(doc, "{\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "	this.person[this.person.length] = new PI(e, n);\n");
	doc_puts(doc, "	return true;\n");
	doc_puts(doc, "}\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "function\n");
	doc_puts(doc, "KI(w, p, e, n)\n");
	doc_puts(doc, "{\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "	this.w = w;\n");
	doc_puts(doc, "	this.p = p;\n");
	doc_puts(doc, "	this.person = new Array;\n");
	doc_puts(doc, "	this.show = kindex_show;\n");
	doc_puts(doc, "	this.add = kindex_add;\n");
	doc_puts(doc, "}\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "function\n");
	doc_puts(doc, "show_kindex(doc, ki)\n");
	doc_puts(doc, "{\n");
	doc_puts(doc, "	var	i;\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "	doc.write('<tr align=\"left\" valign=\"top\" bgcolor=\"'+kindex_bgc+'\"><td>'+ki.w+'</td><td><span class=listing2>'+ki.p+'</span></td><td>');\n");
	doc_puts(doc, "	for(i=0; i<ki.person.length; ++i) {\n");
	doc_puts(doc, "		doc.write('<a href=\"person/'+ki.person[i].e+'/profile-ja.html\" target=\"_blank\">'+ki.person[i].n+'</a>');\n");
	doc_puts(doc, "		if(i+1<ki.person.length) doc.write(' &nbsp;/&nbsp; ');\n");
	doc_puts(doc, "	}\n");
	doc_puts(doc, "	doc.writeln('</td></tr>');\n");
	doc_puts(doc, "	return true;\n");
	doc_puts(doc, "}\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "function\n");
	doc_puts(doc, "kindex_find(tb)\n");
	doc_puts(doc, "{\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "	if(!tb.value) {\n");
	doc_puts(doc, "		window.alert('Text box is empty.');\n");
	doc_puts(doc, "	} else {\n");
	doc_puts(doc, "		var	win = window.open('', 'result','width=768,height=256,resizable=yes,scrollbars=yes');\n");
	doc_puts(doc, "		var	doc = win.document;\n");
	doc_puts(doc, "		var	i, num=0;\n");
	doc_puts(doc, "		var	re = new RegExp(tb.value, \"i\");\n");
	doc_puts(doc, "\n");
	doc_puts(doc, "		doc.writeln('<?xml version=\"1.0\" encoding=\"utf-8\" ?>');\n");
	doc_puts(doc, "		doc.writeln('<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">');\n");
	doc_puts(doc, "		doc.writeln('<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"ja\" lang=\"ja\">\n');");
	doc_puts(doc, "		doc.writeln('	<head>');\n");
	doc_puts(doc, "		doc.writeln('		<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />');\n");
	doc_puts(doc, "		doc.writeln('		<meta http-equiv=\"Content-Style-Type\" content=\"text/css; charset=utf-8\" />');\n");
	doc_puts(doc, "		doc.writeln('		<title>Result</title>');\n");
	doc_puts(doc, "		doc.writeln('		<link rel=\"stylesheet\" type=\"text/css\" href=\"css/erd-ja.css\" title=\"erd\" />');\n");
	doc_puts(doc, "		doc.writeln('	</head>');\n");
	doc_puts(doc, "		doc.writeln('<body text=\"black\" bgcolor=\"white\" style=\"margin:0px 5%\">');\n");
	doc_puts(doc, "		doc.writeln('<h2>Result for Search ( '+tb.value+' ) <input type=\"button\" value=\"Dismiss\" onclick=\"self.close(self);\"></h2>');\n");
	doc_puts(doc, "		doc.writeln('<hr />');\n");
	doc_puts(doc, "		doc.writeln('<table border=\"0\" width=\"100%\" align=\"center\" class=listing>');\n");
	doc_puts(doc, "		for(i=0; i<kindex.length; ++i) {\n");
	doc_puts(doc, "			if(kindex[i].w.search(re)>=0) {\n");
	doc_puts(doc, "				kindex[i].show(doc);\n");
	doc_puts(doc, "				++num;\n");
	doc_puts(doc, "			}\n");
	doc_puts(doc, "		}\n");
	doc_puts(doc, "		if(num==0) doc.writeln('<tr><td>見つかりませんでした．</td></tr>');\n");
	doc_puts(doc, "		doc.writeln('</table>');\n");
	doc_puts(doc, "		doc.writeln('<hr />');\n");
	doc_puts(doc, "		doc.writeln('<i>'+num+' keywords were found.</i>');\n");
	doc_puts(doc, "		doc.writeln('</body>');\n");
	doc_puts(doc, "		doc.writeln('</html>');\n");
	doc_puts(doc, "		doc.close();\n");
	doc_puts(doc, "		win.focus();\n");
	doc_puts(doc, "	}\n");
	doc_puts(doc, "	return false;\n");
	doc_puts(doc, "}\n");
	doc_puts(doc, "\n");
	JavaScript_end(doc);
}

void
erd_keyword_index_page(EdbContext ec, ErdPage page, int japanese)
{
	EdbPrint	ep_html = PAGE_EP_HTML(page, japanese);
	Doc		doc_html = ep_html->doc;
	EdbPrint	ep_tex = PAGE_EP_TeX(page, japanese);
	Doc		doc_tex = ep_tex->doc;
	Link		l, l_keyword;
	int		num;
	ucs4_t	*cap_index0;
	ucs4_t	*cap_index1;
	int		cap_idx, cap_max;
	int		i;

	edb_print_eoi_add(ep_html, MY_ORGAN_EID);

	erd_page_begin(page, japanese);

	keyword_index_js(doc_html);

	doc_puts(doc_html, "<table border=\"0\" width=\"100%\">\n");

	doc_puts(doc_html, "<tr valign=\"top\"><td class=affiliation>\n");
	erd_page_menu(page, japanese, FALSE);
//	doc_text_puts3(doc_html, "<a href=\"index.html\">", BILINGUAL(japanese, NAME_OF_CONTENTS_ja, NAME_OF_CONTENTS_en), "</a>");
	doc_text_puts(doc_html, " > ");
	doc_puts(doc_html, "</td></tr>\n");

	doc_puts(doc_html, "<tr valign=\"top\">\n");
	doc_puts(doc_html, "<td bgcolor=\"" INDEX_TITLE_BGC "\">");
	doc_puts(doc_html, "<table border=\"0\" width=\"100%\">\n");
	doc_puts(doc_html, "<tr class=titlebar><td>");
	doc_text_puts(doc_html, PAGE_TITLE(page, japanese));
	doc_puts(doc_html, "</td><td width=\"1\" class=button1>");
	doc_printf(doc_html, "<input type=\"button\" value=\"%s\" onclick=\"top.location='%s';\">",
		BILINGUAL(japanese, "English", "Japanese"), PAGE_NAME_HTML(page, !japanese));
	doc_puts(doc_html, "</td></tr></table>\n");
	doc_puts(doc_html, "</td></tr>\n");
	doc_puts(doc_html, "<tr><td><hr /></td></tr>\n");

#if 1
	doc_puts(doc_html, "<form onsubmit=\"return kindex_find(this.elements['searchword']);\">");
#endif
	doc_puts(doc_html, "<tr><td>\n");
	if(japanese) {
		cap_index0 = UTF8toUCS4(LEXICOGRAPHICAL_ORDER_ja);
		cap_index1 = UCS4_STRDUP(cap_index0);
		ucs4_str_tokatakana(cap_index1);
	} else {
		cap_index0 = UTF8toUCS4(LEXICOGRAPHICAL_ORDER_en);
		cap_index1 = UCS4_STRDUP(cap_index0);
	}
	cap_max = ucs4_strlen(cap_index0);

	for(i=0; i<cap_max; ++i) {
		ucs4_t	u[2];

		doc_puts(doc_html, "<input type=\"button\" value=\"");
		u[0] = cap_index0[i];
		u[1] = UCS4_NULL;
		doc_text_puts(doc_html, UCS4toUTF8(u));
		doc_printf(doc_html, "\" onclick=\"top.location.replace('#%04x');\" class=button1>\n", cap_index0[i]);
	}

#if 1
	doc_puts(doc_html, "&nbsp;&nbsp;&nbsp;&nbsp;\n");
	doc_puts(doc_html, "<nobr>");
	doc_puts(doc_html, "<input name=\"searchword\" type=\"text\" size=\"20\">");
	doc_puts(doc_html, "<input type=\"submit\" value=\"Search\">");
	doc_puts(doc_html, "</nobr>\n");
#endif

	doc_puts(doc_html, "</td></tr>");
#if 1
	doc_puts(doc_html, "</form>");
#endif
	doc_puts(doc_html, "<tr><td><hr /></td></tr>\n");
	doc_puts(doc_html, "<tr><td align=\"center\">\n");

	l_keyword = bitree_to_link(BILINGUAL(japanese, bt_keyword_ja, bt_keyword_en));
	if(japanese) {
		ForLink(l, l_keyword) {
			KeywordInfo	ki = l->d;
			ucs4_t	*word, *pr;
			char		*ja = UCS4toUTF8(ki->word);

			if(UCS4_STRisVALID(ki->pr)) continue;
			doc_sflush(test_doc);
			doc_text_puts(test_doc, ja);
			word = doc_ucs4_sgets(test_doc);
			pr = dict_japanese_to_pronounce(word);
			if(pr) {
				pronounce_convert_order(pr);
				FREE(ki->order);
				ki->order = pr;
				ki->pr_synth = TRUE;
			}
			FREE(word);
		}
	}
	link_sort(l_keyword, BILINGUAL(japanese, keyword_sort_compare_ja, keyword_sort_compare_en), NULL, NULL, NULL);

	num = 0;
	cap_idx = -1;
	doc_puts(doc_html, "<table border=\"0\" class=listing>\n");
#if 1
	JavaScript_begin(doc_html);
	ForLink(l, l_keyword) {
		KeywordInfo	ki = l->d;
		ucs4_t		new_cap;
		char			*txt;

		new_cap = ki->order[0];
		if(!erd_preview && new_cap!=UCS4_NULL && !ucs4_isascii(new_cap) && !ucs4_iskatakana(new_cap)) continue;
		if(cap_idx<0 || cap_index1[cap_idx]<new_cap) {
			while(cap_idx+1<cap_max && cap_index1[cap_idx+1]<=new_cap) {
				ucs4_t	u[2];

				JavaScript_end(doc_html);
				++cap_idx;
				doc_printf(doc_html, "<tr align=\"left\" valign=\"top\"><td bgcolor=\"%s\" align=\"left\">", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
				doc_printf(doc_html, "<span class=cap><a name=\"%04x\">", cap_index0[cap_idx]);
				u[0] = cap_index0[cap_idx];
				u[1] = UCS4_NULL;
				doc_text_puts(doc_html, UCS4toUTF8(u));
				doc_text_puts3(doc_tex, "\\CAPTION{", UCS4toUTF8(u), "}%\n");
				doc_puts(doc_html, "</a></span>");
				doc_puts(doc_html, BILINGUAL(japanese, RtnTOP_ja, RtnTOP_en));
				doc_puts(doc_html, "</td></tr>\n");
				++num;
				JavaScript_begin(doc_html);
				doc_printf(doc_html, "kindex_bgc = '%s';\n", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
			}
		}
		txt = UCS4toUTF8(ki->word);
		doc_text_puts3(doc_tex, "\\KEYWORDINDEX{", txt, "}{");
		if(STRisVALID(txt) && edb_catalogue_size(ki->ca_page)>0) {
			Link			lp;

			doc_printf(doc_html, "ki = kindex[kindex.length] = new KI('");
			doc_puts(doc_html, txt);
			doc_puts(doc_html, "','");
			if(erd_preview && japanese) {
				char	*s = UCS4toUTF8(ki->order);

				if(ki->pr_synth) doc_puts(doc_html, "<span style=\"color : #800000\">");
				doc_text_puts(doc_html, s);
				if(ki->pr_synth) doc_puts(doc_html, "</span>");
			}
			doc_puts(doc_html, "'); ");
			ForLink(lp, BILINGUAL(japanese, l_person_eid_order_ja, l_person_eid_order_en)) {
				ErdPage		info_page;

				info_page = edb_catalogue_lookup(ki->ca_page, ATOI(lp->d));
				if(info_page == NULL) continue;
				doc_puts(doc_html, "ki.add(");
				doc_printf(doc_html, "%d", info_page->eid);
				doc_text_puts3(doc_html, ",'", PAGE_TITLE(info_page, japanese), "'); ");
				doc_printf(doc_tex, "\\REFERENCE{%d}", info_page->eid);
			}
			doc_puts(doc_html, "ki.show();\n");
		}
		doc_puts(doc_tex, "}%\n");
		++num;
	}
	JavaScript_end(doc_html);
#else
	ForLink(l, l_keyword) {
		KeywordInfo	ki = l->d;
		ucs4_t		new_cap;
		char			*txt;

		new_cap = ki->order[0];
		if(!erd_preview && new_cap!=UCS4_NULL && !ucs4_isascii(new_cap) && !ucs4_iskatakana(new_cap)) continue;
		if(cap_idx<0 || cap_index1[cap_idx]<new_cap) {
			while(cap_idx+1<cap_max && cap_index1[cap_idx+1]<=new_cap) {
				ucs4_t	u[2];

				++cap_idx;
				doc_printf(doc_html, "<tr align=\"left\" valign=\"top\"><td bgcolor=\"%s\" align=\"left\">", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
				doc_printf(doc_html, "<span class=cap><a name=\"%04x\">", cap_index0[cap_idx]);
				u[0] = cap_index0[cap_idx];
				u[1] = UCS4_NULL;
				doc_text_puts(doc_html, UCS4toUTF8(u));
				doc_text_puts3(doc_tex, "\\CAPTION{", UCS4toUTF8(u), "}%\n");
				doc_puts(doc_html, "</a></span>");
				doc_puts(doc_html, BILINGUAL(japanese, RtnTOP_ja, RtnTOP_en));
				doc_puts(doc_html, "</td></tr>\n");
				++num;
			}
		}
		txt = UCS4toUTF8(ki->word);
		if(STRisVALID(txt) && edb_catalogue_size(ki->ca_page)>0) {
			Link			lp;

			doc_printf(doc_html, "<tr align=\"left\" valign=\"top\" bgcolor=\"%s\"><td>", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
			doc_text_printf(doc_html, "%s", txt);
			doc_puts(doc_html, "</td><td>");
			if(erd_preview && japanese) {
				char	*s = UCS4toUTF8(ki->order);

				doc_puts(doc_html, "<span class=listing2>");
				if(ki->pr_synth) doc_puts(doc_html, "<span style=\"color : #800000\">");
				doc_text_printf(doc_html, "%s", s);
				if(ki->pr_synth) doc_puts(doc_html, "</span>");
				doc_puts(doc_html, "</span>");
				doc_puts(doc_html, "</td><td>");
			}
			if(edb_catalogue_size(ki->ca_page)==1) {
				ForCatalogueLink(lp, ki->ca_page) erd_linkto_person_page(page, CatalogueObject(lp), 0, japanese);
			} else {
				int	n = 0;

				ForLink(lp, BILINGUAL(japanese, l_person_eid_order_ja, l_person_eid_order_en)) {
					ErdPage		info_page;

					info_page = edb_catalogue_lookup(ki->ca_page, ATOI(lp->d));
					if(info_page == NULL) continue;
					if(n>0) doc_puts(doc_html, " &nbsp;/&nbsp; ");
					erd_linkto_person_page(page, info_page, 0, japanese);
					++n;
				}
			}
			doc_puts(doc_html, "</td></tr>\n");
		}
		++num;
	}
#endif
	while(cap_idx+1<cap_max) {
		ucs4_t	u[2];

		++cap_idx;
		doc_printf(doc_html, "<tr align=\"left\" valign=\"top\"><td bgcolor=\"%s\" align=\"left\">", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
		doc_printf(doc_html, "<span class=cap><a name=\"%04x\">", cap_index0[cap_idx]);
		u[0] = cap_index0[cap_idx];
		u[1] = UCS4_NULL;
		doc_text_puts(doc_html, UCS4toUTF8(u));
		doc_text_puts3(doc_tex, "\\CAPTION{", UCS4toUTF8(u), "}%\n");
		doc_puts(doc_html, "</a></span>");
		doc_puts(doc_html, BILINGUAL(japanese, RtnTOP_ja, RtnTOP_en));
		doc_puts(doc_html, "</td></tr>\n");
		++num;
	}
	doc_puts(doc_html, "</table>\n");
	doc_puts(doc_html, "</td></tr></table>\n");
	link_free(l_keyword);

	erd_page_end(page, japanese);
}

