/*
	$Id: organization.c,v 1.2 2004/07/22 08:24:27 alex Exp alex $
	organization---
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: organization.c,v 1.2 2004/07/22 08:24:27 alex Exp alex $";
#endif

#include	<stdio.h>
#include	<math.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<ctype.h>
#include	<string.h>
#include	<signal.h>
#include	"edb/edb.h"
#include	"edb/html.h"
#include	"edb/publish.h"
#include	"edb/javascript.h"
#include	"erd.h"

struct ORGANIZATION_INFO {
	EdbTupleInfo	tuple;
	char			*name_ja;
	char			*name_en;
	char			*name_en_idx;
	char			*name_pr;
	ucs4_t		*order_ja;
	ucs4_t		*order_en;
	EdbCatalogue	ca_person_info;
};
typedef	struct ORGANIZATION_INFO	*OrganizationInfo;

static
struct ERD_PAGE	organization_pages_template = {
	ORGANIZATION_PAGE_HTML_ja,
	ORGANIZATION_PAGE_HTML_en,
	ORGANIZATION_PAGE_TeX_ja,
	ORGANIZATION_PAGE_TeX_en,
	"所属別教職員",
	"List of person classified by affiliation",
	TRUE,
	2,
	NULL,
};

static
OrganizationInfo
organization_info_new(EdbContext ec)
{
	OrganizationInfo	oi;

	oi = (OrganizationInfo)malloc(sizeof(struct ORGANIZATION_INFO));
	bzero(oi, sizeof(struct ORGANIZATION_INFO));
	oi->ca_person_info = edb_catalogue_new();
	return oi;
}

static
void
organization_info_free(EdbContext ec, OrganizationInfo oi)
{

	if(oi==NULL) return;
	edb_catalogue_free(oi->ca_person_info);
	free(oi);
}


eid_t
erd_organization_attribute(EdbContext ec, EdbTupleInfo organization)
{
	register EdbTupleColumn	tc;
	register EdbDatum	d;

	tc = edb_tuple_seek_column(ec, organization, "@.attribute");
	if(tc && (d=tc->datum)) return DatumEID(d);
	return 0;
}

static	int	erd_organization_page_sub		__P((EdbContext ec, ErdPage page, EdbTupleInfo organ, int layer, int layer2, int));

static
int
erd_organization_page_sub_child(EdbContext ec, ErdPage page, EdbTupleInfo organ, int layer, int layer2, int japanese)
{
	EdbPrint	ep_html = PAGE_EP_HTML(page, japanese);
	Doc	doc_html = ep_html->doc;
	EdbPrint	ep_tex = PAGE_EP_TeX(page, japanese);
	Doc	doc_tex = ep_tex->doc;
	EdbTupleColumn	tc;

	tc = edb_tuple_seek_column(ec, organ, "@.member");
	if(tc && tc->datum) {
		EdbDatum	d;
		int	num = 0;

		ForTCDatum(d, tc) {
			EdbTupleInfo		child;
			int				i;
			eid_t				attribute;
			EdbCatalogue		ca_child;
			ErdPage			opage;
			OrganizationInfo	oi;
			int				number_of_persons;
			Link				l;

			if(!DatumEIDisValid(d)) continue;
			if(!edb_datum_at_present(ec, d)) continue;
			child = edb_tuple_get(ec, DatumEID(d), 0);
			if(!TupleIsOrganization(child)) continue;
			if(edb_catalogue_lookup(ca_erd_omit_eid, TupleEID(child))) continue;

			if(TupleMapto(child)) continue;
			attribute = erd_organization_attribute(ec, child);
			if(edb_catalogue_lookup(ca_erd_omit_eid, attribute)) continue;

			opage = erd_organization_page(ec, page->ec_univ, TupleEID(child), layer+1, japanese);
			oi = opage->info;
			ForCatalogueLink(l, oi->ca_person_info) {
				PersonInfo	pi = CatalogueObject(l);

				edb_catalogue_append(((OrganizationInfo)page->info)->ca_person_info, TupleEID(pi->tuple), pi);
			}
			number_of_persons = edb_catalogue_size(oi->ca_person_info);

			if(!erd_preview && number_of_persons==0) continue;

			if(layer>0 && layer2==0) doc_puts(doc_html, "<tr><td><hr /></td></tr>\n");
			if(layer2==0) {
				doc_puts(doc_html, "<tr bgcolor=\"" ORGANIZATION_NAME_BGC2 "\"><td>");
				doc_puts(doc_html, "<table border=\"0\" width=\"100%\"><tr align=\"left\">");
				doc_puts(doc_html, "<td class=subtitlebar>");
			} else if(layer2==1) {
				doc_puts(doc_html, "<tr class=subsubtitlebar><td>");
			} else {
				doc_puts(doc_html, "<tr><td>");
			}
			if(number_of_persons>0) doc_printf(doc_html, "<a href=\"../../%s\">", PAGE_PATH_HTML(opage, japanese));
			doc_text_puts(doc_html, PAGE_TITLE(opage, japanese));
			if(number_of_persons>0) doc_puts(doc_html, "</a>");
			if(number_of_persons>0) {
				doc_puts(doc_html, "&nbsp;&nbsp;&nbsp;&nbsp;<span class=subsubtitlebar>");
				if(number_of_persons==1) {
					doc_text_printf(doc_html, "( 1 %s )", BILINGUAL(japanese, "名", "person"));
				} else {
					doc_text_printf(doc_html, "( %d %s )", number_of_persons, BILINGUAL(japanese, "名", "persons"));
				}
				doc_puts(doc_html, "</span>");
			}
			if(layer2==0) {
				doc_puts(doc_html, "</td><td width=\"1\">");
				JavaScript_printf(doc_html, "edb_ref_by_eid(%d,'button2');", TupleEID(oi->tuple));
				doc_puts(doc_html, "</td></tr></table>");
			}
			doc_puts(doc_html, "</td></tr>\n");
			if(number_of_persons>0 && layer2==0) doc_printf(doc_tex, "\\ORGANIZATION{%d}%%\n", opage->eid);

			if(layer>0) {
				doc_puts(doc_html, "<tr><td align=\"right\"><table width=\"95%\">\n");
				erd_organization_page_sub(ec, page, child, layer, layer2+1, japanese);
				doc_puts(doc_html, "</table></td></tr>\n");
			}
			++num;
		}
	}

	return TRUE;
}

static
int
erd_organization_page_sub_person(EdbContext ec, ErdPage page, EdbTupleInfo organ, int layer, int layer2, int top, int japanese)
{
	EdbPrint	ep_html = PAGE_EP_HTML(page, japanese);
	Doc	doc_html = ep_html->doc;
	EdbPrint	ep_tex = PAGE_EP_TeX(page, japanese);
	Doc	doc_tex = ep_tex->doc;
	EdbTupleColumn	tc;
	OrganizationInfo	oi = page->info;
	int	num = 0;
	Link	l_title;

	tc = edb_tuple_seek_column(ec, organ, "@.member");
	if(tc==NULL || tc->datum==NULL) return FALSE;

	ForCatalogueLink(l_title, ca_erd_person_title_listing_order) {
		char	*title_str = CatalogueObject(l_title);
		eid_t	title_eid = ATOI(title_str);
		EdbTupleInfo	title_tuple = edb_tuple_get(ec, title_eid, 0);
		EdbDatum	d;
		EdbCatalogue	ca_listing_person = edb_catalogue_new();
		Link	ld;

		ForTCDatum(d, tc) {
			EdbTupleInfo	person;
			EdbTupleColumn	ttc;
			EdbDatum	td;
			eid_t	do_listing = FALSE;

			if(!DatumEIDisValid(d)) continue;
			if(!edb_datum_at_present(ec, d)) continue;
			person = edb_tuple_get(ec, DatumEID(d), 0);
			if(!TupleIsPerson(person)) continue;
			if(edb_catalogue_lookup(ca_erd_omit_eid, TupleEID(person))) continue;

			ttc = edb_tuple_seek_TC_by_xmlname(ec, organ, tc, d, "@.title");
			if(ttc) {
				ForTCDatum(td, ttc) {
					if(DatumEIDisValid(td) && edb_datum_at_present(ec, td)) {
						if(DatumEID(td)==title_eid) {
							int	prefer_top;

							prefer_top = edb_catalogue_lookup(ca_erd_prefer_listing_top_eid, DatumEID(td))!=NULL;
							if((top && prefer_top) || (!top && !prefer_top)) do_listing = TRUE;
						}
					}
				}
			}
			if(!do_listing) continue;
			edb_catalogue_append(ca_listing_person, TupleEID(person), person);
		}

//		if(!top && edb_catalogue_size(ca_listing_person)>0 && layer2==0) doc_puts(doc_tex, "\\OBOUNDARY%\n");
		ForLink(ld, BILINGUAL(japanese, l_person_eid_order_ja, l_person_eid_order_en)) {
			eid_t		eid = ATOI(ld->d);
			ErdPage	ppage;
			EdbTupleInfo	person;

			if(!(person=edb_catalogue_lookup(ca_listing_person, eid))) continue;
			if(num==0 && layer2==0) doc_puts(doc_html, "<tr><td><hr /></td></tr>\n");
			if(num%NUM_OF_PORTRAITs_PER_COLUMN==0) {
				doc_puts(doc_html, (num==0) ? "<tr><td align=\"center\"><table border=\"0\">" : "</tr>");
				doc_puts(doc_html, "<tr align=\"center\" valign=\"top\">\n");
			}
			doc_printf(doc_html, "<td width=\"%d\">", PORTRAIT_ICON_WIDTH);
			ppage = erd_person_page(ec, page->ec_univ, TupleEID(person));
			edb_catalogue_append(oi->ca_person_info, TupleEID(((PersonInfo)ppage->info)->tuple), ppage->info);
			erd_linkto_person_page(page, ppage, P_SMALL|P_PORTRAIT|P_BOLD, japanese);
			if(title_tuple!=NULL) edb_print_tuple_named_column(ep_html, title_tuple, NULL, "<br /><span class=listing2>", "", "@.name", NULL, ", ", "</span>", "", 0);
			doc_puts(doc_html, "</td>\n");
			if(layer2==0) {
				doc_printf(doc_tex, "\\PERSONwithTITLE{%d}{", ppage->eid);
				if(title_tuple!=NULL) edb_print_tuple_named_column(ep_tex, title_tuple, NULL, "", "", "@.name", NULL, ", ", "", "", 0);
				doc_puts(doc_tex, "}%\n");
			}
			++num;
		}
		edb_catalogue_free(ca_listing_person);
	}
	if(num>0) {
		for( ; num<NUM_OF_PORTRAITs_PER_COLUMN; ++num) {
			doc_printf(doc_html, "<td width=\"%d\"><img src=\"../../%s\" alt=\"\" width=\"%d\" height=\"%d\" border=\"0\" /></td>\n",
				PORTRAIT_ICON_WIDTH, ICON_DUMMY_PORTRAIT, PORTRAIT_ICON_WIDTH, PORTRAIT_ICON_HEIGHT);
		}
		doc_puts(doc_html, "</tr></table></td></tr>\n");
	}
	return TRUE;
}

static
int
erd_organization_page_sub(EdbContext ec, ErdPage page, EdbTupleInfo organ, int layer, int layer2, int japanese)
{
	EdbPrint	ep = PAGE_EP_HTML(page, japanese);
	Doc	doc = ep->doc;
	EdbTupleColumn	tc;

	if(layer>10) return FALSE;
	tc = edb_tuple_seek_column(ec, organ, "@.member");
	if(!(tc && tc->datum)) return FALSE;
	if(layer==0 && layer2==0) {
		erd_organization_page_sub_person(ec, page, organ, layer, layer2, TRUE, japanese);
		doc_puts(doc, "<tr><td align=\"center\"><table width=\"80%\"><tr><td><hr /></td></tr>");
		erd_organization_page_sub_child(ec, page, organ, layer, layer2, japanese);
		doc_puts(doc, "</table></td></tr>");
		erd_organization_page_sub_person(ec, page, organ, layer, layer2, FALSE, japanese);
	} else {
		if(layer>1 || layer2==0) erd_organization_page_sub_person(ec, page, organ, layer, layer2, TRUE, japanese);
		erd_organization_page_sub_child(ec, page, organ, layer, layer2, japanese);
		if(layer>1 || layer2==0) erd_organization_page_sub_person(ec, page, organ, layer, layer2, FALSE, japanese);
	}
	return TRUE;
}

int
erd_organization_print_upper(EdbPrint ep, eid_t eid)
{
	EdbContext	ec = ep->ec;
	Doc			doc = ep->doc;
	EdbTupleInfo	organization = NULL;
	Caption		cap;
	int			japanese = (EpLang(ep)!=EP_LANG_English);

	if(!EIDisVALID(eid)) return FALSE;
	organization = edb_tuple_get(ec, eid, 0);
	if(organization==NULL) return FALSE;

	if(eid!=MY_ORGAN_EID) erd_organization_print_upper(ep, edb_tuple_hierarchyparent(ec, organization));

	if(DocIsHTML(doc)) {
		doc_printf(doc, "<a href=\"../../%s/%d/%s\">", TABLE_XN(TupleTable(organization)), TupleEID(organization), BILINGUAL(japanese, ORGANIZATION_PAGE_HTML_ja, ORGANIZATION_PAGE_HTML_en));
		cap = edb_make_caption(ec, eid, CAP_norecurse);
		if(japanese && STRisVALID(cap->c_main_japanese)) {
			doc_text_printf(doc, "%s", cap->c_main_japanese);
		} else if(!japanese && STRisVALID(cap->c_main_english)) {
			doc_text_printf(doc, "%s", cap->c_main_english);
		} else if(STRisVALID(cap->c_main)) {
			doc_text_puts(doc, cap->c_main);
		}
		doc_puts(doc, "</a>");
		doc_text_puts(doc, " > ");
		edb_caption_free(ec, cap);
	} else if(DocIsTeX(doc)) {
		cap = edb_make_caption(ec, eid, CAP_norecurse);
		if(japanese && STRisVALID(cap->c_main_japanese)) {
			doc_text_printf(doc, "%s", cap->c_main_japanese);
		} else if(!japanese && STRisVALID(cap->c_main_english)) {
			doc_text_printf(doc, "%s", cap->c_main_english);
		} else if(STRisVALID(cap->c_main)) {
			doc_text_puts(doc, cap->c_main);
		}
		doc_puts(doc, " ");
		edb_caption_free(ec, cap);
	}
	return TRUE;
}

static
int
erd_organization_page_body(EdbContext ec, ErdPage page, eid_t eid, int layer, int japanese)
{
	EdbPrint			ep_html = PAGE_EP_HTML(page, japanese);
	Doc				doc_html = ep_html->doc;
	EdbPrint			ep_tex = PAGE_EP_TeX(page, japanese);
	Doc				doc_tex = ep_tex->doc;
	Link				l;
	int				i;
	EdbTupleInfo		organization;
	EdbTupleColumn	tc;
	EdbDatum			datum;
	eid_t				parent_eid;

	organization = edb_tuple_get(ec, eid, 0);
	if(organization==NULL) return FALSE;

	edb_print_tuple_postfix_register_func(ep_html, PrintStyle(ep_html), erd_print_tuple_postfix);
	edb_print_eoi_add(ep_html, TupleEID(organization));

	erd_page_begin(page, japanese);
	doc_puts(doc_html, "<table border=\"0\" width=\"100%\">\n");

	doc_puts(doc_html, "<tr valign=\"top\">");
	doc_puts(doc_html, "<td class=affiliation>\n");
	erd_page_menu(page, japanese, FALSE);
//	doc_text_puts3(doc_html, "<a href=\"../../index.html\">", BILINGUAL(japanese, NAME_OF_CONTENTS_ja, NAME_OF_CONTENTS_en), "</a>");
	doc_text_puts(doc_html, " > ");
	parent_eid = edb_tuple_hierarchyparent(ec, organization);
	if(EIDisVALID(parent_eid)) erd_organization_print_upper(ep_html, parent_eid);
	doc_puts(doc_html, "</td></tr>\n");

	doc_puts(doc_html, "<tr valign=\"top\">\n");
	doc_puts(doc_html, "<td bgcolor=\"" ORGANIZATION_NAME_BGC "\">");
	doc_puts(doc_html, "<table border=\"0\" width=\"100%\">\n");
	doc_puts(doc_html, "<tr class=titlebar><td>");
	doc_text_puts(doc_html, PAGE_TITLE(page, japanese));
	doc_puts(doc_html, "</td><td width=\"1\" class=button1>");
	doc_printf(doc_html, "<input type=\"button\" value=\"%s\" onclick=\"top.location='%s';\">",
		BILINGUAL(japanese, "English", "Japanese"), PAGE_NAME_HTML(page, !japanese));
	doc_puts(doc_html, "</td><td width=\"1\">");
	JavaScript_printf(doc_html, "edb_ref_by_eid(%d,'button1');", TupleEID(organization));
	doc_puts(doc_html, "</td></tr></table>\n");
	doc_puts(doc_html, "</td></tr>\n");
	/* HTML */
	if(layer<=1) doc_puts(doc_html, "<div style=\"width:90%; margin-left:5%\">\n");
	/* TeX */
	switch(layer) {
	case 1: doc_text_puts3(doc_tex, "\\begin{chapter}{", PAGE_TITLE(page, japanese), "}\n");	 break;
	case 2: doc_text_puts3(doc_tex, "\\begin{section}{", PAGE_TITLE(page, japanese), "}\n");	 break;
	case 3: doc_text_puts3(doc_tex, "\\begin{subsection}{", PAGE_TITLE(page, japanese), "}\n");	 break;
	case 4: doc_text_puts3(doc_tex, "\\begin{subsubsection}{", PAGE_TITLE(page, japanese), "}\n");	 break;
	default:
		if(layer>=5) doc_text_puts3(doc_tex, "\\OCAPTION{", PAGE_TITLE(page, japanese), "}%\n");
		break;
	}

	erd_organization_page_sub(ec, page, organization, layer, 0, japanese);

	/* HTML */
	if(layer<=1) doc_puts(doc_html, "</div>\n");
	doc_puts(doc_html, "</table>\n");
	/* TeX */
	switch(layer) {
	case 1: doc_puts(doc_tex, "\\end{chapter}\n");	 break;
	case 2: doc_puts(doc_tex, "\\end{section}\n");	 break;
	case 3: doc_puts(doc_tex, "\\end{subsection}\n");	 break;
	case 4: doc_puts(doc_tex, "\\end{subsubsection}\n");	 break;
	default: break;
	}

	erd_page_end(page, japanese);
	return TRUE;
}

static	char	*abbreviate_list[] = {
	"Center for ",
	"Course in ",
	"Course of ",
	"Department of ",
	"Faculty of ",
	"Graduate School of ",
	"Institute for ",
	"Institute of ",
	"Major in ",
	"School of ",
	"Office of ",
	"The ",
};

ErdPage
erd_organization_page(EdbContext ec, EdbContext ec_univ, eid_t eid, int layer, int japanese)
{
	EdbTupleInfo		organization;
	Caption			cap;
	ErdPage			page;
	OrganizationInfo	oi;
	int				i;

	if(page=edb_catalogue_lookup(ca_erd_organization_page, eid)) return page;
	organization = edb_tuple_get(ec, eid, 0);
	if(organization==NULL) return NULL;

	page = erd_page_new(ec, ec_univ, &organization_pages_template, organization);

	page->info = oi = organization_info_new(ec);
	oi->tuple = organization;
	edb_catalogue_append(ca_erd_organization_page, eid, page);

	cap = edb_make_caption(ec, eid, CAP_norecurse);
	if(STRisVALID(cap->c_main_japanese)) {
		oi->name_ja = strdup(cap->c_main_japanese);
	} else if(STRisVALID(cap->c_main_english)) {
		oi->name_ja = strdup(cap->c_main_english);
	} else if(STRisVALID(cap->c_main)) {
		oi->name_ja = strdup(cap->c_main);
	} else {
		asprintf(&oi->name_ja, "[%d]", TupleEID(organization));
	}
	if(STRisVALID(cap->c_main_english)) {
		oi->name_en = strdup(cap->c_main_english);
	} else if(STRisVALID(cap->c_main_japanese)) {
		oi->name_en = strdup(cap->c_main_japanese);
	} else if(STRisVALID(cap->c_main)) {
		oi->name_en = strdup(cap->c_main);
	} else {
		asprintf(&oi->name_en, "[%d]", TupleEID(organization));
	}
	if(STRisVALID(cap->c_main_pronounce)) {
		oi->name_pr = strdup(cap->c_main_pronounce);
	} else {
		oi->name_pr = strdup("");
	}
	oi->order_ja = UTF8toUCS4(oi->name_pr);
	pronounce_convert_order(oi->order_ja);
	oi->order_ja = ucs4_strdup(oi->order_ja);

	for(i=0; i<Ns(abbreviate_list); ++i) {
		int	len;

		if(!strncasecmp(oi->name_en, abbreviate_list[i], len=strlen(abbreviate_list[i]))) {
			asprintf(&oi->name_en_idx, "%s, %s", &oi->name_en[len], abbreviate_list[i]);
			break;
		}
	}
	if(i==Ns(abbreviate_list)) oi->name_en_idx = strdup(oi->name_en);
	oi->order_en = UTF8toUCS4(oi->name_en_idx);
	ucs4_str_capitalize1(oi->order_en);
	oi->order_en = ucs4_strdup(oi->order_en);

	page->title_en = strdup(oi->name_en);
	page->title_ja = strdup(oi->name_ja);

	if(layer<2) {
		for(i=0; i<layer; ++i) printf("  ");
		printf("%s\n", oi->name_en);
	}
	edb_caption_free(ec, cap);

	erd_page_open(page);
	erd_organization_page_body(ec, page, eid, layer, TRUE);
	erd_organization_page_body(ec, page, eid, layer, FALSE);
	erd_page_close(page);

	return page;
}

static
int
erd_organization_print_upper_for_index(EdbPrint ep, eid_t eid)
{
	EdbContext	ec = ep->ec;
	Doc			doc = ep->doc;
	EdbTupleInfo	organization = NULL;
	Caption		cap;
	int			japanese = (EpLang(ep)!=EP_LANG_English);
	eid_t			parent;

	if(!EIDisVALID(eid)) return FALSE;
	organization = edb_tuple_get(ec, eid, 0);
	if(organization==NULL) return FALSE;

	if(eid==MY_ORGAN_EID) return FALSE;
	parent = edb_tuple_hierarchyparent(ec, organization);
	if(parent==MY_ORGAN_EID) parent = 0;
	if(japanese && EIDisVALID(parent)) {
		erd_organization_print_upper_for_index(ep, parent);
		doc_puts(doc, " ");
	}

	cap = edb_make_caption(ec, eid, CAP_norecurse);
	if(japanese && STRisVALID(cap->c_main_japanese)) {
		doc_text_printf(doc, "%s", cap->c_main_japanese);
	} else if(!japanese && STRisVALID(cap->c_main_english)) {
		doc_text_printf(doc, "%s", cap->c_main_english);
	} else if(STRisVALID(cap->c_main)) {
		doc_text_puts(doc, cap->c_main);
	}
	if(japanese) {
	} else if(EIDisVALID(parent)) {
		doc_text_puts(doc, ", ");
	} else {
		doc_text_puts(doc, ".");
	}

	if(!japanese && EIDisVALID(parent)) erd_organization_print_upper_for_index(ep, parent);

	edb_caption_free(ec, cap);
	return TRUE;
}

static
int
organization_sort_compare_ja(LinkObj o1, LinkObj o2)
{
	OrganizationInfo	oi1 = ((ErdPage)o1->d)->info;
	OrganizationInfo	oi2 = ((ErdPage)o2->d)->info;

	return ucs4_strcmp(oi1->order_ja, oi2->order_ja);
}

static
int
organization_sort_compare_en(LinkObj o1, LinkObj o2)
{
	OrganizationInfo	oi1 = ((ErdPage)o1->d)->info;
	OrganizationInfo	oi2 = ((ErdPage)o2->d)->info;

	return ucs4_strcmp(oi1->order_en, oi2->order_en);
}

void
erd_organization_index_page(EdbContext ec, ErdPage page, int japanese)
{
	EdbPrint	ep_html = PAGE_EP_HTML(page, japanese);
	Doc		doc_html = ep_html->doc;
	EdbPrint	ep_tex = PAGE_EP_TeX(page, japanese);
	Doc		doc_tex = ep_tex->doc;
	Link		l, l_page;
	int		num;
	ucs4_t	*cap_index0;
	ucs4_t	*cap_index1;
	int		cap_idx, cap_max;
	int		i;

	edb_print_tuple_postfix_register_func(ep_html, PrintStyle(ep_html), erd_print_tuple_postfix);
	edb_print_eoi_add(ep_html, MY_ORGAN_EID);

	erd_page_begin(page, japanese);

	doc_puts(doc_html, "<table border=\"0\" width=\"100%\">\n");

	doc_puts(doc_html, "<tr valign=\"top\">");
	doc_puts(doc_html, "<td class=affiliation>\n");
	erd_page_menu(page, japanese, FALSE);
//	doc_text_puts3(doc_html, "<a href=\"index.html\">", BILINGUAL(japanese, NAME_OF_CONTENTS_ja, NAME_OF_CONTENTS_en), "</a>");
	doc_text_puts(doc_html, " > ");
	doc_puts(doc_html, "</td></tr>\n");

	doc_puts(doc_html, "<tr valign=\"top\">\n");
	doc_puts(doc_html, "<td bgcolor=\"" INDEX_TITLE_BGC "\">");
	doc_puts(doc_html, "<table border=\"0\" width=\"100%\">\n");
	doc_puts(doc_html, "<tr class=titlebar><td>");
	doc_text_puts(doc_html, PAGE_TITLE(page, japanese));
	doc_puts(doc_html, "</td><td width=\"1\" class=button1>");
	doc_printf(doc_html, "<input type=\"button\" value=\"%s\" onclick=\"top.location='%s';\">",
		BILINGUAL(japanese, "English", "Japanese"), PAGE_NAME_HTML(page, !japanese));
	doc_puts(doc_html, "</td></tr></table>\n");
	doc_puts(doc_html, "</td></tr>\n");
	doc_puts(doc_html, "<tr><td><hr /></td></tr>\n");

	doc_puts(doc_html, "<tr><td>\n");
	if(japanese) {
		cap_index0 = UTF8toUCS4(LEXICOGRAPHICAL_ORDER_ja);
		cap_index1 = UCS4_STRDUP(cap_index0);
		ucs4_str_tokatakana(cap_index1);
	} else {
		cap_index0 = UTF8toUCS4(LEXICOGRAPHICAL_ORDER_en);
		cap_index1 = UCS4_STRDUP(cap_index0);
	}
	cap_max = ucs4_strlen(cap_index0);

	for(i=0; i<cap_max; ++i) {
		ucs4_t	u[2];

		doc_puts(doc_html, "<input type=\"button\" value=\"");
		u[0] = cap_index0[i];
		u[1] = UCS4_NULL;
		doc_text_puts(doc_html, UCS4toUTF8(u));
		doc_printf(doc_html, "\" onclick=\"top.location.replace('#%04x');\" class=button2>\n", cap_index0[i]);
	}

	doc_puts(doc_html, "</td></tr>");
	doc_puts(doc_html, "<tr><td><hr /></td></tr>\n");
	doc_puts(doc_html, "<tr><td align=\"center\">\n");

	l_page = NULL;
	ForCatalogueLink(l, ca_erd_organization_page) {
		ErdPage		opage = CatalogueObject(l);

		l_page = link_append(link_atom_new(opage, NULL), l_page);
	}
	link_sort(l_page, BILINGUAL(japanese, organization_sort_compare_ja, organization_sort_compare_en), NULL, NULL, NULL);

	num = 0;
	cap_idx = -1;
	doc_puts(doc_html, "<table border=\"0\" class=listing>\n");
	ForLink(l, l_page) {
		ErdPage		opage = l->d;
		OrganizationInfo	oi = opage->info;
		ucs4_t		new_cap;
		eid_t			parent;
		int			npersons;

		new_cap = BILINGUAL(japanese, oi->order_ja[0], oi->order_en[0]);
		if(cap_idx<0 || cap_index1[cap_idx]<new_cap) {
			while(cap_idx+1<cap_max && cap_index1[cap_idx+1]<=new_cap) {
				ucs4_t	u[2];

				++cap_idx;
				doc_printf(doc_html, "<tr align=\"left\" valign=\"top\"><td bgcolor=\"%s\" align=\"left\">", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
				doc_printf(doc_html, "<span class=cap><a name=\"%04x\">", cap_index0[cap_idx]);
				u[0] = cap_index0[cap_idx];
				u[1] = UCS4_NULL;
				doc_text_puts(doc_html, UCS4toUTF8(u));
				doc_puts(doc_html, "</a></span>");
				doc_puts(doc_html, BILINGUAL(japanese, RtnTOP_ja, RtnTOP_en));
				doc_puts(doc_html, "</td></tr>\n");
				++num;
			}
		}
		npersons = edb_catalogue_size(oi->ca_person_info);
		doc_printf(doc_html, "<tr align=\"left\" valign=\"top\" bgcolor=\"%s\"><td>", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
		if(npersons>0) doc_printf(doc_html, "<a href=\"%s\" target=\"_blank\">", PAGE_PATH_HTML(opage, japanese));
		doc_text_printf(doc_html, "%s", BILINGUAL(japanese, oi->name_ja, oi->name_en_idx));
		if(npersons>0) doc_puts(doc_html, "</a>");
		doc_puts(doc_html, "</td><td>");
		doc_printf(doc_tex, "%%%%ORGANIZATION %d\n", opage->eid);
		parent = edb_tuple_hierarchyparent(ec, oi->tuple);
		if(EIDisVALID(parent)) erd_organization_print_upper_for_index(ep_html, parent);
		doc_puts(doc_html, "</td></tr>\n");
		++num;
	}
	while(cap_idx+1<cap_max) {
		ucs4_t	u[2];

		++cap_idx;
		doc_printf(doc_html, "<tr align=\"left\" valign=\"top\"><td bgcolor=\"%s\" align=\"left\">", cap_idx%2? ITEM_BGC1 : ITEM_BGC2);
		doc_printf(doc_html, "<span class=cap><a name=\"%04x\">", cap_index0[cap_idx]);
		u[0] = cap_index0[cap_idx];
		u[1] = UCS4_NULL;
		doc_text_puts(doc_html, UCS4toUTF8(u));
		doc_puts(doc_html, "</a></span>");
		doc_puts(doc_html, BILINGUAL(japanese, RtnTOP_ja, RtnTOP_en));
		doc_puts(doc_html, "</td></tr>\n");
		++num;
	}
	link_free(l_page);
	doc_puts(doc_html, "</table>\n");
	doc_puts(doc_html, "</td></tr></table>\n");

	erd_page_end(page, japanese);
}


