/*
	$Id: page.c,v 1.3 2004/07/22 08:24:27 alex Exp alex $
	page.c
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: page.c,v 1.3 2004/07/22 08:24:27 alex Exp alex $";
#endif

#include	<stdio.h>
#include	<math.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<ctype.h>
#include	<string.h>
#include	<signal.h>
#include	"edb/edb.h"
#include	"edb/html.h"
#include	"edb/publish.h"
#include	"edb/javascript.h"
#include	"erd.h"


ErdPage
erd_page_new(EdbContext ec, EdbContext ec_univ, ErdPage template, EdbTupleInfo tuple)
{
	ErdPage	page;

	page = (ErdPage)malloc(sizeof(struct ERD_PAGE));
	bcopy(template, page, sizeof(struct ERD_PAGE));
	page->ec = ec;
	page->ec_univ = ec_univ;
	page->tuple = tuple;
	if(tuple) {
		page->eid = TupleEID(tuple);
		asprintf(&page->dir, "%s/%d", TABLE_XN(TupleTable(tuple)), TupleEID(tuple));
		makedir_hierarchy(page->dir);
		page->depth = 2;
		asprintf(&page->path_html_ja, "%s/%s", page->dir, page->name_html_ja);
		asprintf(&page->path_html_en, "%s/%s", page->dir, page->name_html_en);
		asprintf(&page->path_tex_ja, "%s/%s", page->dir, page->name_tex_ja);
		asprintf(&page->path_tex_en, "%s/%s", page->dir, page->name_tex_en);
	} else {
		page->eid = 0;
		page->dir = strdup("");
		page->depth = 0;
		page->path_html_ja = strdup(page->name_html_ja);
		page->path_html_en = strdup(page->name_html_en);
		page->path_tex_ja = strdup(page->name_tex_ja);
		page->path_tex_en = strdup(page->name_tex_en);
	}
	asprintf(&page->path_html_tmp_ja, "%s.tmp", page->path_html_ja);
	asprintf(&page->path_html_tmp_en, "%s.tmp", page->path_html_en);
	asprintf(&page->path_tex_tmp_ja, "%s.tmp", page->path_tex_ja);
	asprintf(&page->path_tex_tmp_en, "%s.tmp", page->path_tex_en);
	return page;
}

int
erd_page_open(ErdPage page)
{
	EdbContext	ec;
	Doc			doc;

	if(page==NULL) return FALSE;

	ec = page->ec;
	doc = doc_open(DocML_HTML, page->path_html_tmp_ja, "w+");
	page->ep_html_ja = edb_print_new(ec, doc, PrintStyle_STANDARD);
	EpLang(page->ep_html_ja) = EP_LANG_Japanese;
	EpUserData(page->ep_html_ja) = page;
	page->ep_html_ja->ct->noalert = !erd_preview;
	page->ep_html_ja->ct->nocolor = !erd_preview;

	if(!strcmp(page->path_html_tmp_ja, page->path_html_tmp_en)) {
		doc = doc_fpopen(DocML_HTML, page->ep_html_ja->doc->fp);
	} else {
		doc = doc_open(DocML_HTML, page->path_html_tmp_en, "w+");
	}
	page->ep_html_en = edb_print_new(ec, doc, PrintStyle_STANDARD);
	EpLang(page->ep_html_en) = EP_LANG_English;
	EpUserData(page->ep_html_en) = page;
	page->ep_html_en->ct->noalert = !erd_preview;
	page->ep_html_en->ct->nocolor = !erd_preview;

	doc = doc_open(DocML_TeX, page->path_tex_tmp_ja, "w+");
	page->ep_tex_ja = edb_print_new(ec, doc, PrintStyle_STANDARD);
	EpLang(page->ep_tex_ja) = EP_LANG_Japanese;
	EpUserData(page->ep_tex_ja) = page;
	page->ep_tex_ja->ct->noalert = !erd_preview;
	page->ep_tex_ja->ct->nocolor = !erd_preview;

	if(!strcmp(page->path_tex_tmp_ja, page->path_tex_tmp_en)) {
		doc = doc_fpopen(DocML_TeX, page->ep_tex_ja->doc->fp);
	} else {
		doc = doc_open(DocML_TeX, page->path_tex_tmp_en, "w+");
	}
	page->ep_tex_en = edb_print_new(ec, doc, PrintStyle_STANDARD);
	EpLang(page->ep_tex_en) = EP_LANG_English;
	EpUserData(page->ep_tex_en) = page;
	page->ep_tex_en->ct->noalert = !erd_preview;
	page->ep_tex_en->ct->nocolor = !erd_preview;

	return TRUE;
}

int
erd_page_close(ErdPage page)
{

	if(page==NULL) return FALSE;

	if(page->ep_tex_en) {
		edb_print_free(page->ep_tex_en);
		if(!strcmp(page->path_tex_tmp_ja, page->path_tex_tmp_en)) {
			doc_fpclose(page->ep_tex_en->doc);
		} else {
			doc_close(page->ep_tex_en->doc);
			file_update(page->path_tex_tmp_en, page->path_tex_en);
		}
	}

	if(page->ep_tex_ja) {
		edb_print_free(page->ep_tex_ja);
		doc_close(page->ep_tex_ja->doc);
		file_update(page->path_tex_tmp_ja, page->path_tex_ja);
	}

	if(page->ep_html_en) {
		edb_print_free(page->ep_html_en);
		if(!strcmp(page->path_html_tmp_ja, page->path_html_tmp_en)) {
			doc_fpclose(page->ep_html_en->doc);
		} else {
			doc_close(page->ep_html_en->doc);
			file_update(page->path_html_tmp_en, page->path_html_en);
		}
	}

	if(page->ep_html_ja) {
		edb_print_free(page->ep_html_ja);
		doc_close(page->ep_html_ja->doc);
		file_update(page->path_html_tmp_ja, page->path_html_ja);
	}

	return TRUE;
}

void
erd_page_free(ErdPage page)
{

	if(page==NULL) return;

	FREE(page->dir);
	FREE(page->path_html_ja);
	FREE(page->path_html_en);
	FREE(page->path_html_tmp_ja);
	FREE(page->path_html_tmp_en);
	FREE(page->path_tex_ja);
	FREE(page->path_tex_en);
	FREE(page->path_tex_tmp_ja);
	FREE(page->path_tex_tmp_en);
	free(page);
}

void
erd_page_begin(ErdPage page, int japanese)
{
	EdbPrint		ep = PAGE_EP_HTML(page, japanese);
	Doc	doc = ep->doc;
	int	depth = page->depth;
	char	*dirprefix;

	if(depth==0) {
		dirprefix = "";
	} else if(depth==1) {
		dirprefix = "../";
	} else if(depth==2) {
		dirprefix = "../../";
	} else {
		dirprefix = "";
	}
	doc_puts(doc, "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n");
	doc_puts(doc, "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");
	doc_puts(doc, "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"ja\" lang=\"ja\">\n");
	doc_puts(doc, "	<head>\n");
	doc_puts(doc, "		<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Content-Style-Type\" content=\"text/css; charset=utf-8\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Pragma\" content=\"no-cache\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Cache-Control\" content=\"no-cache\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Expires\" content=\"Thu, 01 Dec 1994 16:00:00 GMT\" />\n");
	doc_printf(doc, BILINGUAL3(japanese, "		<link rel=\"stylesheet\" type=\"text/css\" href=\"%s", ERD_CSS_NAME_ja, ERD_CSS_NAME_en, "\" title=\"erd\" />\n"), dirprefix);
	doc_puts(doc, "		<title>");
	doc_text_puts(doc, BILINGUAL(japanese, NAME_OF_CONTENTS_ja, NAME_OF_CONTENTS_en));
	doc_text_puts(doc, " --- ");
	doc_text_puts(doc, PAGE_TITLE(page, japanese));
	if(erd_preview) doc_text_puts(doc, " --- (preview)");
	doc_puts(doc, "		</title>\n");
	doc_puts(doc, "	</head>\n");
	doc_printf(doc, "<body text=\"black\" bgcolor=\"white\" style=\"margin:0px 5%%\">\n");
	doc_puts(doc, "<a name=\"" TOPANC "\">");
	if(depth==0) {
		JavaScript_source(doc, ERD_JS_NAME);
	} else if(depth==1) {
		JavaScript_source(doc, "../" ERD_JS_NAME);
	} else if(depth==2) {
		JavaScript_source(doc, "../../" ERD_JS_NAME);
	}
	doc_puts(doc, "</a>\n");
	JavaScript_printf(doc, "lang_is_japanese = %s;\nwrite_if_top('<br />');\n", (EpLang(ep)==EP_LANG_English) ? "false" : "true");
}

void
erd_page_footer(ErdPage page, int japanese)
{
	EdbPrint		ep = PAGE_EP_HTML(page, japanese);
	EdbContext	ec = ep->ec;
	Doc	doc = ep->doc;
	char	timestr[BUFSIZ];

	JavaScript_begin(doc);
	doc_puts(doc, "if(top==self) {\n");
	doc_puts(doc,  "	document.writeln('<hr /><span class=colophon>');\n");
	if(japanese) {
		doc_puts(doc,  "	document.writeln('本ページの内容は');\n");
		doc_puts(doc,  "	document.writeln('<a href=\"" WEB_PORTAL_URL "\" target=\"_blank\">" NAME_OF_DATABASE_ja " (" NAME_OF_DB ")</a>');\n");
		doc_puts(doc,  "	document.writeln('の登録情報に基づいて作成されています．');\n");
	} else {
		doc_puts(doc,  "	document.writeln('Content of this page is based on the information registered in ');\n");
		doc_puts(doc,  "	document.writeln('<a href=\"" WEB_PORTAL_URL "\" target=\"_blank\">" NAME_OF_DATABASE_en " (" NAME_OF_DB ")</a>.');\n");
	}
	ctime_r(&ec->now, timestr);
	str_oneline(timestr);
	doc_printf(doc, "	document.writeln('<br />');\n");
	doc_printf(doc, "	document.writeln('--- %s &lt;%s&gt; at %s.');\n", EDB_WG_NAME, EDB_ADMIN_MAILADDR_SHOW, timestr);
	doc_puts(doc,  "	document.writeln('</span>');\n");
	doc_puts(doc, "}\n\n");
	JavaScript_end(doc);
}

void
erd_page_end(ErdPage page, int japanese)
{
	EdbPrint		ep;
	Doc			doc;

	ep = PAGE_EP_HTML(page, japanese);
	doc = ep->doc;
	erd_page_footer(page, japanese);
	doc_puts(doc, "</body>\n");
	doc_puts(doc, "</html>\n");

	ep = PAGE_EP_TeX(page, japanese);
	doc = ep->doc;
	doc_puts(doc, "\\endinput\n");
}

static
void
erd_page_menu_item(EdbPrint ep, char *name, char *file, int depth)
{
	Doc	doc = ep->doc;
	int	i;

	doc_puts(doc, "<option value=\"");
	for(i=0; i<depth; ++i) doc_puts(doc, "../");
	doc_puts(doc, file);
	doc_text_puts3(doc, "\">", name, "</option>\n");
}

void
erd_page_menu(ErdPage page, int japanese, int linkto_manual)
{
	EdbPrint		ep = PAGE_EP_HTML(page, japanese);
	Doc			doc = ep->doc;
	char	*s;

	doc_puts(doc, "<select name=\"page_menu\" onchange=\"move_page(this);\" class=affiliation>\n");
	doc_text_puts3(doc, "<option value=\"\">", BILINGUAL(japanese, NAME_OF_CONTENTS_ja, NAME_OF_CONTENTS_en), "</option>\n");
	if(japanese) {
		erd_page_menu_item(ep, " ⇒目次", "index.html", page->depth);
		asprintf(&s, "organization/%d/index-ja.html", MY_ORGAN_EID);
		erd_page_menu_item(ep, " ⇒所属別教職員一覧", s, page->depth);
		FREE(s);
		erd_page_menu_item(ep, " ⇒氏名索引", "person-idx2-ja.html", page->depth);
		erd_page_menu_item(ep, " ⇒氏名索引 (顔写真付)", "person-idx-ja.html", page->depth);
		erd_page_menu_item(ep, " ⇒キーワード索引", "keyword-idx-ja.html", page->depth);
		erd_page_menu_item(ep, " ⇒組織名索引", "organization-idx-ja.html", page->depth);
		if(linkto_manual) {
			doc_text_puts3(doc, "<option value=\"\">", "---", "</option>\n");
			doc_text_puts3(doc, "<option value=\"https://web.db.tokushima-u.ac.jp/Assistance/mypage/erd/\">", "⇒内容修正", "</option>\n");
		}
	} else {
		erd_page_menu_item(ep, " -> Table of Contents", "index.html", page->depth);
		asprintf(&s, "organization/%d/index-ja.html", MY_ORGAN_EID);
		erd_page_menu_item(ep, " -> Faculty List by Affiliation", s, page->depth);
		FREE(s);
		erd_page_menu_item(ep, " -> Faculty Index", "person-idx2-en.html", page->depth);
		erd_page_menu_item(ep, " -> Faculty Index (with portrait)", "person-idx-en.html", page->depth);
		erd_page_menu_item(ep, " -> Keyword Index", "keyword-idx-en.html", page->depth);
		erd_page_menu_item(ep, " -> Affiliation Index", "organization-idx-en.html", page->depth);
		if(linkto_manual) {
			doc_text_puts3(doc, "<option value=\"\">", "---", "</option>\n");
			doc_text_puts3(doc, "<option value=\"https://web.db.tokushima-u.ac.jp/Assistance/mypage/erd/\">", "-> How to modify", "</option>\n");
		}
	}
	doc_puts(doc, "</select>\n");
}

int
erd_page_dir_exist(EdbContext ec, EdbTupleInfo tuple)
{
	int	st;
	char	*dir;

	asprintf(&dir, "%s/%d", TABLE_XN(TupleTable(tuple)), TupleEID(tuple));
	st = access(dir, F_OK);
	FREE(dir);
	return st==0;
}

