/*
	$Id: person.c,v 1.3 2004/07/22 08:24:27 alex Exp alex $
	erd--- Educator & Researcher Directory
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: person.c,v 1.3 2004/07/22 08:24:27 alex Exp alex $";
#endif

#include	<stdio.h>
#include	<math.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<ctype.h>
#include	<string.h>
#include	<signal.h>
#include	"edb/edb.h"
#include	"edb/html.h"
#include	"edb/publish.h"
#include	"edb/javascript.h"
#include	"erd.h"

static
struct ERD_PAGE	person_profile_pages_template = {
	PERSON_PROFILE_HTML_ja,
	PERSON_PROFILE_HTML_en,
	PERSON_PROFILE_TeX_ja,
	PERSON_PROFILE_TeX_en,
	"個人",
	"Person",
	TRUE,
	2,
	NULL,
};

static
struct ERD_PAGE	person_work_pages_template = {
	PERSON_WORK_HTML_ja,
	PERSON_WORK_HTML_en,
	PERSON_WORK_TeX_ja,
	PERSON_WORK_TeX_en,
	"個人",
	"Person",
	FALSE,
	2,
	NULL,
};

static
int
erd_print_birthday(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	register Doc	doc = ep->doc;

	if(datum) {
		int	v;
		v = edb_datum_get_DATE(ec, datum);

		if(DocIsTeX(doc)) {
			doc_printf(doc, "%d", DATE_Y(v));
		} else if(EpLang(ep)==EP_LANG_English) {
			doc_printf(doc, "%d A.D.", DATE_Y(v));
		} else {
			doc_printf(doc, "西暦%d年", DATE_Y(v));
		}
		return TRUE;
	}
	return FALSE;
}

static
int
erd_print_YM_TeX(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	register Doc	doc = ep->doc;

	if(datum) {
		int	v;
		v = edb_datum_get_DATE(ec, datum);

		if(0<DATE_M(v) && DATE_M(v)<=12) {
			doc_printf(doc, "\\年月{%d.{\\EdbZSp}%d.}", DATE_Y(v), DATE_M(v));
		} else {
			doc_printf(doc, "\\年月{%d.}", DATE_Y(v));
		}
		return TRUE;
	}
	return FALSE;
}

static
int
erd_print_DateYM_TeX(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	register Doc	doc = ep->doc;

	if(datum) {
		int	v;
		v = edb_datum_get_DATE(ec, datum);

		if(0<DATE_M(v) && DATE_M(v)<=12) {
			doc_printf(doc, "\\DateYM{%d.{\\EdbZSp}%d}", DATE_Y(v), DATE_M(v));
		} else {
			doc_printf(doc, "\\DateYM{%d}", DATE_Y(v));
		}
		return TRUE;
	}
	return FALSE;
}

static
int
erd_print_PERIOD_TeX(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	register Doc	doc = ep->doc;
	int	status = FALSE;
	int	y1, y2, m1, m2, date1, date2;

	edb_datum_get_DATE2(ep->ec, datum, &date1, &date2);
	if(date1==EDB_DATE_MIN && date2==EDB_DATE_MAX) return FALSE;

	y1 = DATE_Y(date1);
	m1 = DATE_M(date1);
	y2 = DATE_Y(date2);
	m2 = DATE_M(date2);
	doc_puts(doc, "\\期間{");
	if(date1>EDB_DATE_MIN) {
		doc_printf(doc, "%d", y1);
		if(1<=m1 && m1<=12) doc_printf(doc, ".{\\EdbZSp}%d", m1);
	}
	if(date2==EDB_DATE_MAX) {
		if(date1>EDB_DATE_MIN) doc_puts(doc, "{\\EdbZSp}$\\sim${\\EdbZSp}");
	} else if(y1==y2 && m2>m1) {
		doc_puts(doc, "{\\EdbZSp}$\\sim${\\EdbZSp}");
		if(1<=m2 && m2<=12) doc_printf(doc, "%d", m2);
	} else if(y2>y1) {
		doc_puts(doc, "{\\EdbZSp}$\\sim${\\EdbZSp}");
		doc_printf(doc, "%d", y2);
		if(1<=m2 && m2<=12) doc_printf(doc, ".{\\EdbZSp}%d", m2);
	}
	doc_puts(doc, "}");
	return status;
}

static
int
erd_print_email(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{

	if(DatumEnglishIsValid(datum)) {
		return edb_print_emailaddress_in_HTML_avoid_SPAM(ep->doc, DatumEnglish(datum));
	}
	return FALSE;
}

static
int
erd_print_phone(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	register Doc	doc = ep->doc;

	if(DocIsHTML(doc)) {
		if(DatumEnglishIsValid(datum)) {
			char	*s = DatumEnglish(datum);

			if(EpLang(ep)==EP_LANG_English && s[0]=='0') {
				doc_puts(doc, "+&nbsp;81&nbsp;-&nbsp;");
				++s;
			}

			while(*s) {
				switch(*s) {
				case '-':
					doc_puts(doc, "&nbsp;-&nbsp;");
					break;
				default:
					doc_putc(doc, *s);
					break;
				}
				++s;
			}
			return TRUE;
		}
	} else if(DocIsTeX(doc)) {
		if(DatumEnglishIsValid(datum)) {
			char	*s = DatumEnglish(datum);

			if(EpLang(ep)==EP_LANG_English && s[0]=='0') {
				doc_puts(doc, "+81-");
				++s;
			}

			doc_puts(doc, s);
			return TRUE;
		}
	}
	return FALSE;
}

static
PersonInfo
person_info_new(EdbContext ec)
{
	PersonInfo	pi;

	pi = (PersonInfo)malloc(sizeof(struct PERSON_INFO));
	bzero(pi, sizeof(struct PERSON_INFO));
	pi->ca_work = (EdbCatalogue *)malloc(sizeof(EdbCatalogue)*edb_catalogue_size(ca_erd_person_work_list));
	return pi;
}

static
void
person_info_free(EdbContext ec, PersonInfo pi)
{
	int	i, n;

	if(pi==NULL) return;
	edb_catalogue_free(pi->ca_teaching);
	edb_catalogue_free(pi->ca_study);
	edb_catalogue_free(pi->ca_history_a);
	edb_catalogue_free(pi->ca_history_w);
	edb_catalogue_free(pi->ca_asa);
	edb_catalogue_free(pi->ca_sa_and_ph);
	edb_catalogue_free(pi->ca_prize);
	n = edb_catalogue_size(ca_erd_person_work_list);
	for(i=0; i<n; ++i) edb_catalogue_free(pi->ca_work[i]);
	free(pi->ca_work);
	free(pi);
}

static
void
tuple_expand_sub(EdbContext ec, EdbCatalogue ca, EdbTupleInfo tuple)
{
	Link	l;
	EdbTupleInfo	rtuple;

	if(tuple==NULL) return;
	if(edb_catalogue_lookup(ca, TupleEID(tuple))) return;
	edb_catalogue_append(ca, TupleEID(tuple), tuple);

	edb_tuple_expand_check(ec, tuple);
	ForLink(l, tuple->expand_map) tuple_expand_sub(ec, ca, (EdbTupleInfo)l->d);
	if(EIDisVALID(TupleMapto(tuple))) {
		rtuple = edb_tuple_get(ec, TupleMapto(tuple), 0);
		if(rtuple) tuple_expand_sub(ec, ca, rtuple);
	}
}

static
EdbCatalogue
tuple_expand(EdbContext ec, EdbTupleInfo tuple)
{
	EdbCatalogue	ca;

	ca = edb_catalogue_new();
	tuple_expand_sub(ec, ca, tuple);
	return ca;
}

static
EdbCatalogue
erd_get_person_history(EdbContext ec, eid_t person, eid_t kind)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = edb_condition_make(ec, erd_table_history, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.subject", person),
		ConditionEIDMatch("@.attribute", kind),
		ConditionTerminate);
	ca = erd_catalogue_get(ec, erd_table_history, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
erd_get_person_affiliate(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL, ca_new;
	char	*condition = NULL;
	Link	l;
	EdbTupleInfo	ptuple = edb_tuple_get(ec, person, 0);

	condition = edb_condition_make(ec, erd_table_organization, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.member", person),
		ConditionTerminate);
	ca = erd_catalogue_get(ec, erd_table_organization, condition);
	FREE(condition);

	ca_new = edb_catalogue_new();
	ForCatalogueLink(l, ca) {
		EdbTupleInfo	otuple = CatalogueObject(l);
		eid_t	attribute;

		if(otuple==NULL) continue;
		if(edb_catalogue_lookup(ca_erd_omit_eid, TupleEID(otuple))) continue;
		attribute = erd_organization_attribute(ec, otuple);
		if(!EIDisValid(attribute) || edb_catalogue_lookup(ca_erd_omit_eid, attribute)) continue;
		if(!edb_organization_is_inside(ec, otuple, MY_ORGAN_EID)) continue;
		if(!erd_page_dir_exist(ec, otuple)) continue;
		if(edb_organization_person_is_belonged(ec, otuple, ptuple)) edb_catalogue_append(ca_new, TupleEID(otuple), otuple);
	}
	edb_catalogue_free(ca);

	return ca_new;
}


struct TEACHING_SORT_DATA {
	EdbContext	ec;
	int	order_course;
	int	order_style;
	int	order_name;
};
static
void *
teaching_sort_aux(EdbTupleInfo teaching, void *data)
{
	EdbContext	ec;
	EdbTupleColumn	tc;
	// int	printed = FALSE;
	struct TEACHING_SORT_DATA	*sd0 = data;
	struct TEACHING_SORT_DATA	*sd;

	sd = (struct TEACHING_SORT_DATA *)malloc(sizeof(struct TEACHING_SORT_DATA));
	bzero(sd, sizeof(struct TEACHING_SORT_DATA));
	sd->order_name = ++sd0->order_name;
	sd->order_course = 0;
	sd->order_style = 0;
	ec = sd0->ec;

	tc = edb_tuple_seek_column(ec, teaching, "@.course");
	if(tc && tc->datum) {
		EdbDatum	program_datum = tc->datum;

		if(program_datum && DatumEIDisValid(program_datum)) {
			eid_t	program_eid = DatumEID(program_datum);
			struct LECTURE_COURSE_to_NAME	*lc2n;
			EdbTupleInfo	course;

			if((lc2n = edb_catalogue_lookup(ca_erd_lecture_course_to_name, program_eid))!=NULL) sd->order_course = lc2n->order;
			if(sd->order_course==0) {
				course = edb_tuple_get(ec, DatumEID(program_datum), 0);

				if(course) {
					EdbTupleColumn	course_tc = edb_tuple_seek_column(ec, course, "@.course");

					if(course_tc && course_tc->datum) {
						EdbDatum	course_datum = course_tc->datum;

						if(DatumEIDisValid(course_datum)) {
							eid_t	course_eid = DatumEID(course_datum);
							struct LECTURE_COURSE_to_NAME	*lc2n;

							if((lc2n = edb_catalogue_lookup(ca_erd_lecture_course_to_name, course_eid))!=NULL) sd->order_course = lc2n->order;
						}
					}
				}
			}
		}
	}
	tc = edb_tuple_seek_column(ec, teaching, "@.style");
	if(tc && tc->datum) {
		EdbDatum	program_datum = tc->datum;

		if(program_datum && DatumEIDisValid(program_datum)) {
			eid_t	program_eid = DatumEID(program_datum);
			struct LECTURE_STYLE_to_NAME	*ls2n;
			EdbTupleInfo	style;

			if((ls2n = edb_catalogue_lookup(ca_erd_lecture_style_to_name, program_eid))!=NULL) sd->order_style = ls2n->order;
			if(sd->order_style==0) {
				style = edb_tuple_get(ec, DatumEID(program_datum), 0);

				if(style) {
					EdbTupleColumn	style_tc = edb_tuple_seek_column(ec, style, "@.style");

					if(style_tc && style_tc->datum) {
						EdbDatum	style_datum = style_tc->datum;

						if(DatumEIDisValid(style_datum)) {
							eid_t	style_eid = DatumEID(style_datum);
							if((ls2n = edb_catalogue_lookup(ca_erd_lecture_style_to_name, style_eid))!=NULL) sd->order_style = ls2n->order;
						}
					}
				}
			}
		}
	}
	if(sd->order_course == 0) sd->order_course = 1000000;
	if(sd->order_style == 0) sd->order_style = 1000000;
	return sd;
}

static
int
teaching_compare(LinkObj o1, LinkObj o2)
{
	struct TEACHING_SORT_DATA	*sd1 = o1->aux;
	struct TEACHING_SORT_DATA	*sd2 = o2->aux;

	if(sd1->order_course < sd2->order_course) return -1;
	if(sd1->order_course > sd2->order_course) return 1;
	if(sd1->order_style < sd2->order_style) return -1;
	if(sd1->order_style > sd2->order_style) return 1;
	if(sd1->order_name < sd2->order_name) return -1;
	if(sd1->order_name > sd2->order_name) return 1;
	return 0;
}

static
EdbCatalogue
erd_get_person_teaching(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;
	struct TEACHING_SORT_DATA	sd0;

	condition = edb_condition_make(ec, erd_table_teaching, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.teacher", person),
		ConditionIntegerEQ("@.year", ERD_THIS_YEAR(ec)),
		ConditionTerminate);
	ca = erd_catalogue_get(ec, erd_table_teaching, condition);
	FREE(condition);

	bzero(&sd0, sizeof(struct TEACHING_SORT_DATA));
	sd0.ec = ec;
	ca->link = link_sort(ca->link, teaching_compare, teaching_sort_aux, (void *)&sd0, free);
	return ca;
}

static
EdbCatalogue
erd_get_person_study(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = edb_condition_make(ec, erd_table_study, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.researcher", person),
		ConditionTerminate);
	ca = erd_catalogue_get(ec, erd_table_study, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
erd_get_person_asa(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = edb_condition_make(ec, erd_table_asa, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.person", person),
		ConditionTerminate);
	ca = erd_catalogue_get(ec, erd_table_asa, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
erd_get_person_sa(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = edb_condition_make(ec, erd_table_sa, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.person", person),
		ConditionTerminate);
	ca = erd_catalogue_get(ec, erd_table_sa, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
erd_get_person_posthistory(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = edb_condition_make(ec, erd_table_posthistory, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.person", person),
		ConditionTerminate);
	ca = erd_catalogue_get(ec, erd_table_posthistory, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
erd_get_person_prize(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = edb_condition_make(ec, erd_table_prize, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.awardee", person),
		ConditionTerminate);
	ca = erd_catalogue_get(ec, erd_table_prize, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
erd_get_person_article_by_kind(EdbContext ec, eid_t person, eid_t kind)
{
	char	*condition = NULL;

	condition = edb_condition_make(ec, erd_table_article, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.author", person),
		ConditionEIDMatch("@.kind", kind),
		ConditionTerminate);
	return erd_catalogue_get(ec, erd_table_article, condition);
}

static
EdbCatalogue
erd_get_person_patent_by_kind(EdbContext ec, eid_t person, eid_t kind)
{
	char	*condition = NULL;

	condition = edb_condition_make(ec, erd_table_patent, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.inventor", person),
		ConditionEIDMatch("@.kind", kind),
		ConditionTerminate);
	return erd_catalogue_get(ec, erd_table_patent, condition);
}

static
int
erd_person_regist_keyword(EdbContext ec, EdbTupleInfo tuple, int layer, EdbTupleColumn tc, EdbDatum datum, EdbTupleCBData cbd)
{
	ErdPage	page = cbd->user_data;

#if 0
	if(!strcmp(ColumnXN(tc->ci), "study.field") || !strcmp(ColumnXN(tc->ci), "study.keyword")) erd_keyword_regist(ec, datum, page);
#else
	if(!strcmp(ColumnXN(tc->ci), "study.keyword")) erd_keyword_regist(ec, datum, page);
#endif
	return TRUE;
}

static
PersonInfo
erd_get_person_info(ErdPage profile_page, EdbContext ec, EdbTupleInfo person)
{
	EdbTupleColumn	tc;
	EdbDatum			d;
	PersonInfo		pi;
	int				i;
	ucs4_t			*u;
	Link				l, l_en, l_ja, l_pr, lw;
	char				*s;
	EdbCatalogue		ca_ph;
	char				*picdir = NULL, *picpath = NULL;
	int				inc_x, inc_y;

	pi = person_info_new(ec);
	pi->tuple = person;

	l_en = l_ja = l_pr = NULL;
	tc = edb_tuple_seek_column(ec, person, "@.surname");
	if(tc && (d=tc->datum)) {
		if(DatumEnglishIsValid(d)) {
			LinkAppend(l_en, link_atom_new(DatumEnglish(d), NULL));
		} else if(DatumJapaneseIsValid(d)) {
			LinkAppend(l_en, link_atom_new(DatumJapanese(d), NULL));
		}
		if(DatumJapaneseIsValid(d)) {
			LinkAppend(l_ja, link_atom_new(DatumJapanese(d), NULL));
		} else if(DatumEnglishIsValid(d)) {
			LinkAppend(l_ja, link_atom_new(DatumEnglish(d), NULL));
		}
		if(DatumPronounceIsValid(d)) LinkAppend(l_pr, link_atom_new(DatumPronounce(d), NULL));
	}
	LinkAppend(l_en, link_atom_new(", ", NULL));
	LinkAppend(l_ja, link_atom_new(" ", NULL));
	LinkAppend(l_pr, link_atom_new(" ", NULL));

	tc = edb_tuple_seek_column(ec, person, "@.givenname");
	if(tc && (d=tc->datum)) {
		if(DatumEnglishIsValid(d)) {
			LinkAppend(l_en, link_atom_new(DatumEnglish(d), NULL));
		} else if(DatumJapaneseIsValid(d)) {
			LinkAppend(l_en, link_atom_new(DatumJapanese(d), NULL));
		}
		if(DatumJapaneseIsValid(d)) {
			LinkAppend(l_ja, link_atom_new(DatumJapanese(d), NULL));
		} else if(DatumEnglishIsValid(d)) {
			LinkAppend(l_ja, link_atom_new(DatumEnglish(d), NULL));
		}
		if(DatumPronounceIsValid(d)) LinkAppend(l_pr, link_atom_new(DatumPronounce(d), NULL));
	}

	LinkAppend(l_en, link_atom_new(" ", NULL));
	LinkAppend(l_ja, link_atom_new(" ", NULL));
	LinkAppend(l_pr, link_atom_new(" ", NULL));

	tc = edb_tuple_seek_column(ec, person, "@.middlename");
	if(tc && (d=tc->datum)) {
		if(DatumEnglishIsValid(d)) {
			LinkAppend(l_en, link_atom_new(DatumEnglish(d), NULL));
		} else if(DatumJapaneseIsValid(d)) {
			LinkAppend(l_en, link_atom_new(DatumJapanese(d), NULL));
		}
		if(DatumJapaneseIsValid(d)) {
			LinkAppend(l_ja, link_atom_new(DatumJapanese(d), NULL));
		} else if(DatumEnglishIsValid(d)) {
			LinkAppend(l_ja, link_atom_new(DatumEnglish(d), NULL));
		}
		if(DatumPronounceIsValid(d)) LinkAppend(l_pr, link_atom_new(DatumPronounce(d), NULL));
	}

	s = str_concatenate0(l_en);	link_free(l_en);	str_oneline(s);	pi->name_en = strdup(s);	FREE(s);
	s = str_concatenate0(l_ja);	link_free(l_ja);	str_oneline(s);	pi->name_ja = strdup(s);	FREE(s);
	s = str_concatenate0(l_pr);	link_free(l_pr);	str_oneline(s);	pi->name_pr = strdup(s);	FREE(s);

	u = UTF8toUCS4(pi->name_pr);
	pronounce_convert_order(u);
	pi->order_ja = ucs4_strdup(u);

	if(str_isascii(pi->name_en)) {
		u = UTF8toUCS4(pi->name_en);
	} else {
		u = UTF8toUCS4("");
	}
	ucs4_str_capitalize1(u);
	pi->order_en = ucs4_strdup(u);

	pi->ca_affiliate = erd_get_person_affiliate(ec, TupleEID(pi->tuple));
	pi->ca_teaching = erd_get_person_teaching(ec, TupleEID(pi->tuple));
	pi->ca_study = erd_get_person_study(ec, TupleEID(pi->tuple));
	pi->ca_asa = erd_get_person_asa(ec, TupleEID(pi->tuple));
	pi->ca_sa_and_ph = erd_get_person_sa(ec, TupleEID(pi->tuple));
	ca_ph = erd_get_person_posthistory(ec, TupleEID(pi->tuple));
	ForCatalogueLink(l, ca_ph) {
		EdbTupleInfo	tuple = CatalogueObject(l);

		edb_catalogue_append(pi->ca_sa_and_ph, TupleEID(tuple), tuple);
	}
	edb_catalogue_free(ca_ph);
	pi->ca_prize = erd_get_person_prize(ec, TupleEID(pi->tuple));
	pi->ca_history_a = erd_get_person_history(ec, TupleEID(pi->tuple), erd_history_academic_kind);	/* 学歴 */
	pi->ca_history_w = erd_get_person_history(ec, TupleEID(pi->tuple), erd_history_work_kind);	/* 職歴 */

	pi->has_work = FALSE;
	i = 0;
	ForCatalogueLink(lw, ca_erd_person_work_list) {
		struct PERSON_WORK_LIST	*wl = CatalogueObject(lw);

		if(!strcmp(wl->tablename, EDB_TABLE_Article_XN)) {
			pi->ca_work[i] = erd_get_person_article_by_kind(ec, TupleEID(pi->tuple), wl->kind);
			pi->ca_work[i]->link = link_reverse(pi->ca_work[i]->link);
		} else if(!strcmp(wl->tablename, EDB_TABLE_Patent_XN)) {
			pi->ca_work[i] = erd_get_person_patent_by_kind(ec, TupleEID(pi->tuple), wl->kind);
			pi->ca_work[i]->link = link_reverse(pi->ca_work[i]->link);
		}
		if(edb_catalogue_size(pi->ca_work[i])>0) pi->has_work = TRUE;

#if 0
		ForCatalogueLink(l, pi->ca_work[i]) {
			EdbTupleInfo	tuple = CatalogueObject(l);
			EdbTupleColumn	tc;
			EdbDatum	datum;

			tc = edb_tuple_seek_column(ec, tuple, "@.keyword");
			ForTCDatum(datum, tc) {
				ucs4_t	*ja, *pr;

				if(datum==NULL) continue;
				if(!DatumJapaneseIsValid(datum)) continue;
				if(!DatumPronounceIsValid(datum)) continue;
				ja = UTF8toUCS4(DatumJapanese(datum));
				pr = UTF8toUCS4(DatumPronounce(datum));
				dict_register(ja, pr);
			}
		}
#endif
		++i;
	}

	/* portrait */
	asprintf(&picpath, "/tmp/edb_erd.%d.%d", getpid(), TupleEID(pi->tuple));
	asprintf(&picdir, "%s/%d", TABLE_XN(TupleTable(person)), TupleEID(person));
	makedir_hierarchy(picdir);

	tc = edb_tuple_seek_column(ec, pi->tuple, "@.portrait");
	inc_x = inc_y = 0;
	if(tc && (d=tc->datum) && DatumEIDisValid(d) && !edb_catalogue_lookup(ca_erd_omit_eid, DatumEID(d))) {
		pi->portrait = edb_tuple_get(ec, DatumEID(d), 0);
		if(pi->portrait) pi->picture = edb_picture_new(ec, pi->portrait);
		if(pi->picture) {
			if(!strcmp(pi->picture->mimetype, "image/jpeg")) {
				pi->portrait_name = "portrait.jpg";
			} else if(!strcmp(pi->picture->mimetype, "image/png")) {
				pi->portrait_name = "portrait.png";
			}
			if(pi->portrait_name) {
				FILE	*fp;

				fp = fopen(picpath, "w");
				if(fp) {
					fwrite(pi->picture->body, sizeof(char), pi->picture->bytes, fp);
					fclose(fp);
					pi->portrait_is_set = TRUE;
				}
			}
			if(pi->picture->xsize*4<pi->picture->ysize*3) {
				inc_x = pi->picture->ysize*3/4 - pi->picture->xsize;
				inc_x = max(0, inc_x);
			} else if(pi->picture->xsize*4>pi->picture->ysize*3) {
				inc_y = pi->picture->xsize*4/3 - pi->picture->ysize;
				inc_y = max(0, inc_y);
			}
		}
	}
	if(access(picpath, F_OK)) {
		char	*cmd = NULL;

		asprintf(&cmd, "cp %s %s", ICON_BROKEN_PORTRAIT, picpath);
		system(cmd);
		FREE(cmd);
	}
	if(!access(picpath, F_OK)) {
		char	*cmd = NULL;

		pi->portrait_name = "portrait.jpg";
		asprintf(&pi->portrait_thumbnail, "%d.jpg", TupleEID(person));
		asprintf(&cmd, "../make-portrait.sh %s %s %s %d %d %d %d &",
			picpath, picdir, pi->portrait_thumbnail, inc_x/2, inc_x - inc_x/2, 0, inc_y);
		system(cmd);
		FREE(cmd);
	}
	FREE(picpath);
	FREE(picdir);

	/* keyword */
	if(pi->ca_study && edb_catalogue_size(pi->ca_study)>0) {
		ForCatalogueLink(l, pi->ca_study) edb_tuple_TCD_callback(ec, CatalogueObject(l), NULL, erd_person_regist_keyword, profile_page);
	}
	profile_page->info = pi;
	return pi;
}

#define	M_BULLET	0x0001
#define	M_WIDE		0x0002
#define	M_SMALL		0x0004
#define	M_NARROW	0x0008
#define	M_NOEDBLINK	0x0010

static
int
erd_catalogue_print_in_table(EdbPrint ep, char *prefix, EdbCatalogue ca, int (*print_func) __P((EdbPrint, EdbTupleInfo)), int mode, char *postfix)
{
	Doc	doc = ep->doc;
	int	status = FALSE;
	int	n = 1;

	if(DocIsHTML(doc)) {
		if(ca && edb_catalogue_size(ca)>0) {
			Link	l;

			if(STRisVALID(prefix)) doc_puts(doc, prefix);
			doc_puts(doc, "<table border=\"0\" bgcolor=\"white\"");
			if(mode&M_WIDE) doc_puts(doc, " width=\"100%\"");
			if(mode&M_SMALL) doc_puts(doc, " class=listing");
			doc_puts(doc, ">\n");
			ForCatalogueLink(l, ca) {
				EdbTupleInfo	tuple = CatalogueObject(l);

				doc_printf(doc, "<tr valign=\"top\" bgcolor=\"%s\">", n%2? ITEM_BGC1 : ITEM_BGC2);
				if(mode&M_BULLET) doc_puts(doc, "<td align=\"right\" width=\"1\">○</td>");
				doc_printf(doc, "<td align=\"left\"%s>", (mode&M_NARROW)?" width=\"1\"":"");
				(*print_func)(ep, tuple);
				doc_puts(doc, "</td>");
				if(!(mode&M_NOEDBLINK)) {
					doc_puts(doc, "<td width=\"1\">");
					JavaScript_printf(doc, "edb_ref_by_eid(%d,'button3');", TupleEID(tuple));
					doc_puts(doc, "</td>");
				}
				doc_puts(doc, "</tr>");
				++n;
			}
			doc_puts(doc, "</table>\n");
			if(STRisVALID(postfix)) doc_puts(doc, postfix);
			status = TRUE;
		} else if(erd_preview) {
			doc_puts(doc, "<span style=\"color : " VANISH_FGC "\">");
			if(STRisVALID(prefix)) doc_puts(doc, prefix);
			doc_puts(doc, "( <i>not registered</i> )");
			if(STRisVALID(postfix)) doc_puts(doc, postfix);
			doc_puts(doc, "</span>");
		}
	} else if(DocIsTeX(doc)) {
		if(STRisVALID(prefix)) doc_puts(doc, prefix);
		if(ca && edb_catalogue_size(ca)>0) {
			Link	l;

			ForCatalogueLink(l, ca) {
				EdbTupleInfo	tuple = CatalogueObject(l);

				if(mode&M_BULLET) doc_puts(doc, "\\BULLET{}");
				(*print_func)(ep, tuple);
				doc_puts(doc, " %\n");
				++n;
			}
			status = TRUE;
		} else {
			doc_puts(doc, "{\\NONE}");
		}
		if(STRisVALID(postfix)) doc_puts(doc, postfix);
	}
	return status;
}

static
int
erd_person_work_print_all(EdbContext ec, PersonInfo pi, EdbPrint profile_ep, EdbPrint ep)
{
	int	status = FALSE;
	int	i;
	Link	lw;

	edb_print_push(ep);
	if(EpLang(ep)!=EP_LANG_English) EpLang(ep) = EP_LANG_Auto;
	i = 0;
	ForCatalogueLink(lw, ca_erd_person_work_list) {
		struct PERSON_WORK_LIST	*wl = CatalogueObject(lw);
		char	*cs;

		if(edb_catalogue_size(pi->ca_work[i])>0) {
			if(DocIsHTML(ep->doc)) {
				asprintf(&cs, "<dt><a name=\"%s\">%s</a>:</dt>\n<dd>",
					wl->anchor,
					BILINGUAL(EpLang(ep)!=EP_LANG_English, wl->caption_ja, wl->caption_en));
				status |= erd_catalogue_print_in_table(ep, cs, pi->ca_work[i], edb_print_tuple, M_BULLET|M_WIDE|M_SMALL, "</dd>\n");
				doc_printf(profile_ep->doc, "<option value=\"%s\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%s</option>\n",
					wl->anchor,
					BILINGUAL(EpLang(ep)!=EP_LANG_English, wl->caption_ja, wl->caption_en));
				FREE(cs);
			} else if(DocIsTeX(ep->doc)) {
				asprintf(&cs, "(%s)", BILINGUAL(EpLang(ep)!=EP_LANG_English, wl->caption_ja, wl->caption_en));
				status |= erd_catalogue_print_in_table(ep, cs, pi->ca_work[i], edb_print_tuple, M_BULLET|M_WIDE|M_SMALL, "%\n");
				FREE(cs);
			}
		}
		++i;
	}
	edb_print_pop(ep);
	return status;
}

static
int
erd_person_work_print_num(EdbContext ec, PersonInfo pi, EdbPrint profile_ep, EdbPrint ep)
{
	int	status = FALSE;
	int	i;
	Link	lw;

	edb_print_push(ep);
	if(EpLang(ep)!=EP_LANG_English) EpLang(ep) = EP_LANG_Auto;
	i = 0;
	ForCatalogueLink(lw, ca_erd_person_work_list) {
		struct PERSON_WORK_LIST	*wl = CatalogueObject(lw);
		char	*cs;

		if(edb_catalogue_size(pi->ca_work[i])>0) {
			if(DocIsHTML(ep->doc)) {
			} else if(DocIsTeX(ep->doc)) {
				doc_text_puts3(ep->doc, "\\著作件数{", BILINGUAL(EpLang(ep)!=EP_LANG_English, wl->caption_ja, wl->caption_en), "}{");
				doc_printf(ep->doc, "%d}", edb_catalogue_size(pi->ca_work[i]));
			}
		}
		++i;
	}
	edb_print_pop(ep);
	return status;
}

static
char *
erd_person_portrait_name(EdbContext ec, PersonInfo pi)
{
	char	*pname = NULL;

	if(pi->picture) {
		if(!strcmp(pi->picture->mimetype, "image/jpeg")) {
			pname = "portrait.jpg";
		} else if(!strcmp(pi->picture->mimetype, "image/png")) {
			pname = "portrait.png";
		}
	}
	return pname;
}

static
void
erd_person_print_name(EdbPrint ep, PersonInfo pi)
{
	// EdbContext ec = ep->ec;
	Doc	doc = ep->doc;

	if(DocIsHTML(doc)) {
		doc_puts(doc, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\">");
		if(EpLang(ep)==EP_LANG_English) {
			doc_puts(doc, "<td>Name</td><td><b>");
		} else {
			doc_puts(doc, "<td>氏名</td><td><b>");
			doc_text_puts(doc, pi->name_ja);
			doc_text_puts3(doc, " ( ", pi->name_pr, " )");
			doc_puts(doc, "<br />\n");
		}
		doc_text_puts(doc, pi->name_en);
		doc_puts(doc, "</b></td></tr>\n");
	} else if(DocIsTeX(doc)) {
		doc_text_puts3(doc, "\\DEF{名前:日}{", pi->name_ja, "}%\n");
		doc_text_puts3(doc, "\\DEF{名前:読}{", pi->name_pr, "}%\n");
		doc_text_puts3(doc, "\\DEF{名前:英}{", pi->name_en, "}%\n");
	}
}

static
int
print_date(EdbPrint ep, EdbTupleInfo tuple, EdbDatum super_datum)
{
	register int	status;

	if(super_datum==NULL) return FALSE;
	status = edb_print_tuple_named_column(ep, tuple, super_datum, " (", "", "@.date", NULL, ", ", ")", "", EP_ALERT);
	return status;
}

static
int
print_nod(EdbPrint ep, EdbTupleInfo tuple, EdbDatum super_datum)
{
	register int	status;

	if(super_datum==NULL) return FALSE;
	if(EpUserData(ep)) {
		PersonInfo	pi = EpUserData(ep);

		if( !edb_tuple_seek_TC_by_xmlname(ep->ec, tuple, super_datum->super, super_datum, "@.conferrer")
		|| !edb_tuple_seek_TC_by_xmlname(ep->ec, tuple, super_datum->super, super_datum, "@.date")) {
			pi->nod_is_set = FALSE;
		}
	}
	status = edb_print_tuple_named_column(ep, tuple, super_datum, " (", "", "@.conferrer", NULL, ", ", ")", NULL, EP_ALERT);
	status = edb_print_tuple_named_column(ep, tuple, super_datum, " (", "", "@.date", NULL, ", ", ")", NULL, EP_ALERT);
	return status;
}

static
int
print_period(EdbPrint ep, EdbTupleInfo tuple, EdbDatum super_datum)
{
	register int	status;

	if(super_datum==NULL) return FALSE;
	status = edb_print_tuple_named_column(ep, tuple, super_datum, " [", "", "@.period", NULL, ", ", "]", "", EP_ALERT);
	return status;
}

static
int
print_lecture(EdbPrint ep, EdbTupleInfo lecture)
{
	EdbTupleColumn	tc;

#if 0
	edb_print_push(ep);
	ep->ct->tcd_mode |= EP_TCD_MODE_BOLD;
	edb_print_tuple_named_column(ep, lecture, NULL, "", "", "@.name", NULL, ", ", "", NULL, EP_ALERT);
	edb_print_pop(ep);
#endif

	tc = edb_tuple_seek_column(ep->ec, lecture, "@.course");
	if(tc && tc->datum) {
		EdbDatum	datum = tc->datum;

		if(DatumEIDisValid(datum)) {
			EdbTupleInfo	program = edb_tuple_get(ep->ec, DatumEID(datum), 0);

			if(program) {
				EdbTupleColumn	program_tc = edb_tuple_seek_column(ep->ec, program, "@.course");
				int	printed = FALSE;

				if(program_tc && program_tc->datum) {
					EdbDatum	program_datum = program_tc->datum;

					if(program_datum && DatumEIDisValid(program_datum)) {
						eid_t	program_eid = DatumEID(program_datum);
						struct LECTURE_COURSE_to_NAME	*lc2n;

						if((lc2n = edb_catalogue_lookup(ca_erd_lecture_course_to_name, program_eid))!=NULL) {
							doc_text_puts3(ep->doc, " (", BILINGUAL(EpLang(ep)!=EP_LANG_English, lc2n->name_ja, lc2n->name_en), ")");
							printed = TRUE;
						}
					}
				}
				if(!printed) {
					edb_print_tuple_named_column(ep, program, NULL, " (", "", "@.course", NULL, ", ", ")", "", 0);
					printed = TRUE;
				}
			}
		}
	}
	return TRUE;
}

static
int
print_teaching(EdbPrint ep, EdbTupleInfo teaching)
{
	EdbTupleColumn	tc;
	int	printed = FALSE;

	edb_print_push(ep);
	ep->ct->tcd_mode |= EP_TCD_MODE_BOLD;
	edb_print_tuple_named_column(ep, teaching, NULL, "", "", "@.subject", NULL, ", ", "", NULL, EP_ALERT);
	edb_print_pop(ep);

	tc = edb_tuple_seek_column(ep->ec, teaching, "@.course");
	if(tc && tc->datum) {
		EdbDatum	program_datum = tc->datum;

		if(program_datum && DatumEIDisValid(program_datum)) {
			eid_t	program_eid = DatumEID(program_datum);
			struct LECTURE_COURSE_to_NAME	*lc2n;

			if((lc2n = edb_catalogue_lookup(ca_erd_lecture_course_to_name, program_eid))!=NULL) {
				doc_text_puts3(ep->doc, " (", BILINGUAL(EpLang(ep)!=EP_LANG_English, lc2n->name_ja, lc2n->name_en), ")");
				printed = TRUE;
			}
#if 1
			if(!printed) {
				EdbTupleInfo	course = edb_tuple_get(ep->ec, DatumEID(program_datum), 0);

				if(course) {
					EdbTupleColumn	course_tc = edb_tuple_seek_column(ep->ec, course, "@.course");

					if(course_tc && course_tc->datum) {
						EdbDatum	course_datum = course_tc->datum;

						if(DatumEIDisValid(course_datum)) {
							eid_t	course_eid = DatumEID(course_datum);
							struct LECTURE_COURSE_to_NAME	*lc2n;

							if((lc2n = edb_catalogue_lookup(ca_erd_lecture_course_to_name, course_eid))!=NULL) {
								doc_text_puts3(ep->doc, " (", BILINGUAL(EpLang(ep)!=EP_LANG_English, lc2n->name_ja, lc2n->name_en), ")");
								printed = TRUE;
							}
						}
					}
				}
			}
#endif
		}
		if(!printed) edb_print_tuple_named_column(ep, teaching, NULL, " (", "", "@.course", NULL, ", ", ")", "", 0);
	} else {
		EdbDatum	datum;

		tc = edb_tuple_seek_column(ep->ec, teaching, "@.subject");
		if(tc && (datum=tc->datum) && DatumEIDisValid(datum)) {
			EdbTupleInfo	lecture = edb_tuple_get(ep->ec, DatumEID(datum), 0);

			print_lecture(ep, lecture);
		}
	}
	return TRUE;
}

static
int
print_study1(EdbPrint ep, EdbTupleInfo study)
{

	edb_print_push(ep);
	ep->ct->tcd_mode |= EP_TCD_MODE_BOLD;
	edb_print_tuple_named_column(ep, study, NULL, "", "", "@.field", NULL, ", ", "", "", EP_ALERT);
	edb_print_pop(ep);
	return TRUE;
}

static
int
print_study2(EdbPrint ep, EdbTupleInfo study)
{

	if(DocIsHTML(ep->doc)) {
		edb_print_tuple_named_column(ep, study, NULL, "", "", "@.theme", NULL, ", ", "", "", 0);
		edb_print_push(ep);
		ep->ct->tcd_mode |= EP_TCD_MODE_BOLD;
		edb_print_tuple_named_column(ep, study, NULL, " (", "", "@.keyword", NULL, ", ", ")", "", 0);
		edb_print_pop(ep);
	} else if(DocIsTeX(ep->doc)) {
		edb_print_tuple_named_column(ep, study, NULL, "", "", "@.theme", NULL, ", ", "", "", 0);
		edb_print_push(ep);
		ep->ct->tcd_mode |= EP_TCD_MODE_BOLD;
		edb_print_tuple_named_column(ep, study, NULL, " (\\KEYWORD{", "\\KEYWORD{", "@.keyword", NULL, "}, ", "})", "", 0);
		edb_print_pop(ep);
	}
	return TRUE;
}

static
int
print_asa(EdbPrint ep, EdbTupleInfo asa)
{

	edb_print_push(ep);
	ep->ct->column_mode |=EP_COLUMN_MODE_AT_ANYTIME;
	edb_print_push(ep);
	ep->ct->tcd_mode |= EP_TCD_MODE_BOLD;
	edb_print_tuple_named_column(ep, asa, NULL, "", "", "@.academicsociety", NULL, ", ", "", NULL, EP_ALERT);
	edb_print_pop(ep);
	edb_print_tuple_named_column(ep, asa, NULL, " (", "", "@.post", print_period, ", ", ")", "", 0);
	edb_print_pop(ep);
	return TRUE;
}

static
int
print_sa_and_ph(EdbPrint ep, EdbTupleInfo tuple)
{

	edb_print_push(ep);
	ep->ct->column_mode |=EP_COLUMN_MODE_AT_ANYTIME;
	if(TABLE_EID(TupleTable(tuple)) == TABLE_EID(erd_table_sa)) {
		edb_print_push(ep);
		ep->ct->tcd_mode |= EP_TCD_MODE_BOLD;
		edb_print_tuple_named_column(ep, tuple, NULL, "", "", "@.organization", NULL, ", ", "", NULL, EP_ALERT);
		edb_print_pop(ep);
		edb_print_tuple_named_column(ep, tuple, NULL, " (", "", "@.post", print_period, ", ", ")", "", 0);
	} else if(TABLE_EID(TupleTable(tuple)) == TABLE_EID(erd_table_posthistory)) {
		edb_print_push(ep);
		ep->ct->tcd_mode |= EP_TCD_MODE_BOLD;
		edb_print_tuple_named_column(ep, tuple, NULL, "", "", "@.post", NULL, ", ", "", "", 0);
		edb_print_pop(ep);
		edb_print_tuple_named_column(ep, tuple, NULL, " (", "", "@.period", NULL, ", ", ")", "", 0);
	}
	edb_print_pop(ep);
	return TRUE;
}

static
int
print_prize(EdbPrint ep, EdbTupleInfo prize)
{

	if(DocIsHTML(ep->doc)) {
		edb_print_tuple_named_column(ep, prize, NULL, "", "", "@.date", NULL, ", ", ", ", NULL, EP_ALERT);
	} else if(DocIsTeX(ep->doc)) {
		edb_print_tuple_named_column(ep, prize, NULL, "", "", "@.date", NULL, ", ", "", NULL, EP_ALERT);
	}
	edb_print_push(ep);
	ep->ct->tcd_mode |= EP_TCD_MODE_BOLD;
	edb_print_tuple_named_column(ep, prize, NULL, " ", "", "@.name", NULL, ", ", "", "", 0);
	edb_print_pop(ep);
	edb_print_tuple_named_column(ep, prize, NULL, " (", "", "@.awarder", NULL, ", ", ")", "", 0);
	return TRUE;
}

static
int
print_history(EdbPrint ep, EdbTupleInfo history)
{
	EdbContext	ec = ep->ec;
	Doc			doc = ep->doc;
	EdbTupleColumn	tc;
	EdbDatum			datum;

	tc = edb_tuple_seek_column(ec, history, "@.date");
	if(DocIsHTML(doc)) {
		if(tc && (datum=tc->datum) && DatumEnglishIsValid(datum)) {
			int	ymd = ATOI(DatumEnglish(datum));

			doc_printf(doc, "%04d.</td>", DATE_Y(ymd));
			if(DATE_M(ymd)<10) {
				doc_printf(doc, "<td align=\"right\" width=\"1\">&nbsp;&nbsp;%d.</td>", DATE_M(ymd));
			} else {
				doc_printf(doc, "<td align=\"right\" width=\"1\">%d.</td>", DATE_M(ymd));
			}
		} else {
			doc_printf(doc, "????.</td><td align=\"right\" width=\"1\">??.</td>");
		}
		doc_puts(doc, "<td align=\"left\">");
		edb_print_tuple_named_column(ep, history, NULL, "", "", "@.event", NULL, ", ", "", NULL, EP_ALERT);
	} else if(DocIsTeX(doc)) {
		doc_puts(doc, "\t\\履歴{");
		if(tc && (datum=tc->datum) && DatumEnglishIsValid(datum)) {
			int	ymd = ATOI(DatumEnglish(datum));

			doc_printf(doc, "%04d.{\\EdbZSp}%d.", DATE_Y(ymd), DATE_M(ymd));
		} else {
			doc_printf(doc, "????.{\\EdbZSp}??.");
		}
		doc_puts(doc, "}{");
		edb_print_tuple_named_column(ep, history, NULL, "", "", "@.event", NULL, ", ", "", NULL, EP_ALERT);
		doc_puts(doc, "}%\n");
	}
	return TRUE;
}

void
erd_linkto_person_page(ErdPage page, ErdPage person_page, int mode, int japanese)
{
	EdbPrint		ep = PAGE_EP_HTML(page, japanese);
	Doc			doc = ep->doc;
	char			*dirprefix = "";
	PersonInfo	pi = person_page->info;
	EdbTupleInfo	person = pi->tuple;

	if(pi==NULL) return;
	if(mode&P_SMALL) doc_puts(doc, "<span class=listing>");
	if(page->depth==2) {
		dirprefix = "../../";
	} else if(page->depth==1) {
		dirprefix = "../";
	} else {
		dirprefix = "";
	}
	if(mode&P_PORTRAIT) {
		doc_printf(doc, "<a href=\"%s%s\" target=\"_blank\">", dirprefix, PAGE_PATH_HTML(person_page, japanese));
		if(STRisVALID(pi->portrait_thumbnail)) {
			doc_printf(doc, "<img src=\"%s%s/%s\" alt=\"Portrait\" width=\"%d\" height=\"%d\" border=\"0\" /><br />",
				dirprefix, ERD_ICONS_DIR, pi->portrait_thumbnail, PORTRAIT_ICON_WIDTH, PORTRAIT_ICON_HEIGHT);
		} else {
			doc_printf(doc, "<img src=\"%s%s\" alt=\"N/A\" width=\"%d\" height=\"%d\" border=\"0\" /><br />",
				dirprefix, ICON_BROKEN_PORTRAIT, PORTRAIT_ICON_WIDTH, PORTRAIT_ICON_HEIGHT);
		}
		if(mode&P_BOLD) doc_puts(doc, "<span style=\"font-weight : bolder\">");
		doc_text_puts(doc, PAGE_TITLE(person_page, japanese));
		if(mode&P_BOLD) doc_puts(doc, "</span>");
		doc_puts(doc, "</a>");
		if(mode&P_TITLE) edb_print_tuple_named_column(ep, person, NULL, "<br /><span class=listing2>", "", "@.title", NULL, ", ", "</span>", "", 0);
	} else {
		char	*delimiter = " : ";

		if(mode&P_InTABLE) delimiter = "</td><td>" ;
		doc_printf(doc, "<a href=\"%s%s\" target=\"_blank\">", dirprefix, PAGE_PATH_HTML(person_page, japanese));
		if(mode&P_BOLD) doc_puts(doc, "<span style=\"font-weight : bolder\">");
		doc_text_puts(doc, PAGE_TITLE(person_page, japanese));
		if(mode&P_BOLD) doc_puts(doc, "</span>");
		doc_puts(doc, "</a>");
		if(mode&P_TITLE) edb_print_tuple_named_column(ep, person, NULL, delimiter, "", "@.title", NULL, ", ", "", delimiter, 0);
		if(mode&P_AFFILIATION) edb_print_tuple_named_column(ep, person, NULL, delimiter, "", "@.affiliation", NULL, " / ", "", delimiter, 0);
	}
	if(mode&P_SMALL) doc_puts(doc, "</span>");
}

static
int
erd_person_page_work(ErdPage profile_page, ErdPage page, PersonInfo pi, EdbTupleInfo person, int japanese)
{
	EdbPrint		profile_ep = PAGE_EP_HTML(profile_page, japanese);
	EdbPrint		ep_html = PAGE_EP_HTML(page, japanese);
	EdbPrint		ep_tex = PAGE_EP_TeX(profile_page, japanese);
	EdbContext	ec = ep_html->ec;
	Doc			profile_doc = profile_ep->doc;
	Doc			doc_html = ep_html->doc;
	Doc			doc_tex = ep_tex->doc;
	Link			l;
	char			*cs;
	int			status;

	/* HTML */
	doc_puts(profile_doc, "<select onchange=\"work_frame_by_selection(this);\">\n");
	if(japanese) {
		doc_puts(profile_doc, "<option value=\"\" selected=\"selected\">フレーム表示の移動</option>\n<option value=\"\">⇒ 別ウィンドウで開く</option>\n");
	} else {
		doc_puts(profile_doc, "<option value=\"\" selected=\"selected\">Frame Controller</option>\n<option value=\"\">Open in a new window</option>\n");
	}
	edb_print_tuple_postfix_register_func(ep_html, PrintStyle(ep_html), erd_print_tuple_postfix);
	edb_print_tcd_register_func(ep_html, "person.birthday", PrintStyle(ep_html), erd_print_birthday);
	edb_print_type_register_func(ep_html, EDB_TYPE_EMAIL, PrintStyle(ep_html), erd_print_email);
	ForCatalogueLink(l, pi->ca_person) edb_print_eoi_add(ep_html, TupleEID((EdbTupleInfo)CatalogueObject(l)));

	erd_page_begin(page, japanese);

	JavaScript_printf(doc_html, "write_if_top('<h1>');");
	JavaScript_printf(doc_html, "write_if_top('%s');", BILINGUAL(japanese, pi->name_ja, pi->name_en));
	JavaScript_printf(doc_html, "write_if_top('</h1>');");
	JavaScript_printf(doc_html, "write_if_top('<hr />');");

	doc_puts(doc_html, "<dl>\n");

	asprintf(&cs, BILINGUAL3(japanese, "<dt><a name=\"lecture\">", "担当授業科目", "His Lecture", "</a></dt>\n<dd>"));
	status = erd_catalogue_print_in_table(ep_html, cs, pi->ca_teaching, print_teaching, M_BULLET|M_WIDE|M_SMALL, "</dd>\n");
	if(status) doc_printf(profile_doc, BILINGUAL3(japanese, "<option value=\"lecture\">", "⇒ 担当授業科目", "His Lecture", "</option>\n"));
	FREE(cs);

	status = erd_catalogue_print_in_table(ep_html,
		BILINGUAL3(japanese, "<dt><a name=\"study1\">", "専門分野", "Field of Study", "</a></dt>\n<dd>"),
		pi->ca_study, print_study1, M_BULLET|M_WIDE|M_SMALL, "</dd>\n");
	if(status) doc_puts(profile_doc, BILINGUAL3(japanese, "<option value=\"study1\">", "⇒ 専門分野", "Field of Study", "</option>\n"));

	status = erd_catalogue_print_in_table(ep_html,
		BILINGUAL3(japanese, "<dt><a name=\"study2\">", "研究概要", "Subject of Study", "</a></dt>\n<dd>"),
		pi->ca_study, print_study2, M_BULLET|M_WIDE|M_SMALL, "</dd>\n");
	if(status) doc_puts(profile_doc, BILINGUAL3(japanese, "<option value=\"study2\">", "⇒ 研究概要", "Subject of Study", "</option>\n"));

	if(pi->has_work) {
		doc_printf(doc_html, BILINGUAL3(japanese, "<dt><a name=\"work\">", "著書・論文", "Book / Paper", "</a></dt>\n"));
		doc_puts(profile_doc, BILINGUAL3(japanese, "<option value=\"work\">", "⇒ 著書・論文", "Book / Paper", "</option>\n"));
		status = erd_person_work_print_all(ec, pi, profile_ep, ep_html);
	} else if(erd_preview) {
		doc_printf(doc_html, BILINGUAL3(japanese, "<dt style=\"color : " VANISH_FGC "\"><a name=\"work\">", "著書・論文", "Book / Paper", "</a></dt>\n"));
		doc_puts(doc_html, "<span style=\"color : " VANISH_FGC "\"><dd>( <i>not registered</i> )</dd></span>\n");
	}

	status = erd_catalogue_print_in_table(ep_html,
		BILINGUAL3(japanese, "<dt><a name=\"academicsocietyactivity\">", "所属学会(役職)", "Activity of Academic Society (Post)", "</a></dt>\n<dd>"),
		pi->ca_asa, print_asa, M_BULLET|M_WIDE|M_SMALL, "</dd>\n");
	if(status) doc_puts(profile_doc, BILINGUAL3(japanese, "<option value=\"academicsocietyactivity\">", "⇒ 所属学会(役職)", "Activity of Academic Society (Post)", "</option>\n"));

	status = erd_catalogue_print_in_table(ep_html,
		BILINGUAL3(japanese, "<dt><a name=\"socialactivity\">", "社会・学内活動等", "Social Activity / University Management", "</a></dt>\n<dd>"),
		pi->ca_sa_and_ph, print_sa_and_ph, M_BULLET|M_WIDE|M_SMALL, "</dd>\n");
	if(status) doc_puts(profile_doc, BILINGUAL3(japanese, "<option value=\"socialactivity\">", "⇒ 社会・学内活動等", "Social Activity / University Management", "</option>\n"));

	status = erd_catalogue_print_in_table(ep_html,
		BILINGUAL3(japanese, "<dt><a name=\"prize\">", "受賞等", "Prize / Award", "</a></dt>\n<dd>"),
		pi->ca_prize, print_prize, M_BULLET|M_WIDE|M_SMALL, "</dd>\n");
	if(status) doc_printf(profile_doc, BILINGUAL3(japanese, "<option value=\"prize\">", "⇒ 受賞等", "Prize / Award", "</option>\n"));

	doc_puts(doc_html, "</dl>\n");

	doc_puts(profile_doc, "</select><br />\n");

	/* TeX */
	edb_print_tuple_postfix_register_func(ep_tex, PrintStyle(ep_tex), erd_print_tuple_postfix);
	edb_print_tcd_register_func(ep_tex, "person.birthday", PrintStyle(ep_tex), erd_print_birthday);
	edb_print_tcd_register_func(ep_tex, "prize.date", PrintStyle(ep_tex), erd_print_YM_TeX);
	edb_print_tcd_register_func(ep_tex, "socialactivity.post.period", PrintStyle(ep_tex), erd_print_PERIOD_TeX);
	edb_print_tcd_register_func(ep_tex, "posthistory.period", PrintStyle(ep_tex), erd_print_PERIOD_TeX);
	edb_print_tcd_register_func(ep_tex, "academicsocietyactivity.post.period", PrintStyle(ep_tex), erd_print_PERIOD_TeX);
//	edb_print_type_register_func(ep_tex, EDB_TYPE_EMAIL, PrintStyle(ep_tex), erd_print_email);
	ForCatalogueLink(l, pi->ca_person) edb_print_eoi_add(ep_tex, TupleEID((EdbTupleInfo)CatalogueObject(l)));

	status = erd_catalogue_print_in_table(ep_tex, "\\DEF{担当授業科目}{%\n", pi->ca_teaching, print_teaching, M_BULLET|M_WIDE|M_SMALL, "}%\n");
	status = erd_catalogue_print_in_table(ep_tex, "\\DEF{専門分野}{%\n", pi->ca_study, print_study1, M_BULLET|M_WIDE|M_SMALL, "}%\n");
	status = erd_catalogue_print_in_table(ep_tex, "\\DEF{研究概要}{%\n", pi->ca_study, print_study2, M_BULLET|M_WIDE|M_SMALL, "}%\n");
	doc_puts(doc_tex, "\\DEF{著書・論文:数}{%\n");
	if(pi->has_work) {
		status = erd_person_work_print_num(ec, pi, profile_ep, ep_tex);
	} else {
		doc_puts(doc_tex, "{\\NONE}");
	}
	doc_puts(doc_tex, "}%\n");
	doc_puts(doc_tex, "\\DEF{著書・論文}{%\n");
/*
	if(pi->has_work) {
		status = erd_person_work_print_all(ec, pi, profile_ep, ep_tex);
	} else {
		doc_puts(doc_tex, "{\\NONE}");
	}
*/
	doc_puts(doc_tex, "}%\n");
	doc_printf(doc_tex, "\\DEF{所属学会(役職):数}{%d}%%\n", edb_catalogue_size(pi->ca_asa));
	doc_printf(doc_tex, "\\DEF{社会・学内活動等:数}{%d}%%\n", edb_catalogue_size(pi->ca_sa_and_ph));
	doc_printf(doc_tex, "\\DEF{受賞等:数}{%d}%%\n", edb_catalogue_size(pi->ca_prize));
	status = erd_catalogue_print_in_table(ep_tex, "\\DEF{所属学会(役職)}{%\n", pi->ca_asa, print_asa, M_BULLET|M_WIDE|M_SMALL, "}%\n");
	status = erd_catalogue_print_in_table(ep_tex, "\\DEF{社会・学内活動等}{%\n", pi->ca_sa_and_ph, print_sa_and_ph, M_BULLET|M_WIDE|M_SMALL, "}%\n");
	status = erd_catalogue_print_in_table(ep_tex, "\\DEF{受賞等}{", pi->ca_prize, print_prize, M_BULLET|M_WIDE|M_SMALL, "}%\n");

	erd_page_end(page, japanese);
	return TRUE;
}

static
int
erd_print_person_column(EdbPrint ep, EdbTupleInfo tuple, char *prefix, char *dprefix, char *xn, EdbPrintTupleColumnCallbackFunc_t print, char *dpostfix, char *postfix, int alert, int *set)
{
	char	*pre, *post, *substitute;
	EdbTupleColumn	tc;
	int	status = FALSE;

	if(set) *set = FALSE;
	tc = edb_tuple_seek_column(ep->ec, tuple, xn);
	if(DocIsHTML(ep->doc)) {
		if(tc && tc->datum) {
			asprintf(&pre, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\"><td>%s</td><td>", prefix);
			asprintf(&post, "%s</td></tr>\n", postfix);
			if(set) *set = TRUE;
		} else if(erd_preview) {
			asprintf(&pre, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\" style=\"color : " VANISH_FGC "\"><td>%s</td><td>", prefix);
			asprintf(&post, "%s</td></tr>\n", postfix);
		} else {
			return FALSE;
		}
		asprintf(&substitute, "%s( <i>not registered</i> )%s", pre, post);
		status = edb_print_tuple_named_column(ep, tuple, NULL, pre, dprefix, xn, print, dpostfix, post, substitute, alert);
		FREE(pre);
		FREE(post);
		FREE(substitute);
	} else if(DocIsTeX(ep->doc)) {
		if(tc && tc->datum) {
			asprintf(&pre, "\\DEF{%s}{", prefix);
			asprintf(&post, "%s}%%\n", postfix);
			if(set) *set = TRUE;
		} else {
			asprintf(&pre, "\\DEF{%s}{", prefix);
			asprintf(&post, "%s}%%\n", postfix);
		}
		asprintf(&substitute, "%s{\\NONE}%s", pre, post);
		status = edb_print_tuple_named_column(ep, tuple, NULL, pre, dprefix, xn, print, dpostfix, post, substitute, alert);
		FREE(pre);
		FREE(post);
		FREE(substitute);
	}
	return status;
}

static
int
erd_print_person_column2(EdbPrint ep, EdbTupleInfo tuple, char *prefix, char *dprefix, char *xn, EdbPrintTupleColumnCallbackFunc_t print, char *dpostfix, char *postfix, int alert, int *set)
{
	char	*pre, *post, *substitute;
	EdbTupleColumn	tc;
	int	status;

	if(set) *set = FALSE;
	tc = edb_tuple_seek_column(ep->ec, tuple, xn);
	if(tc && tc->datum) {
		asprintf(&pre, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\"><td>%s</td><td colspan=\"2\">", prefix);
		asprintf(&post, "%s</td></tr>\n", postfix);
		if(set) *set = TRUE;
	} else if(erd_preview) {
		asprintf(&pre, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\" style=\"color : " VANISH_FGC "\"><td>%s</td><td colspan=\"2\">", prefix);
		asprintf(&post, "%s</td></tr>\n", postfix);
	} else {
		return FALSE;
	}
	asprintf(&substitute, "%s( <i>not registered</i> )%s", pre, post);
	status = edb_print_tuple_named_column(ep, tuple, NULL, pre, dprefix, xn, print, dpostfix, post, substitute, alert);
	FREE(pre);
	FREE(post);
	FREE(substitute);
	return status;
}

static
int
erd_person_page_body(EdbContext ec, ErdPage profile_page, ErdPage work_page, int japanese)
{
	EdbPrint		ep_html = PAGE_EP_HTML(profile_page, japanese);
	Doc			doc_html = ep_html->doc;
	EdbPrint		ep_tex = PAGE_EP_TeX(profile_page, japanese);
	Doc			doc_tex = ep_tex->doc;
	PersonInfo	pi = profile_page->info;
 	Link			l;
	EdbTupleInfo	person = pi->tuple;
	// eid_t			eid = TupleEID(person);
	EdbTupleColumn	tc;
	EdbDatum			d;
	char		*portrait_name = "../../" ICON_BROKEN_PORTRAIT;
	EdbCatalogue	ca_aff_printed;

	/* HTML */
	edb_print_tuple_postfix_register_func(ep_html, PrintStyle(ep_html), erd_print_tuple_postfix);
	edb_print_tcd_register_func(ep_html, "person.birthday", PrintStyle(ep_html), erd_print_birthday);
	edb_print_type_register_func(ep_html, EDB_TYPE_EMAIL, PrintStyle(ep_html), erd_print_email);
	edb_print_type_register_func(ep_html, EDB_TYPE_PHONE, PrintStyle(ep_html), erd_print_phone);
	ForCatalogueLink(l, pi->ca_person) edb_print_eoi_add(ep_html, TupleEID((EdbTupleInfo)CatalogueObject(l)));

	/* TeX */
	edb_print_tuple_postfix_register_func(ep_tex, PrintStyle(ep_tex), erd_print_tuple_postfix);
	edb_print_tcd_register_func(ep_tex, "person.birthday", PrintStyle(ep_tex), erd_print_birthday);
	edb_print_tcd_register_func(ep_tex, "person.officialnameofdegree.date", PrintStyle(ep_tex), erd_print_DateYM_TeX);
//	edb_print_type_register_func(ep_tex, EDB_TYPE_EMAIL, PrintStyle(ep_tex), erd_print_email);
	edb_print_type_register_func(ep_tex, EDB_TYPE_PHONE, PrintStyle(ep_tex), erd_print_phone);
	ForCatalogueLink(l, pi->ca_person) edb_print_eoi_add(ep_tex, TupleEID((EdbTupleInfo)CatalogueObject(l)));

	erd_page_begin(profile_page, japanese);

	if(STRisVALID(pi->portrait_name)) portrait_name = pi->portrait_name;

	/* TeX */
	doc_puts(doc_tex, "\\begin{PROFILE}\n");

	/* HTML */
	doc_puts(doc_html, "<table border=\"0\" width=\"100%\">\n");
	doc_puts(doc_html, "<tr valign=\"top\"><td colspan=\"2\" class=affiliation>\n");
	erd_page_menu(profile_page, japanese, TRUE);
//	doc_text_puts3(doc_html, "<a href=\"../../index.html\">", BILINGUAL(japanese, NAME_OF_CONTENTS_ja, NAME_OF_CONTENTS_en), "</a>");
	doc_text_puts(doc_html, " > ");

	ca_aff_printed = edb_catalogue_new();
	tc = edb_tuple_seek_column(ec, person, "@.affiliation");
	d = NULL;
	doc_puts(doc_tex, "\\DEF{所属}{");
	if(tc) {
		ForTCDatum(d, tc) {
			EdbTupleInfo	otuple;
			int	size = edb_catalogue_size(ca_aff_printed);
			if(!DatumEIDisValid(d)) continue;
			if(!edb_datum_at_present(ec, d)) continue;
			otuple = edb_tuple_get(ec, DatumEID(d), 0);
			if(otuple==NULL) continue;
			if(!erd_page_dir_exist(ec, otuple)) continue;
			/* HTML */
			if(size>0) doc_puts(doc_html, "<br /><span class=affiliation2>");
			erd_organization_print_upper(ep_html, TupleEID(otuple));
			if(size>0) doc_puts(doc_html, "</span>");
			/* TeX */
			doc_printf(doc_tex, size==0 ? "\\所属P{%d}{":"\\所属S{%d}{", TupleEID(otuple));
			erd_organization_print_upper(ep_tex, TupleEID(otuple));
			doc_puts(doc_tex, "}");
			edb_catalogue_append(ca_aff_printed, TupleEID(otuple), otuple);
		}
	}
	if(pi->ca_affiliate!=NULL && edb_catalogue_size(pi->ca_affiliate)>0) {
		ForCatalogueLink(l, pi->ca_affiliate) {
			EdbTupleInfo	otuple = CatalogueObject(l);
			int	size = edb_catalogue_size(ca_aff_printed);

			if(edb_catalogue_lookup(ca_aff_printed, TupleEID(otuple))) continue;
			/* HTML */
			if(size>0) doc_puts(doc_html, "<br /><span class=affiliation2>");
			erd_organization_print_upper(ep_html, TupleEID(otuple));
			if(size>0) doc_puts(doc_html, "</span>");
			/* TeX */
			doc_printf(doc_tex, size==0 ? "\\所属P{%d}{":"\\所属S{%d}{", TupleEID(otuple));
			erd_organization_print_upper(ep_tex, TupleEID(otuple));
			doc_puts(doc_tex, "}");
			edb_catalogue_append(ca_aff_printed, TupleEID(otuple), otuple);
		}
	}
	doc_puts(doc_tex, "}%\n");
	edb_catalogue_free(ca_aff_printed);

	/* HTML */
	doc_puts(doc_html, "</td></tr>\n");

	doc_puts(doc_html, "<tr valign=\"top\">\n");

	doc_puts(doc_html, "<td bgcolor=\"" PERSON_NAME_BGC "\">");
	doc_puts(doc_html, "<table border=\"0\" width=\"100%\">\n");
	doc_puts(doc_html, "<tr class=titlebar><td>");
	edb_print_tuple_named_column(ep_html, person, NULL, "", "", "@.title", NULL, ", ", " : ", "", EP_ALERT);
	doc_text_puts(doc_html, BILINGUAL(japanese, pi->name_ja, pi->name_en));
	doc_puts(doc_html, "</td><td width=\"1\" class=button1>\n");
	doc_printf(doc_html, "<input type=\"button\" value=\"%s\" onclick=\"top.location='%s';\">", japanese?"English":"Japanese", japanese?PERSON_PROFILE_HTML_en:PERSON_PROFILE_HTML_ja);
	doc_puts(doc_html, "</td><td width=\"1\">\n");
	JavaScript_printf(doc_html, "edb_ref_by_eid(%d,'button1');", TupleEID(person));
	doc_puts(doc_html, "</td></tr></table></td>\n");
	doc_printf(doc_html, "<td rowspan=\"3\" width=\"%d\"><img src=\"%s\" alt=\"Portrait\" width=\"%d\" height=\"%d\" /></td></tr>", PORTRAIT_WIDTH, portrait_name, PORTRAIT_WIDTH, PORTRAIT_HEIGHT);
	doc_puts(doc_html, "<tr><td><hr /></td></tr>\n");

	doc_puts(doc_html, "<tr valign=\"top\"><td><table border=\"0\" cellpadding=\"3\" cellspacing=\"3\" width=\"100%\" />\n");
	erd_person_print_name(ep_html, pi);
	erd_print_person_column(ep_html, person, BILINGUAL(japanese, "職名", "Title"), "", "@.title", NULL, ", ", "", EP_ALERT, &pi->title_is_set);
	/*
	Now, ERD pages has same permission to EDB publicity policy.
	But, "Year of Birth" and "Phone" are closed inside of  university permission.
	So, read them from the database for university and print.
	*/
#if 0
	erd_print_person_column(ep_html, person, BILINGUAL(japanese, "生年", "Year of Birth"), "", "@.birthday", NULL, ", ", "", EP_WARNING, &pi->birthdate_is_set);
	erd_print_person_column(ep_html, person, BILINGUAL(japanese, "電話", "Phone"), "", "@.phone", NULL, ", ", "", EP_WARNING, &pi->phone_is_set);
#else
	{
		EdbTupleInfo uperson = edb_tuple_get(profile_page->ec_univ, TupleEID(person), 0);
		if(uperson==NULL) uperson = person;
		ep_html->ec = profile_page->ec_univ;
		erd_print_person_column(ep_html, uperson, BILINGUAL(japanese, "生年", "Year of Birth"), "", "@.birthday", NULL, ", ", "", EP_WARNING, &pi->birthdate_is_set);
		erd_print_person_column(ep_html, uperson, BILINGUAL(japanese, "電話", "Phone"), "", "@.phone", NULL, ", ", "", EP_WARNING, &pi->phone_is_set);
		ep_html->ec = profile_page->ec;
	}
#endif
	erd_print_person_column(ep_html, person, BILINGUAL(japanese, "電子メール", "E-mail"), "", "@.email", NULL, ", ", "", EP_WARNING, &pi->email_is_set);

	erd_catalogue_print_in_table(ep_html,
		BILINGUAL3(japanese, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\"><td>", "学歴", "Academic<br />Background", "</td><td>\n"),
		pi->ca_history_a, print_history, M_WIDE|M_SMALL|M_NARROW, "</td></tr>\n");

	edb_print_push(ep_html);
	EpUserData(ep_html) = pi;
	erd_print_person_column(ep_html, person, BILINGUAL(japanese, "学位", "Degree"), "", "@.officialnameofdegree", print_nod, "<br />", "", EP_ALERT, &pi->nod_is_set);
	edb_print_pop(ep_html);

	erd_catalogue_print_in_table(ep_html,
		BILINGUAL3(japanese, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\"><td>", "職歴", "Work History", "</td><td>\n"),
		pi->ca_history_w, print_history, M_WIDE|M_SMALL|M_NARROW, "</td></tr>\n");

	doc_puts(doc_html, "</table>\n");
	doc_puts(doc_html, "</td></tr>\n");

	/* TeX */
	doc_printf(doc_tex, "\\DEF{EID}{%d}%%\n", TupleEID(person));
	edb_print_tuple_named_column(ep_tex, person, NULL, "\\DEF{肩書}{", "", "@.title", NULL, ", ", "}%\n", "", EP_ALERT);
	doc_text_puts3(doc_tex, "\\DEF{名前}{", BILINGUAL(japanese, pi->name_ja, pi->name_en), "}%\n");

	erd_person_print_name(ep_tex, pi);
	erd_print_person_column(ep_tex, person, "職名", "", "@.title", NULL, ", ", "", EP_ALERT, &pi->title_is_set);
	/*
	Now, ERD pages has same permission to EDB publicity policy.
	But, "Year of Birth" and "Phone" are closed inside of  university permission.
	So, read them from the database for university and print.
	*/
#if 0
	erd_print_person_column(ep_tex, person, "生年", "", "@.birthday", NULL, ", ", "", EP_WARNING, &pi->birthdate_is_set);
	erd_print_person_column(ep_tex, person, "電話", "", "@.phone", NULL, ", ", "", EP_WARNING, &pi->phone_is_set);
#else
	{
		EdbTupleInfo uperson = edb_tuple_get(profile_page->ec_univ, TupleEID(person), 0);
		if(uperson==NULL) uperson = person;
		ep_tex->ec = profile_page->ec_univ;
		erd_print_person_column(ep_tex, uperson, "生年", "", "@.birthday", NULL, ", ", "", EP_WARNING, &pi->birthdate_is_set);
		erd_print_person_column(ep_tex, uperson, "電話", "", "@.phone", NULL, ", ", "", EP_WARNING, &pi->phone_is_set);
		ep_tex->ec = profile_page->ec;
	}
#endif
	erd_print_person_column(ep_tex, person, "電子メール", "", "@.email", NULL, ", ", "", EP_WARNING, &pi->email_is_set);

	erd_catalogue_print_in_table(ep_tex, "\\DEF{学歴}{%%\n", pi->ca_history_a, print_history, M_WIDE|M_SMALL|M_NARROW, "}%%\n");

	edb_print_push(ep_tex);
	EpUserData(ep_tex) = pi;
	erd_print_person_column(ep_tex, person, "学位", "", "@.officialnameofdegree", print_nod, "\n", "", EP_ALERT, &pi->nod_is_set);
	edb_print_pop(ep_tex);

	erd_catalogue_print_in_table(ep_tex, "\\DEF{職歴}{%%\n", pi->ca_history_w, print_history, M_WIDE|M_SMALL|M_NARROW, "}%%\n");

	/* HTML */
	doc_puts(doc_html, "<tr><td colspan=\"2\">\n");
	erd_person_page_work(profile_page, work_page, pi, person, japanese);

	doc_printf(doc_html, "<iframe src=\"%s\" name=\"" PERSON_WORK_FRAME "\" width=\"100%%\" height=\"%d\" frameborder=\"1\" marginheight=\"1\">",
		BILINGUAL(japanese, PERSON_WORK_HTML_ja, PERSON_WORK_HTML_en),
		PERSON_WORK_FRAME_HEIGHT);
	doc_puts(doc_html, "<span style=\"color : red\">");
	if(japanese) {
		doc_puts(doc_html, "インラインフレーム対応のブラウザで御覧ください．\n");
	} else {
		doc_puts(doc_html, "Please use the browser which can operate inline frames.\n");
	}
	doc_puts(doc_html, "</span>");
	doc_puts(doc_html, "</iframe>\n");

	doc_puts(doc_html, "</td></tr></table>\n");

	/* TeX */
	doc_puts(doc_tex, "\\end{PROFILE}\n");

	erd_page_end(profile_page, japanese);
	return TRUE;
}

ErdPage
erd_person_page(EdbContext ec, EdbContext ec_univ, eid_t eid)
{
	EdbTupleInfo	person;
	PersonInfo	pi;
	ErdPage		profile_page, work_page;

	profile_page = (ErdPage)edb_catalogue_lookup(ca_erd_person_page, eid);
	if(profile_page) return profile_page;

	person = edb_tuple_get(ec, eid, 0);
	if(person==NULL) return NULL;

	if(!TupleIsPerson(person)) return NULL;

	profile_page = erd_page_new(ec, ec_univ, &person_profile_pages_template, person);
	work_page = erd_page_new(ec, ec_univ, &person_work_pages_template, person);

	pi = erd_get_person_info(profile_page, ec, person);
	edb_catalogue_append(ca_erd_person_page, eid, profile_page);

	profile_page->title_ja = strdup(pi->name_ja);
	profile_page->title_en = strdup(pi->name_en);
	work_page->title_ja = strdup(pi->name_ja);
	work_page->title_en = strdup(pi->name_en);

	pi->ca_person = tuple_expand(ec, person);

	erd_page_open(profile_page);
	erd_page_open(work_page);
	erd_person_page_body(ec, profile_page, work_page, TRUE);
	erd_person_page_body(ec, profile_page, work_page, FALSE);
	erd_page_close(profile_page);
	erd_page_close(work_page);
	FREE(work_page->title_ja);
	FREE(work_page->title_en);
	erd_page_free(work_page);

	return profile_page;
}

int
erd_person_page_for_all(ErdPage page, ErdPage person_page, int japanese)
{
	EdbPrint		ep = PAGE_EP_HTML(page, japanese);
	EdbContext	ec = ep->ec;
	Doc			doc = ep->doc;
	PersonInfo	pi = person_page->info;
 	Link			l;
	EdbTupleInfo	person = pi->tuple;
	// eid_t			eid = TupleEID(person);
	EdbTupleColumn	tc;
	EdbDatum			d;
	char		*portrait_name = "../../" ICON_BROKEN_PORTRAIT;
	EdbCatalogue	ca_aff_printed;

	edb_print_tuple_postfix_register_func(ep, PrintStyle(ep), erd_print_tuple_postfix);
	edb_print_tcd_register_func(ep, "person.birthday", PrintStyle(ep), erd_print_birthday);
	edb_print_type_register_func(ep, EDB_TYPE_EMAIL, PrintStyle(ep), erd_print_email);
	edb_print_type_register_func(ep, EDB_TYPE_PHONE, PrintStyle(ep), erd_print_phone);
	ForCatalogueLink(l, pi->ca_person) edb_print_eoi_add(ep, TupleEID((EdbTupleInfo)CatalogueObject(l)));

	if(STRisVALID(pi->portrait_name)) portrait_name = pi->portrait_name;

	doc_puts(doc, "<table border=\"0\" width=\"100%\">\n");
	doc_puts(doc, "<tr valign=\"top\"><td colspan=\"3\" class=affiliation>\n");

	ca_aff_printed = edb_catalogue_new();
	tc = edb_tuple_seek_column(ec, person, "@.affiliation");
	d = NULL;
	if(tc) {
		ForTCDatum(d, tc) {
			EdbTupleInfo	otuple;
			int	size = edb_catalogue_size(ca_aff_printed);
			if(!DatumEIDisValid(d)) continue;
			if(!edb_datum_at_present(ec, d)) continue;
			otuple = edb_tuple_get(ec, DatumEID(d), 0);
			if(otuple==NULL) continue;
			if(!erd_page_dir_exist(ec, otuple)) continue;
			if(size>0) doc_puts(doc, "<br /><span class=affiliation2>");
			erd_organization_print_upper(ep, TupleEID(otuple));
			if(size>0) doc_puts(doc, "</span>");
			edb_catalogue_append(ca_aff_printed, TupleEID(otuple), otuple);
		}
	}
	if(pi->ca_affiliate!=NULL && edb_catalogue_size(pi->ca_affiliate)>0) {
		ForCatalogueLink(l, pi->ca_affiliate) {
			EdbTupleInfo	otuple = CatalogueObject(l);
			int	size = edb_catalogue_size(ca_aff_printed);

			if(edb_catalogue_lookup(ca_aff_printed, TupleEID(otuple))) continue;
			if(size>0) doc_puts(doc, "<br /><span class=affiliation2>");
			erd_organization_print_upper(ep, TupleEID(otuple));
			if(size>0) doc_puts(doc, "</span>");
			edb_catalogue_append(ca_aff_printed, TupleEID(otuple), otuple);
		}
	}
	edb_catalogue_free(ca_aff_printed);

	doc_puts(doc, "</td></tr>\n");

	doc_puts(doc, "<tr valign=\"top\">\n");

	doc_puts(doc, "<td bgcolor=\"" PERSON_NAME_BGC "\" colspan=\"2\">");
	doc_puts(doc, "<table border=\"0\" width=\"100%\">\n");
	doc_puts(doc, "<tr class=titlebar><td>");
	edb_print_tuple_named_column(ep, person, NULL, "", "", "@.title", NULL, ", ", " : ", "", EP_ALERT);
	doc_text_puts(doc, BILINGUAL(japanese, pi->name_ja, pi->name_en));
	doc_puts(doc, "</td></tr></table></td>\n");
	doc_printf(doc, "<td rowspan=\"7\" width=\"%d\"><img src=\"person/%d/%s\" alt=\"Portrait\" width=\"%d\" height=\"%d\" /></td></tr>",
		PORTRAIT_WIDTH/2, TupleEID(person), portrait_name, PORTRAIT_WIDTH/2, PORTRAIT_HEIGHT/2);
	doc_puts(doc, "<tr><td colspan=\"2\"><hr /></td></tr>\n");

	erd_person_print_name(ep, pi);
	erd_print_person_column(ep, person, BILINGUAL(japanese, "職名", "Title"), "", "@.title", NULL, ", ", "", EP_ALERT, &pi->title_is_set);
	/*
	Now, ERD pages has same permission to EDB publicity policy.
	But, "Year of Birth" and "Phone" are closed inside of  university permission.
	So, read them from the database for university and print.
	*/
	{
		EdbTupleInfo uperson = edb_tuple_get(person_page->ec_univ, TupleEID(person), 0);
		if(uperson==NULL) uperson = person;
		ep->ec = person_page->ec_univ;
		erd_print_person_column(ep, uperson, BILINGUAL(japanese, "生年", "Year of Birth"), "", "@.birthday", NULL, ", ", "", EP_WARNING, &pi->birthdate_is_set);
		erd_print_person_column(ep, uperson, BILINGUAL(japanese, "電話", "Phone"), "", "@.phone", NULL, ", ", "", EP_WARNING, &pi->phone_is_set);
		ep->ec = page->ec;
	}
	erd_print_person_column(ep, person, BILINGUAL(japanese, "電子メール", "E-mail"), "", "@.email", NULL, ", ", "", EP_WARNING, &pi->email_is_set);

	erd_catalogue_print_in_table(ep,
		BILINGUAL3(japanese, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\"><td>", "学歴", "Academic<br />Background", "</td><td colspan=\"2\">\n"),
		pi->ca_history_a, print_history, M_WIDE|M_SMALL|M_NARROW|M_NOEDBLINK, "</td></tr>\n");

	edb_print_push(ep);
	EpUserData(ep) = pi;
	erd_print_person_column2(ep, person, BILINGUAL(japanese, "学位", "Degree"), "", "@.officialnameofdegree", print_nod, "<br />", "", EP_ALERT, &pi->nod_is_set);
	edb_print_pop(ep);

	erd_catalogue_print_in_table(ep,
		BILINGUAL3(japanese, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\"><td>", "職歴", "Work History", "</td><td colspan=\"2\">\n"),
		pi->ca_history_w, print_history, M_WIDE|M_SMALL|M_NARROW|M_NOEDBLINK, "</td></tr>\n");

	erd_catalogue_print_in_table(ep,
		BILINGUAL3(japanese, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\"><td>", "専門分野", "Field of Study", "</td><td colspan=\"2\">\n"),
		pi->ca_study, print_study1, M_WIDE|M_SMALL|M_NOEDBLINK, "</td></tr>\n");

	erd_catalogue_print_in_table(ep,
		BILINGUAL3(japanese, "<tr align=\"left\" valign=\"top\" bgcolor=\"" PERSON_COLUMN_BGC "\"><td>", "研究概要", "Subject of Study", "</td><td colspan=\"2\">\n"),
		pi->ca_study, print_study2, M_WIDE|M_SMALL|M_NOEDBLINK, "</td></tr>\n");

	doc_puts(doc, "</table>\n");

	doc_puts(doc, "\n<p style=\"page-break-before:always\">\n\n\n");

	return TRUE;
}


